package cn.wise.sc.pay.common.security2.service;

import cn.wise.sc.pay.common.security2.SysUser;
import cn.wise.sc.pay.common.security2.userDetail.JwtAppUserDetailsService;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @description: app端登录  使用openID
 * @author: qh
 * @create: 2020-04-28 11:20
 **/
@Component
@ConditionalOnClass(value = JwtAppUserDetailsService.class)
public class AppUserJdbcService {

    private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();

    final
    JdbcTemplate jdbcTemplate;

    public static final String DEF_USERS_BY_USERNAME_QUERY = "select wx_openid,ali_openid,password,lock_flag,del_flag from app_sys_user where (wx_openid = ? or ali_openid = ?) and del_flag = 0";

    public AppUserJdbcService(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    /**
     * 根据openID获取用户信息
     *
     * @param openID 用户信息
     * @return
     */
    public List<SysUser> getSysUserByOpenId(String openID) {

        return jdbcTemplate.query(DEF_USERS_BY_USERNAME_QUERY, new String[]{openID, openID}, (rs, rowNum) -> {
            String username = rs.getString(1);
            String aliOpenId = rs.getString(2);
            if (openID.equals(aliOpenId)) {
                username = aliOpenId;
            }
            Integer lock = rs.getInt(4);
            Integer del = rs.getInt(5);
            return SysUser.builder()
                    .username(username)
                    .password(ENCODER.encode("APP"))
                    .enabled(del == 0)
                    .locked(lock == 0)
                    .build();
        });
    }
}
