package cn.wise.sc.pay.common.security2.service;

import cn.wise.sc.pay.common.security2.userDetail.JwtAppUserDetailsService;
import cn.wise.sc.pay.common.security2.userDetail.JwtDeviceDetailsService;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @description: 收银系统设备号服务
 * @author: qh
 * @create: 2020-04-21 15:37
 **/
@Component
@ConditionalOnClass(value = JwtDeviceDetailsService.class)
public class DeviceJdbcService {

    final
    JdbcTemplate jdbcTemplate;
    public static final String DEF_USERS_BY_USERNAME_QUERY = "select device_code,unique_flag from device where device_code = ?";

    public DeviceJdbcService(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    /**
     * 根据设备号获取设备唯一标识构建验权对象
     *
     * @param deviceCode
     * @return
     */
    public List<UserDetails> getDeviceCode(String deviceCode) {

        return jdbcTemplate.query(DEF_USERS_BY_USERNAME_QUERY, new String[]{deviceCode}, (rs, rowNum) -> {
            String username1 = rs.getString(1);
            String password = rs.getString(2);
            UserDetails userDetaill = new User(username1, password, new ArrayList<GrantedAuthority>());
            return userDetaill;
        });
    }
}
