package cn.wise.sc.pay.common.security2.service;

import cn.wise.sc.pay.common.security2.SysUser;
import cn.wise.sc.pay.common.security2.userDetail.JwtUserDetailsService;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @ClassNameSysUserJdbcService
 * @Description 自定义的用户查询JDBC  处理多租户问题
 * @Author qh
 * @Date2020/3/5 15:53
 * @Version V1.0
 **/
@Component
@ConditionalOnClass(value = JwtUserDetailsService.class)
public class SysUserJdbcService {

    final
    JdbcTemplate jdbcTemplate;

    public static final String DEF_USERS_BY_USERNAME_QUERY = "select username,password,lock_flag,del_flag from {table} where username = ? and del_flag=0";

    public SysUserJdbcService(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    /**
     * 根据用户名获取用户账号密码
     * 由于是多租户  tableName需要具体带上
     *
     * @param username
     * @return
     */
    public List<SysUser> getSysUserByUsername(String tableName, String username) {

        String sql = DEF_USERS_BY_USERNAME_QUERY.replace("{table}", tableName);
        return jdbcTemplate.query(sql, new String[]{username}, (rs, rowNum) -> {
            String username1 = rs.getString(1);
            String password = rs.getString(2);
            Integer lock = rs.getInt(3);
            Integer del = rs.getInt(4);
            return SysUser.builder()
                    .username(username1)
                    .password(password)
                    .enabled(del == 0)
                    .locked(lock == 0)
                    .build();
        });
    }
}

