package cn.wise.sc.pay.business.goods.service;

import cn.wise.sc.pay.business.goods.model.GoodsResponse;
import cn.wise.sc.pay.business.goods.model.bean.Goods;
import cn.wise.sc.pay.business.goods.model.query.GoodsQuery;
import cn.wise.sc.pay.business.goods.model.query.GoodsScanQuery;
import cn.wise.sc.pay.business.goods.model.vo.GoodsExcel;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;

/**
 * <p>
 * 商品 服务类
 * </p>
 *
 * @author qh
 * @since 2019-12-03
 */
public interface IGoodsService extends IService<Goods> {

    /**
     * 添加一个新的商品
     *
     * @param goodsQuery
     * @return
     */
    GoodsResponse newGoods(GoodsQuery goodsQuery);

    /**
     * 编辑商品
     *
     * @return
     */
    GoodsResponse editGoods(GoodsQuery goodsQuery);

    /**
     * 根据id,删除商品
     *
     * @param id
     * @return
     */
    GoodsResponse delGoods(long id, long sellerId);

    /**
     * 根据id,下架商品
     *
     * @param id
     * @return
     */
    GoodsResponse unshelveGoods(long id, long sellerId);

    /**
     * 根据id,上架商品
     *
     * @param id
     * @return
     */
    GoodsResponse shelveGoods(long id, long sellerId);

    /**
     * 按条件查询列表
     *
     * @param sellerId 商户id
     * @param key 条件key
     * @param type 类型
     * @param status 状态
     * @return
     */
    BaseResponse list(long sellerId, String key, String type, int status);


    /**
     * 获取商品
     *
     * @param goodsId
     * @return
     */
    GoodsResponse getGoods(int goodsId);


    /**
     * 获取所有商品
     *
     * @return
     */
    BaseResponse getList(String key, String shopName, String goodsType);


    /**
     * 根据商品id审核商品
     *
     * @param goodsId 商品id
     * @param cause   原因
     * @param status  审核状态
     * @return
     */
    GoodsResponse auditGoods(long goodsId, String cause, int status);

    /**
     * 获取待审核的商品集合
     *
     * @return
     */
    BaseResponse getWaitingAuditList(String key, String shopName, String goodsType);

    /**
     * 分页查询
     *
     * @param pageQuery 分页条件
     * @param key       关键字
     * @param shopName  店铺名
     * @param goodsType 商品类型
     * @return
     */
    BaseResponse getPage(PageQuery pageQuery, String key, long shopName, String goodsType);

    /**
     * 获取待审核商品集合 分页
     *
     * @param pageQuery 分页条件
     * @param key       关键字
     * @param shopId    店铺名
     * @param goodsType 商品类型
     * @return
     */
    BaseResponse getWaitingAuditPage(PageQuery pageQuery, String key, Long shopId, String goodsType);

    /**
     * 下架商品  带原因
     *
     * @param goodsId  商品id
     * @param sellerId 商户id
     * @param cause    下架原因
     * @return
     */
    GoodsResponse unshelveGoods(int goodsId, int sellerId, String cause);

    /**
     * 收银端商品excel模板
     *
     * @return
     */
    List<GoodsExcel> getOneGoodsExcel(String typeName);

    /**
     * 导入商品
     *
     * @param datas
     */
    void saveGoods(List<GoodsExcel> datas);

    /**
     * 扫码录入商品
     *
     * @param goodsScanQuery {@link cn.wise.sc.pay.business.goods.model.query.GoodsScanQuery}
     * @return
     */
    BaseResponse<Goods> scanAddGoods(GoodsScanQuery goodsScanQuery);

    /**
     * 扫码点单
     * @param code 二维码
     * @return Goods
     * @see Goods
     */
    BaseResponse<Goods> getGoodsInfoByCode(String code);

    /**
     * 批量添加
     * @param goods  商品集合
     * @return Boolean
     */
    BaseResponse<Boolean> batchAdd(List<Goods> goods);

    /**
     * 根据ids获取商品集合
     * @param ids id集合
     * @return list
     */
    List<Goods> getByGoodIds(List<Long> ids);
}
