package cn.wise.sc.pay.business.goods.service.impl;

import cn.wise.sc.pay.business.goods.model.GoodsTypeResponse;
import cn.wise.sc.pay.business.goods.model.bean.GoodsType;
import cn.wise.sc.pay.business.goods.mapper.GoodsTypeMapper;
import cn.wise.sc.pay.business.goods.model.vo.GoodsTypeVO;
import cn.wise.sc.pay.business.goods.model.query.GoodsTypeQuery;
import cn.wise.sc.pay.business.goods.service.IGoodsTypeService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 商品类型 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-12-03
 */
@Service
@Slf4j
public class GoodsTypeServiceImpl extends ServiceImpl<GoodsTypeMapper, GoodsType> implements IGoodsTypeService {

    @Autowired
    GoodsServiceImpl goodsService;

    @Override
    public GoodsTypeResponse newGoodsType(GoodsTypeQuery typeQuery) {

        GoodsType newType = GoodsType.builder()
                .name(typeQuery.getName())
                .parentId(typeQuery.getParentId())
                .sellerId(typeQuery.getSellerId())
                .status((short) 0)
                .build();

        QueryWrapper qw = new QueryWrapper();
        qw.eq("name", typeQuery.getName());
        qw.eq("seller_id", typeQuery.getSellerId());
        qw.eq("parent_id", typeQuery.getParentId());
        qw.eq("status", 1);
        int count = this.count(qw);
        if (count > 0) {
            return GoodsTypeResponse.error("该类型已存在!");
        }

        boolean isSalve = this.save(newType);
        GoodsTypeVO goodsTypeVO = new GoodsTypeVO(newType);

        return isSalve ? GoodsTypeResponse.ok("添加成功", goodsTypeVO) : GoodsTypeResponse.error("添加失败");
    }

    @Override
    public GoodsTypeResponse editGoodsType(GoodsTypeQuery typeQuery) {

        if (!checkIdAndSeller(typeQuery.getId(), typeQuery.getSellerId())) {
            return GoodsTypeResponse.error("参数错误");
        }

        GoodsType updateType = GoodsType.builder()
                .id(typeQuery.getId())
                .parentId(typeQuery.getParentId())
                .name(typeQuery.getName())
                .sellerId(typeQuery.getSellerId())
                .status(typeQuery.getStatus())
                .build();

        boolean isUpdate = this.updateById(updateType);

        GoodsTypeVO goodsTypeVO = new GoodsTypeVO(updateType);
        return isUpdate ? GoodsTypeResponse.ok("更新成功", goodsTypeVO) : GoodsTypeResponse.error("更新失败");
    }

    @Override
    public GoodsTypeResponse delGoodsType(long id, long sellerId) {

        if (!checkIdAndSeller(id, sellerId)) {
            return GoodsTypeResponse.error("参数错误");
        }

        QueryWrapper qw = new QueryWrapper();
        qw.eq("id", id);
        qw.eq("seller_id", sellerId);

        boolean IsRemove = this.remove(qw);

        return IsRemove ? GoodsTypeResponse.ok("删除成功", new GoodsTypeVO()) : GoodsTypeResponse.error("删除失败");

    }

    @Override
    public BaseResponse getList(long sellerId, short status) {

        if (sellerId == 0) {
            return GoodsTypeResponse.error("参数错误");
        }

        QueryWrapper<GoodsType> qw = new QueryWrapper<>();
        qw.eq("seller_id", sellerId).or().eq("seller_id", -1);
        if (status != -1) {
            qw.eq("status", status);
        }
        try {
            List<GoodsType> goodsTypes = this.list(qw);

            List<GoodsTypeVO> goodsTypeVOS = new ArrayList<>();

            for (GoodsType exhibitionType : goodsTypes) {

                if (exhibitionType.getParentId() == 0) {
                    GoodsTypeVO goodsTypeVO = GoodsTypeVO.builder()
                            .id(exhibitionType.getId())
                            .name(exhibitionType.getName())
                            .parentId(exhibitionType.getParentId())
                            .exhibitionTypeVOS(new ArrayList<>())
                            .build();
                    treeExhibitionType(goodsTypes, goodsTypeVO);

                    goodsTypeVOS.add(goodsTypeVO);
                }
            }
            BaseResponse baseResponse = GoodsTypeResponse.ok("查商品类型成功");
            baseResponse.setData(goodsTypeVOS);
            return baseResponse;
        } catch (Exception ex) {

            return GoodsTypeResponse.error("查询商品类型失败");
        }
    }

    @Override
    public BaseResponse delTypeAndUnShelveGoods(int id, int sellerId) {

        if (!checkIdAndSeller(id, sellerId)) {
            return GoodsTypeResponse.error("参数错误");
        }

        QueryWrapper qw = new QueryWrapper();
        qw.eq("id", id);
        qw.eq("seller_id", sellerId);

        GoodsType goodsType = this.getOne(qw);
        if (goodsType == null) {
            return GoodsTypeResponse.ok("删除成功");
        }
        try {
            this.remove(qw);
            UpdateWrapper goodsQw = new UpdateWrapper();
            try {
                goodsQw.eq("type", goodsType.getName());
                goodsQw.setSql("status = 0");
                if (!goodsService.update(goodsQw)) {
                    return BaseResponse.error("无权删除!", this.getClass().getName());
                }
            } catch (Exception ex) {
                log.error("商品下架失败  ===== " +
                        " cn.wise.sc.pay.business.goods.service.impl.GoodsTypeServiceImpl.delTypeAndUnShelveGoods  =====" +
                        " 异常:{}", ex.getMessage());
            }
            return GoodsTypeResponse.ok("删除成功");
        } catch (Exception ex) {
            log.error("商品类型删除失败  ===== " +
                    " cn.wise.sc.pay.business.goods.service.impl.GoodsTypeServiceImpl.delTypeAndUnShelveGoods  =====" +
                    " 异常:{}", ex.getMessage());
            return GoodsTypeResponse.error("删除失败");
        }
    }

    @Override
    public BaseResponse getAppList(int sellerId, short status) {
        if (sellerId == 0) {
            return GoodsTypeResponse.error("参数错误");
        }
        QueryWrapper<GoodsType> qw = new QueryWrapper<>();
        qw.eq("seller_id", sellerId);
        if (status != -1) {
            qw.eq("status", status);
        }
        try {
            List<GoodsType> goodsTypes = this.list(qw);
            List<GoodsTypeVO> goodsTypeVOS = new ArrayList<>();
            for (GoodsType exhibitionType : goodsTypes) {
                if (exhibitionType.getParentId() == 0) {
                    GoodsTypeVO goodsTypeVO = GoodsTypeVO.builder()
                            .id(exhibitionType.getId())
                            .name(exhibitionType.getName())
                            .parentId(exhibitionType.getParentId())
                            .exhibitionTypeVOS(new ArrayList<>())
                            .build();
                    treeExhibitionType(goodsTypes, goodsTypeVO);
                    goodsTypeVOS.add(goodsTypeVO);
                }
            }
            BaseResponse baseResponse = GoodsTypeResponse.ok("查商品类型成功");
            baseResponse.setData(goodsTypeVOS);
            return baseResponse;
        } catch (Exception ex) {
            return GoodsTypeResponse.error("查询商品类型失败");
        }
    }

    /**
     * 递归获取商品类型树
     *
     * @param exhibitionTypes
     * @param parentTypeVo
     */
    private void treeExhibitionType(List<GoodsType> exhibitionTypes, GoodsTypeVO parentTypeVo) {

        //过滤子类型
        List<GoodsType> childTypes = exhibitionTypes
                .stream()
                .filter(arg -> arg.getParentId().longValue() == parentTypeVo.getId())
                .collect(Collectors.toList());
        if (childTypes.size() == 0) {
            return;
        }

        for (GoodsType childType : childTypes) {
            GoodsTypeVO childTypeVo = new GoodsTypeVO();
            childTypeVo.setName(childType.getName());
            childTypeVo.setId(childType.getId());
            childTypeVo.setStatus(childType.getStatus());
            childTypeVo.setParentId(childType.getParentId());
            parentTypeVo.exhibitionTypeVOS.add(childTypeVo);
            treeExhibitionType(exhibitionTypes, childTypeVo);
        }
    }

    /**
     * 判断商品类型id和商户id是否有效
     *
     * @param id
     * @param sellerId
     * @return
     */
    private boolean checkIdAndSeller(long id, long sellerId) {
        return id != 0 && sellerId != 0;
    }
}
