
package cn.wise.sc.pay.laas.userbac.business.controller;

import cn.wise.sc.pay.laas.userbac.business.service.DeptService;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysDept;
import com.baomidou.mybatisplus.extension.api.R;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.Date;

/**
 * <p>
 * 部门管理 前端控制器
 * </p>

 */
@AllArgsConstructor
//@RestController
//@RequestMapping("/dept")
public class DeptController {

	private final DeptService deptService;

	/**
	 * 通过ID查询
	 *
	 * @param id ID
	 * @return SysDept
	 */
	@ApiOperation("部门信息查询")
	@ApiImplicitParam(name = "id",value = "部门id",dataType = "Integer",paramType = "path")
	@GetMapping("/{id}")
	public R getById(@PathVariable Integer id) {
		return R.ok(deptService.getById(id));
	}


	/**
	 * 返回树形菜单集合
	 *
	 * @return 树形菜单
	 */
	@GetMapping(value = "/tree")
	public R listDeptTrees() {
		return R.ok(deptService.listDeptTrees());
	}

	/**
	 * 返回当前用户树形菜单集合
	 *
	 * @return 树形菜单
	 */
	@GetMapping(value = "/user-tree")
	public R listCurrentUserDeptTrees() {
		return R.ok(deptService.listCurrentUserDeptTrees());
	}

	/**
	 * 添加
	 *
	 * @param sysDept 实体
	 * @return success/false
	 */
	//@SysLog("添加部门")
	@PostMapping
	//@PreAuthorize("@pms.hasPermission('sys_dept_add')")
	public R save(@Valid @RequestBody SysDept sysDept) {
		return R.ok(deptService.saveDept(sysDept));
	}

	/**
	 * 删除
	 *
	 * @param id ID
	 * @return success/false
	 */
	//@SysLog("删除部门")
	@DeleteMapping("/{id}")
	//@PreAuthorize("@pms.hasPermission('sys_dept_del')")
	public R removeById(@PathVariable Integer id) {
		return R.ok(deptService.removeDeptById(id));
	}

	/**
	 * 编辑
	 *
	 * @param sysDept 实体
	 * @return success/false
	 */
	//@SysLog("编辑部门")
	@PutMapping
	//@PreAuthorize("@pms.hasPermission('sys_dept_edit')")
	public R update(@Valid @RequestBody SysDept sysDept) {
		sysDept.setUpdateTime(new Date(System.currentTimeMillis()));
		return R.ok(deptService.updateDeptById(sysDept));
	}
}
