package cn.wise.sc.pay.pass.business.app.controller.association;

import cn.wise.sc.pay.api.association.service.IArticleService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.association.bean.Article;
import cn.wise.sc.pay.domain.association.query.ArticleQuery;
import cn.wise.sc.pay.domain.association.vo.ArticleVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@RestController
@Api(description = "图文/文章")
@RequestMapping("/association/article/")
@CrossOrigin
public class ArticleController {

    final
    IArticleService iArticleService;
    final
    RedissonClient redissonClient;

    /**
     * redis中文章审核开关配置
     */
    String articleAuditSettingKey = "articleAuditSettingKey";

    public ArticleController(IArticleService iArticleService, RedissonClient redissonClient) {
        this.iArticleService = iArticleService;
        this.redissonClient = redissonClient;
    }

    @GetMapping("getArticleVerify")
    public BaseResponse<Page<Article>> getArticleVerify(Integer type, String order, String name, String title,
                                                        @RequestParam("pageNo") Integer pageNo, @RequestParam("pageSize") Integer pageSize) {

        return iArticleService.getArticleVerify(type, order, name, title, pageNo, pageSize);
    }

    @PostMapping("addArticle")
    @ApiOperation("发表图文分享")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "article", value = "图文/文章", paramType = "body", dataType = "Article"),
            @ApiImplicitParam(name = "files", value = "图文/文章相关资源", paramType = "body", dataType = "MultipartFile")
    })
    public BaseResponse addArticle(@RequestBody ArticleQuery articleQuery) {
        List<String> aocSources = articleQuery.getAocSources();
        RBucket<String> bucket = redissonClient.getBucket(articleAuditSettingKey);

        if (bucket.isExists() && "close".equals(bucket.get())) {
            articleQuery.setStatus((byte) 1);
        }
        return iArticleService.addArticle(articleQuery);
    }


    @PostMapping("uploadPic")
    @ApiOperation("上传图片或视频")
    @ApiImplicitParam(name = "file", value = "图片/视频", paramType = "body", dataType = "MultipartFile")
    public BaseResponse<Map<String, String>> uploadPic(MultipartFile file) {
        return iArticleService.uploadPic(file);
    }

    @PutMapping("updateStatus")
    @ApiOperation("修改文章/图文审核状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "文章id,多个用逗号分割", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "status", value = "文章状态", paramType = "path", dataType = "String")
    })
    public BaseResponse updateStatus(String ids, byte status) {
        return iArticleService.updateStatus(ids, status);
    }

    @DeleteMapping("deleteArticles")
    @ApiOperation("批量删除图文 或 批量推荐图文")
    public BaseResponse deleteArticles(String ids, Integer status) {
        return iArticleService.deleteArticles(ids, status);
    }


    @PutMapping("viewNumUp")
    @ApiOperation("图文/文章 浏览量+1")
    @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long")
    public BaseResponse updateViewNum(Long id) {
        return iArticleService.updateViewNum(id);
    }

    @PutMapping("praiseNumUp")
    @ApiOperation("点赞图文/文章")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "articleId", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse updatePraiseNum(Long articleId, Long userId) {
        return iArticleService.updatePraiseNum(articleId, userId);
    }

    @GetMapping("judgePraise")
    @ApiOperation("判断用户对图文/文章已点赞")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "articleId", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse judgePraise(Long articleId, Long userId) {
        return iArticleService.judgePraise(articleId, userId);
    }

    @PutMapping("deletePraise")
    @ApiOperation("取消点赞")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse deletePraise(Long id, Long userId) {
        return iArticleService.deletePraise(id, userId);
    }


    @PostMapping("collection")
    @ApiOperation("收藏")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "articleId", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse collection(Long articleId, Long userId) {
        return iArticleService.collection(articleId, userId);
    }

    @GetMapping("judgeCollection")
    @ApiOperation("判断用户对图文/文章已收藏")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "articleId", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse judgeCollection(Long articleId, Long userId) {
        return iArticleService.judgeCollection(articleId, userId);
    }

    @PutMapping("deleteCollection")
    @ApiOperation("取消收藏")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "图文id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long")
    })
    public BaseResponse deleteCollection(Long id, Long userId) {
        return iArticleService.deleteCollection(id, userId);
    }

    @GetMapping("selectById")
    @ApiOperation("根据id获取图文详情")
    @ApiImplicitParam(name = "id", value = "文章id", paramType = "path", dataType = "Long")
    public BaseResponse<ArticleVO> selectArticleById(Long id) {
        return iArticleService.selectArticleById(id);
    }

    @GetMapping("getArticleNotVerify")
    @ApiOperation("获取未审核图文列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "authorName", value = "发表人名称", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "timeBucket", value = "搜索时间段", paramType = "path", dataType = "TimeBucket"),
            @ApiImplicitParam(name = "pageQuery", value = "分页", paramType = "path", dataType = "PageQuery")
    })
    public BaseResponse<Page<Article>> getArticleNotVerify(String authorName,
                                                           TimeBucket timeBucket, PageQuery pageQuery) {
        return iArticleService.getArticleNotVerify(authorName, timeBucket.getStartTime(), timeBucket.getEndTime(), pageQuery.getPageNo(), pageQuery.getPageSize());
    }

    @GetMapping("userCollections")
    @ApiOperation("用户收藏图文/文章")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse<Page<Article>> getCollectionArticle(Long userId, PageQuery pageQuery) {
        return iArticleService.getCollectionArticle(userId, pageQuery);
    }

    @GetMapping("userPublishs")
    @ApiOperation("用发户布的图文/文章")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse<Page<Article>> getUserPublishArticle(Long userId, PageQuery pageQuery) {
        return iArticleService.getUserPublishArticle(userId, pageQuery);
    }

    @GetMapping("userPraises")
    @ApiOperation("用户点赞的图文/文章")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse<Page<Article>> getUserPraiseArticle(Long userId, PageQuery pageQuery) {
        return iArticleService.getUserPraiseArticle(userId, pageQuery);
    }

    @GetMapping("ponArticle")
    @ApiOperation("上一篇或下一篇图文 pon=0代表上一篇，pon=1代表下一篇")
    public BaseResponse previousOrNextArticle(Long articleId, Integer pon) {
        return iArticleService.previousOrNextArticle(articleId, pon);
    }

    @GetMapping("ponVerifyArticle")
    @ApiOperation("上一篇或下一篇已审核图文 pon=0代表上一篇，pon=1代表下一篇")
    public BaseResponse previousOrNextVerifyArticle(Long articleId, Integer pon) {
        return iArticleService.previousOrNextVerifyArticle(articleId, pon);
    }



}

