package cn.wise.sc.pay.pass.business.app.controller.cashier;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.api.cashier.service.IGoodsService;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.business.goods.model.vo.GoodsVO;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.pass.business.app.config.FastDfsUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @description: 在线点单  商铺
 * @author: qh
 * @create: 2020-06-08 17:15
 **/
@RestController
@Api(description = "在线单点商铺")
@RequestMapping("/app/cashier/shop")
@CrossOrigin
@Validated
public class AppShopController {

    @Autowired
    IMerchantService iMerchantService;
    @Autowired
    IGoodsService iGoodsService;
    @Autowired
    FastDfsUtil fastDfsUtil;

    @PostMapping("/list")
    @ApiOperation("获取正在营业和未关闭的店铺")
    public BaseResponse<Merchant> list(PageQuery pageQuery) {
        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        HashMap<String, Object> eq = new HashMap<>(2);
        eq.put("del_status", 0);
        eq.put("manage_status", 0);
        pageWrapperQuery.setEq(eq);
        pageWrapperQuery.setPageQuery(pageQuery);
        return iMerchantService.getStoreInfoPage(pageWrapperQuery);
    }

    @GetMapping("/{id}")
    @ApiOperation("获取商铺信息")
    public BaseResponse getStore(@PathVariable("id") Long id) {
        return iMerchantService.getStoreInfo(id);
    }


    @GetMapping("/goods/{shopId}")
    @ApiOperation("获取商铺下的商品列表")
    public BaseResponse<List<GoodsVO>> getGoodsByShopId(@PathVariable("shopId") Long shopId, Long tableId, String type) {
        BaseResponse<List<HashMap>> goodsBaseResponse = iGoodsService.list(shopId.intValue(), "", type, 1);
        List<HashMap> list = goodsBaseResponse.getData();
        List<GoodsVO> rts = new ArrayList<>();
        for (HashMap map : list) {
            GoodsVO goodsVO = BeanUtil.mapToBean(map, GoodsVO.class, true);
            goodsVO.setCover(fastDfsUtil.fullUrl(goodsVO.getCover()));
            rts.add(goodsVO);
        }
        return BaseResponse.ok(rts);
    }


}
