package cn.wise.sc.pay.pass.business.app.controller.eshop;

import cn.wise.sc.pay.api.eshopping.service.IEshopGoodsService;
import cn.wise.sc.pay.api.points.service.IPointMemberShipService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.query.CalCulate;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.DecimalMin;
import javax.validation.constraints.Min;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@RestController
@Api(description = "商品")
@RequestMapping("/app/eshop/goods")
@CrossOrigin
@Validated
public class EshopGoodsController {

    @Autowired
    IEshopGoodsService eshopGoodsService;

    @Autowired
    IPointMemberShipService memberShipService;

    @GetMapping("goodsPage")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "商品名称", name = "name", paramType = "query", dataType = "String"),
            @ApiImplicitParam(value = "商品状态", name = "marketEnable", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(value = "页码", name = "pageNo", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "每页条数", name = "pageSize", paramType = "query", dataType = "Integer", required = true)
    })
    public BaseResponse goodsPage(@RequestParam(value = "name",required = false) String name,
                                  @RequestParam(value = "marketEnable",required = false) Integer marketEnable,
                                  @RequestParam(value = "pageNo") @DecimalMin("0") Integer pageNo,
                                  @RequestParam(value = "pageSize") Integer pageSize){
        return eshopGoodsService.goodsPage(name, marketEnable, pageNo, pageSize);
    }

    @GetMapping("conditionGoodsPage")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "商品分类", name = "categoryId", paramType = "query", dataType = "int"),
            @ApiImplicitParam(value = "商品价格排序，0升序，1降序", name = "price", paramType = "query", dataType = "int"),
            @ApiImplicitParam(value = "商品发布时间,0顺序，1倒序", name = "createTime", paramType = "query", dataType = "int"),
            @ApiImplicitParam(value = "页码", name = "pageNo", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "每页条数", name = "pageSize", paramType = "query", dataType = "Integer", required = true)
    })
    public BaseResponse conditionGoodsPage(
            @RequestParam(value = "categoryId",required = false) Integer categoryId,
            @RequestParam(value = "price",required = false) Integer price,
            @RequestParam(value = "createTime",required = false) Integer createTime,
            @RequestParam(value = "multiple",required = false) Integer multiple,
            @RequestParam(value = "pageNo") @DecimalMin("0") Integer pageNo,
            @RequestParam(value = "pageSize") Integer pageSize){
        return eshopGoodsService.conditionGoodsPage(categoryId, price, createTime, multiple, pageNo, pageSize);
    }

    @GetMapping("/detail/{goodsId}")
    public BaseResponse goodsDetail(@Min(value = 0) @PathVariable("goodsId") Integer goodsId) {
        return eshopGoodsService.goodsDetail(goodsId);
    }

    @PostMapping("calculate")
    public BaseResponse calculatePrice(@RequestBody CalCulate calCulate){
        return eshopGoodsService.calculatePrice(calCulate);
    }

}
