package cn.wise.sc.pay.pass.business.app.controller.exhibition;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.wise.sc.pay.api.exhibition.service.IExhibitionOrderService;
import cn.wise.sc.pay.api.exhibition.service.IExhibitionsCommentService;
import cn.wise.sc.pay.api.exhibition.service.IExhibitionsReplyService;
import cn.wise.sc.pay.api.exhibition.service.IExhibitionsService;
import cn.wise.sc.pay.api.exhibition.service.IExhibitionsTypeService;
import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.business.comment.bean.Reply;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.exhibition.bean.EntranceQrcode;
import cn.wise.sc.pay.domain.exhibition.query.GoodsQuery;
import cn.wise.sc.pay.domain.exhibition.query.QRCodeGiftQuery;
import cn.wise.sc.pay.domain.exhibition.vo.ExhibitionTypeVO;
import cn.wise.sc.pay.domain.exhibition.vo.RQCodeInfoVo;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.NonNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @description: 展览
 * @author: qh
 * @create: 2019-12-26 15:21
 **/
@Api(description = "app端展览")
@RestController
@CrossOrigin
@RequestMapping("/app/exhibition")
public class ExhibitionController {

    @Autowired
    IExhibitionsService iExhibitionsService;
    @Autowired
    IExhibitionOrderService iExhibitionOrderService;
    @Autowired
    IExhibitionsCommentService iExhibitionsCommentService;
    @Autowired
    IExhibitionsReplyService iExhibitionsReplyService;
    @Autowired
    IExhibitionsTypeService iExhibitionsTypeService;

    @GetMapping("/QRCode")
    @ApiOperation("获取入场二维码")
    @ApiImplicitParam(name = "orderNo", value = "订单号")
    public BaseResponse getQRCode(@NonNull String orderNo) {

        return iExhibitionsService.getQRCode(orderNo);
    }

    @GetMapping("/list")
    @ApiOperation("获取所有(条件)展览,不分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型(字符串)", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "showType", value = "展示类型 0:上新(默认) 1:推荐", paramType = "query", dataType = "string")
    })
    public BaseResponse getExhibitionList(String type, Integer showType, TimeBucket timeBucket) {

        return iExhibitionsService.getExhibitionList(type, showType, timeBucket);
    }

    @GetMapping("/detail/{id}")
    @ApiOperation("获取展览详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "path", dataType = "long")
    })
    public BaseResponse getExhibitionDetail(@PathVariable("id") long id) {

        return iExhibitionsService.getExhibitionDetail(id);
    }

    @GetMapping("/preOderNo")
    @ApiOperation("获取唯一订单号(下单前)")
    public BaseResponse getPreOrderNo() {
        Snowflake snowflake = IdUtil.getSnowflake(8, 8);
        String newIdStr = snowflake.nextIdStr();
        return BaseResponse.ok(newIdStr);
    }

    @PostMapping("/prePay")
    @ApiOperation("预支付订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse prePayOrder(String orderNo) {
        return iExhibitionOrderService.prePayOrder(orderNo);
    }

    @PostMapping("/pay/success")
    @ApiOperation("支付成功回调")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse payOrderSuccess(String orderNo) {
        return iExhibitionOrderService.payOrderSuccess(orderNo);
    }

    @PostMapping("/pay/fail")
    @ApiOperation("支付失败回调")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse payOrderFail(String orderNo) {
        return iExhibitionOrderService.payOrderFail(orderNo);
    }

    @PostMapping("/preOrder")
    @ApiOperation("预下单")
    @ApiImplicitParam(name = "orderQuery", value = "下单", paramType = "body", dataType = "OrderDetailQuery")
    public BaseResponse preOrder(@RequestBody OrderDetailQuery<GoodsQuery> orderQuery) {

        return iExhibitionOrderService.preOrder(orderQuery);
    }

    @PostMapping("/addComment")
    @ApiOperation("评论")
    @ApiImplicitParam(name = "comment", value = "评论实体", paramType = "body", dataType = "Comment")
    public BaseResponse addComment(@RequestBody Comment comment) {
        return iExhibitionsCommentService.addComment(comment);
    }

    @GetMapping("/commentPage")
    @ApiOperation("根据评论主体 评论列表 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "targetId", value = "评论主体id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse getCommentsByTargetId(Long targetId, PageQuery pageQuery) {
        return iExhibitionsCommentService.getCommentsByTargetId(targetId, pageQuery);
    }

    @GetMapping("/advertising/list")
    @ApiOperation("获取所有(条件)展览广告位,不分页")
    public BaseResponse getAdvertising() {
        return iExhibitionsService.getAdvertising();
    }

    @PostMapping("/addReply")
    @ApiOperation("评论回复")
    @ApiImplicitParam(name = "reply", value = "评论回复实体", paramType = "body", dataType = "Reply")
    public BaseResponse inserReply(@RequestBody Reply reply) {
        return iExhibitionsReplyService.inserReply(reply);
    }

    @GetMapping
    @ApiOperation("获取所有展览类型(树形图)")
    public BaseResponse<List<ExhibitionTypeVO>> getExhibitionTypes() {
        return iExhibitionsTypeService.getExhibitionTypes();
    }

    /**
     * 获取用户票据信息
     *
     * @param userId 用户id
     * @return
     */
    @GetMapping("/QRCode/list/{userId}")
    @ApiOperation("获取用户的所有票据信息")
    BaseResponse<List<RQCodeInfoVo>> getAllQRCode(@PathVariable("userId") String userId) {
        return iExhibitionsService.getAllQRCode(userId);
    }

    /**
     * 票务赠与
     *
     * @param qrCodeGiftQuery 票据信息 {@link QRCodeGiftQuery}
     * @return
     */
    @PutMapping("/gift")
    @ApiOperation("票务赠与")
    @ApiImplicitParam(name = "qrCodeGiftQuery", value = "票务赠与请求对象", paramType = "body", dataType = "QRCodeGiftQuery")
    BaseResponse<EntranceQrcode> shareQRCode(@RequestBody QRCodeGiftQuery qrCodeGiftQuery) {
        return iExhibitionsService.shareQRCode(qrCodeGiftQuery);
    }

    @PutMapping("/check/QRCode")
    @ApiOperation("获取入场二维码")
    @ApiImplicitParam(name = "enQCodeJSON", value = "二维码", paramType = "query", dataType = "String")
    public BaseResponse entranceCheck(@RequestBody String enQCodeJSON) {
        return iExhibitionsService.entranceCheck(enQCodeJSON);
    }
}
