package cn.wise.sc.pay.pass.business.app.controller.orders;

import cn.wise.sc.pay.api.eshopping.service.ILogisticsTemplateService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopGoods;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsOrderInfo;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

/**
 * @description: 网上商城订单
 * @author: qh
 * @create: 2020-03-27 10:54
 **/
@Validated
@CrossOrigin
@RestController
@RequestMapping("/eshop/order")
public class EshopOrderController {

    final
    IOrdersService iOrdersService;
    final
    ILogisticsTemplateService logisticsTemplateService;

    public EshopOrderController(IOrdersService iOrdersService, ILogisticsTemplateService logisticsTemplateService) {
        this.iOrdersService = iOrdersService;
        this.logisticsTemplateService = logisticsTemplateService;
    }

    /**
     * 创建订单
     *
     * @param orderDetailQuery 订单描述
     * @return Orders
     */
    @PostMapping
    public BaseResponse<Orders> create(@RequestBody OrderDetailQuery<EshopGoods> orderDetailQuery) {
        orderDetailQuery.setPaas("ESHOP");
        return iOrdersService.createOrder(orderDetailQuery);
    }

    /**
     * 关联物流信息
     *
     * @param logisticsNo 物流号
     * @param orderNo     订单号儿
     * @return Orders
     */
    @PutMapping("/logistics")
    @ApiOperation("订单关联物流")
    public BaseResponse<Orders> addLogistics(@NotBlank String logisticsNo, @NotBlank String orderNo) {
        OrderDetailQuery<EshopGoodsOrderInfo> orderDetailQuery = new OrderDetailQuery<>();
        orderDetailQuery.setOrderNo(orderNo);
        orderDetailQuery.setLogisticsNo(logisticsNo);
        orderDetailQuery.setStatus(OrderStatus.WAIT_RECEIVE.name());
        return iOrdersService.editOrder(orderDetailQuery);
    }

    @ApiOperation("获取物流信息")
    @GetMapping("/logisticsInfo")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "logisticsNo", value = "物流号", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "com", value = "快递平台编号(拼音)", dataType = "String", paramType = "query")
            }
    )
    public BaseResponse getLogisticsInfo(@NotNull(message = "物流号不能为空") @RequestParam("logisticsNo") String logisticsNo,
                                         @NotNull(message = "快递平台") @RequestParam("com") String com) {
        return logisticsTemplateService.getLogisticsInfo(logisticsNo, com);
    }
}
