package cn.wise.sc.pay.pass.business.cashiers.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.domain.order.model.OrderStatus;

import java.util.Map;

/**
 * @description: 订单
 * @author: qh
 * @create: 2019-12-04 17:10
 **/
public interface IPaasOrderService {

    /**
     * 支付订单
     *
     * @param orderNo
     * @return
     */
    BaseResponse payOrder(String orderNo);

    /**
     * 创建订单
     *
     * @param orderDetail 订单描述
     * @return
     */
    BaseResponse createOrder(OrderDetailQuery orderDetail);

    /**
     * 改变订单状态
     *
     * @param orderNo     订单号
     * @param orderStatus 要改变的状态
     * @param condition   关键条件 <field,value> 如:卖家id
     * @return
     */
    BaseResponse changeOrderStatus(String orderNo, OrderStatus orderStatus, Map<String, Object> condition);

    /**
     * 根据条件查询
     *
     * @param condition 条件 map集合  <field,value>
     * @return list
     */
    BaseResponse getOrders(Map<String, Object> condition);


    /**
     * 根据条件查询 分页
     *
     * @param condition 条件 map集合  <field,value>
     * @param pageQuery 分页条件
     * @return page
     */
    BaseResponse getOrdersPage(Map<String, Object> condition, PageQuery pageQuery);

    /**
     * 根据key删除
     *
     * @param orderNo   订单号儿
     * @param condition 删除条件
     * @return
     */
    BaseResponse delOrders(String orderNo, Map<String, Object> condition);

    /**
     * 获取Orderid
     *
     * @return
     */
    BaseResponse getPreOrderNo();

    /**
     * 编辑订单
     *
     * @param orderQuery
     * @return
     */
    BaseResponse editOrder(OrderDetailQuery orderQuery);

    /**
     * 取消订单
     *
     * @param orderNo 订单号
     * @return
     */
    BaseResponse cancelOrder(String orderNo);

    /**
     * 支付成功后的回调
     *
     * @param orderNo 订单号
     * @return
     */
    BaseResponse afterSuccess(String orderNo);

    /**
     * 支付失败后的回调
     *
     * @param orderNo 订单号
     * @return
     */
    BaseResponse afterFail(String orderNo);

    BaseResponse pageSeller(int seller, int status, long start, long end, PageQuery pageQuery);

    /**
     * 发送当前班次的收入统计
     * @param start
     * @param end
     * @param sellerId
     * @return
     */
     BaseResponse sendMailEarningsAfterWork(Long start, Long end, Long sellerId);
}
