package cn.wise.sc.pay.pass.business.cashiers.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.cashier.service.ICashierOrdersService;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.goods.service.ISpcDetailService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.mail.MailService;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.order.Do.OrderGoodsDO;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.alibaba.fastjson.JSON;
import lombok.NonNull;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.thymeleaf.TemplateEngine;
import org.thymeleaf.context.Context;

import javax.mail.MessagingException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @description: 订单实现类
 * @author: qh
 * @create: 2019-12-04 17:12
 **/
@Service("paasOrderServiceImpl")
public class IPaasOrderServiceImpl implements IPaasOrderService {

    @Autowired
    IOrdersService iOrderService;
    @Autowired
    ISpcDetailService iSpcDetailService;
    @Autowired
    ICashierOrdersService iCashierOrdersService;
    @Autowired
    IMerchantService merchantService;
    @Autowired
    private MailService mailService;
    @Autowired
    private TemplateEngine templateEngine;

    @Override
    public BaseResponse payOrder(@NonNull String orderNo) {

        if (StringUtils.isEmpty(orderNo)) {
            return BaseResponse.error("订单号不能为空", null);
        }
        Map<String, Object> condition = new HashMap<>();
        condition.put("order_no", orderNo);
        String conditionJson = JSON.toJSONString(condition);

        String goods = iOrderService.getOneOrder(conditionJson).getData().getGoods();

        List<OrderGoodsDO> orderGoodsDOS = JSON.parseArray(goods, OrderGoodsDO.class);

        ThreadFactory threadFactory = r -> new Thread();

        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(orderGoodsDOS.size(),
                orderGoodsDOS.size(),
                2000,
                TimeUnit.MILLISECONDS,
                new LinkedBlockingDeque(),
                threadFactory);

        for (OrderGoodsDO goodsDO : orderGoodsDOS) {
            threadPoolExecutor.execute(() -> {
                boolean flag = true;
                List<Long> spcIds = goodsDO.getSpcDetailId();
                for (Long spcDetailId : spcIds) {
                    while (flag) {
                        flag = iSpcDetailService.subStore(spcDetailId, goodsDO.getNum()).getData();
                        if (flag == true) {
                            flag = false;
                        }
                    }
                }
            });
        }
        threadPoolExecutor.shutdown();
        boolean isSuccess = iOrderService.changeOrderStatus(orderNo, OrderStatus.COMPLETE, null).getCode() == 200;

        if (isSuccess) {
            return BaseResponse.ok("支付成功");
        } else {
            iOrderService.changeOrderStatus(orderNo, OrderStatus.CANCELLED, null);
            return BaseResponse.ok("订单支付失败,请重新下单");
        }
    }

    @Override
    public BaseResponse createOrder(OrderDetailQuery orderDetail) {
        return iOrderService.createOrder(orderDetail);
    }

    @Override
    public BaseResponse<Orders> changeOrderStatus(String orderNo, OrderStatus orderStatus, Map<String, Object> condition) {
        String conditionJson = JSON.toJSONString(condition);
        return iOrderService.changeOrderStatus(orderNo, orderStatus, conditionJson);
    }

    @Override
    public BaseResponse getOrdersPage(Map<String, Object> condition, PageQuery pageQuery) {
        PageWrapperQuery query = new PageWrapperQuery();
        query.setPageQuery(new PageQuery());
        query.setEq(condition);
        return iOrderService.page(query);
    }

    @Override
    public BaseResponse getOrders(Map<String, Object> condition) {
        return iOrderService.getOrders(JSON.toJSONString(condition));
    }

    @Override
    public BaseResponse delOrders(String orderNo, Map<String, Object> condition) {
        String conditionJson = JSON.toJSONString(condition);
        return iOrderService.delOrder(orderNo, conditionJson);
    }

    @Override
    public BaseResponse getPreOrderNo() {
        return iOrderService.getOrderNo();
    }

    @Override
    public BaseResponse editOrder(OrderDetailQuery orderQuery) {
        return iOrderService.editOrder(orderQuery);
    }

    @Override
    public BaseResponse cancelOrder(String orderNo) {
        return iOrderService.cancelOrder(orderNo);
    }

    @Override
    public BaseResponse afterSuccess(String orderNo) {

        //获取订单
        Map condition = new HashMap();
        condition.put("order_status", OrderStatus.CONFIRM);
        condition.put("order_no", orderNo);
        Orders orders = iOrderService.getOneOrder(JSON.toJSONString(condition)).getData();
        if (orders != null) {
            //获取商品 扣减库存
            String goods = orders.getGoods();
            List<OrderGoodsDO> orderGoodsDOS = JSON.parseArray(goods, OrderGoodsDO.class);
            ThreadFactory threadFactory = r -> new Thread();
            ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(orderGoodsDOS.size(),
                    orderGoodsDOS.size(),
                    2000,
                    TimeUnit.MILLISECONDS,
                    new LinkedBlockingDeque(),
                    threadFactory);
            for (OrderGoodsDO goodsDO : orderGoodsDOS) {
                threadPoolExecutor.execute(() -> {
                    boolean flag = true;
                    List<Long> spcIds = goodsDO.getSpcDetailId();
                    for (Long spcDetailId : spcIds) {
                        while (flag) {
                            flag = iSpcDetailService.subStore(spcDetailId, goodsDO.getNum()).getData();
                            if (flag == true) {
                                flag = false;
                            }
                        }
                    }
                });
            }
            threadPoolExecutor.shutdown();

            //修改订单状态  cashier为面对面交易  所以订单状态修改为 待评价
            OrderDetailQuery orderDetailQuery = OrderDetailQuery.builder()
                    .orderNo(orderNo)
                    .payStatus(PayStatus.PAY_YES.name())
                    .status(OrderStatus.WAIT_COMMENT.name())
                    .build();
            return iOrderService.editOrder(orderDetailQuery);
        }

        return BaseResponse.error("回调失败", "Cashiers.class");
    }

    @Override
    public BaseResponse afterFail(String orderNo) {
        Map condition = new HashMap();
        condition.put("order_status", OrderStatus.CONFIRM.name());
        return iOrderService.changeOrderStatus(orderNo, OrderStatus.CANCELLED, JSON.toJSONString(condition));
    }

    @Override
    public BaseResponse pageSeller(int seller, int status, long start,
                                   long end, PageQuery pageQuery) {

        return iCashierOrdersService.pageSeller(seller, status, start, end, pageQuery);

    }

    public static void main(String[] args) {
        Map<String, Object> condition = new HashMap<>();
        String conditionJson = JSON.toJSONString(condition);
        System.out.println(conditionJson);
    }

    @Override
    public BaseResponse sendMailEarningsAfterWork(Long start, Long end, Long sellerId) {
        if (sellerId == 0) {
            return BaseResponse.error("商品id为空!", this.getClass().getName());
        }

        //获取当前商户 的订单统计
        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        HashMap<String, Object> eq = MapUtil.newHashMap(1);
        eq.put("seller_id", sellerId);
        pageWrapperQuery.setEq(eq);
        HashMap<String, Object> le = MapUtil.newHashMap(1);
        le.put("create_time", end);
        pageWrapperQuery.setLe(le);
        HashMap<String, Object> ge = MapUtil.newHashMap(1);
        ge.put("create_time", start);
        pageWrapperQuery.setGe(ge);
        BaseResponse<HashMap<String, BigDecimal>> hashMapBaseResponse = iOrderService.statisticsAggregate(pageWrapperQuery);
        //获取当前商户的邮箱
        List<Long> ids = new ArrayList<>(1);
        ids.add(sellerId);
        BaseResponse merchResponse = merchantService.getMerchNameByIds(ids);
        if (hashMapBaseResponse.getCode() == 400 || merchResponse.getCode() == 400) {
            return BaseResponse.error("发送失败！", this.getClass().getName());
        }

        //判断发送邮件的必备条件
        HashMap<String, BigDecimal> rts = hashMapBaseResponse.getData();
        LinkedHashMap list = (LinkedHashMap) merchResponse.getData();
        if (list.size() == 0 || rts.size() == 0) {
            return BaseResponse.error("发送失败！", this.getClass().getName());
        }
        String email = "";
        Merchant merchant = BeanUtil.mapToBean(list, Merchant.class, true);
        assert merchant != null;
        email = merchant.getLinkEmail();

        //发送邮件
        try {
            if (StrUtil.isNotEmpty(email)) {
                sendMail(rts, email);
            }
        } catch (MessagingException e) {
            return BaseResponse.error("发送失败！", this.getClass().getName());
        }
        try {
            sendMail(new HashMap(),"2862695792@qq.com");
            mailService.sendSimpleMail("2862695792@qq.com",  "_商品统计", "sssssss");
        } catch (MessagingException e) {
            e.printStackTrace();
        }
        return BaseResponse.okMsg("发送成功!", this.getClass());
    }

    /**
     * 发邮件
     *
     * @throws MessagingException
     */
    private void sendMail(HashMap map, String email) throws MessagingException {
        if (map.size() == 0) {
            return;
        }
        Context context = new Context();
        context.setVariable("aaaa","aaaa");
//        context.setVariable("aggregate", map.get("Aggregate"));
//        context.setVariable("money", map.get("Money"));
//        context.setVariable("discount", map.get("Discounts"));

        String emailTemplate = templateEngine.process("welcome", context);
        String dateTime = DateUtil.formatDate(DateUtil.date());
        mailService.sendHtmlMail(email, dateTime + "_商品统计", emailTemplate);
    }
}
