package cn.wise.sc.pay.pass.business.manager.controller.association;

import cn.wise.sc.pay.api.association.service.IArticleCommentService;
import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.common.core.query.PageQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@RestController
@Api(description = "图文/文章评论")
@RequestMapping("/manage/association/comment")
@CrossOrigin
public class ArticleCommentController {

    @Autowired
    IArticleCommentService articleCommentService;

    @PostMapping("/addComment")
    @ApiOperation("评论")
    @ApiImplicitParam(name = "comment", value = "评论实体", paramType = "body", dataType = "Comment")
    public ResponseEntity addComment(@RequestBody Comment comment) {
        return ResponseEntity.ok(articleCommentService.addComment(comment).getData());
    }

    @DeleteMapping("/deleteComment")
    @ApiOperation("根据ids删除评论 多个用英文逗号隔开")
    @ApiImplicitParam(name = "idStr", value = "id 多个用逗号分割", paramType = "path", dataType = "String")
    public ResponseEntity deleteComment(String idStr) {
        return ResponseEntity.ok(articleCommentService.deleteComment(idStr).getData());
    }

    @PutMapping("/updatePraiseNum")
    @ApiOperation("评论点赞")
    @ApiImplicitParam(name = "id", value = "评论id", paramType = "path", dataType = "Long")
    public ResponseEntity updateCommentPraise(Long id) {
        return ResponseEntity.ok(articleCommentService.updateCommentPraise(id).getData());
    }

    @GetMapping("/selectComment")
    @ApiOperation("评论列表 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "条数", paramType = "path", dataType = "Integer")
    })
    public ResponseEntity getCommentList(PageQuery pageQuery){
        return ResponseEntity.ok(articleCommentService.getCommentList(pageQuery).getData());
    }

    @GetMapping("/commentPage")
    @ApiOperation("根据评论主体 评论列表 分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "targetId", value = "评论主体id", paramType = "path", dataType = "Long"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "条数", paramType = "path", dataType = "Integer")
    })
    public ResponseEntity getCommentsByTargetId(Long targetId, PageQuery pageQuery){
        return ResponseEntity.ok(articleCommentService.getCommentsByTargetId(targetId,pageQuery.getPageNo(),pageQuery.getPageSize()).getData());
    }
}

