package cn.wise.sc.pay.pass.business.manager.controller.association;


import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.association.service.IBlacklistService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.association.bean.Blacklist;
import cn.wise.sc.pay.domain.association.query.BlacklistQuery;
import cn.wise.sc.pay.laas.userbac.business.service.UserService;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@Api(description = "社群黑名单管理")
@RestController
@RequestMapping("/manage/association/blacklist")
@CrossOrigin
public class BlacklistController {

    @Autowired
    IBlacklistService blacklistService;
    @Autowired
    UserService userService;

    @GetMapping("/list")
    @ApiOperation("获取社群黑名单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userName", value = "用户名称", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "条数", paramType = "path", dataType = "Integer")
    })
    public ResponseEntity<Page<Blacklist>> list(String userName, PageQuery pageQuery) {
        BaseResponse<Page<Blacklist>> response = blacklistService.list(userName, pageQuery);
        if (response.getCode() == 400) {
            return ResponseEntity.ok(null);
        }
        List<Blacklist> records = response.getData().getRecords();
        if (records == null || records.size() == 0) {
            return ResponseEntity.ok(null);
        }
        for (Blacklist blacklist : records) {
            if (blacklist.getUserId() == null || blacklist.getUserId() == 0) {
                break;
            }
            SysUser appUserById = userService.getAppUserById(blacklist.getUserId().intValue());
            blacklist.setPhone(appUserById.getPhone());
        }

        return ResponseEntity.ok(response.getData());
    }

    @PostMapping("/addBlacklist")
    @ApiOperation("新增黑名单成员")
    @ApiImplicitParam(name = "blacklistQuery", value = "黑名单查询类", paramType = "body", dataType = "json")
    public ResponseEntity addBlacklist(@RequestBody BlacklistQuery blacklistQuery) {
        return ResponseEntity.ok(blacklistService.addBlacklist(blacklistQuery).getData());
    }

    @DeleteMapping("/deleteBlacklist")
    @ApiOperation("移除黑名单成员")
    @ApiImplicitParam(name = "id", value = "黑名单id", paramType = "path", dataType = "Long")
    public ResponseEntity deleteBannerById(Long id) {
        return ResponseEntity.ok(blacklistService.deleteBannerById(id).getData());
    }
}











