package cn.wise.sc.pay.pass.business.manager.controller.eshop;

import cn.wise.sc.pay.api.eshopping.service.IEshopBrandService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopBrandQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.DecimalMin;
import java.util.Map;

/**
 * 会员相关信息
 */
@CrossOrigin
@RestController
@Api(description = "网上商城品牌  管理端")
@RequestMapping("/manage/eshop/brand")
@Validated
public class EshopBrandController {

    @Autowired
    IEshopBrandService eshopBrandService;

    @PostMapping
    @ApiImplicitParams({
            @ApiImplicitParam(value = "名称", name = "name", paramType = "body", dataType = "String", required = true),
            @ApiImplicitParam(value = "logo地址", name = "logo", paramType = "body", dataType = "String")
    })
    public BaseResponse addBrand(@RequestBody @Valid EshopBrandQuery eshopBrand){
        return eshopBrandService.addBrand(eshopBrand);
    }

    @PutMapping
    @ApiImplicitParams({
            @ApiImplicitParam(value = "品牌标识", name = "brandId", paramType = "body", dataType = "String", required = true),
            @ApiImplicitParam(value = "名称", name = "name", paramType = "body", dataType = "String", required = true),
            @ApiImplicitParam(value = "logo地址", name = "logo", paramType = "body", dataType = "String")
    })
    public BaseResponse updateBrand(@RequestBody @Valid EshopBrandQuery eshopBrand){
        return eshopBrandService.updateBrand(eshopBrand);
    }

    @GetMapping("brands")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "品牌名称", name = "name", paramType = "query", dataType = "String"),
            @ApiImplicitParam(value = "页码", name = "pageNo", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "每页条数", name = "pageSize", paramType = "query", dataType = "Integer", required = true)
    })
    public BaseResponse brandPage(@RequestParam("name") String name,
                                  @RequestParam(value = "pageNo",required = true) @DecimalMin("0") Integer pageNo,
                                  @RequestParam(value = "pageSize",required = true) Integer pageSize){
        return eshopBrandService.brandPage(name, pageNo, pageSize);
    }

    @DeleteMapping
    public BaseResponse deleteBrand(@RequestParam(value = "id", required = true)  @DecimalMin("0") Integer id){
        return BaseResponse.ok(eshopBrandService.deleteBrand(id));
    }
}
