package cn.wise.sc.pay.pass.business.manager.controller.order;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.order.service.ISafeguardService;
import cn.wise.sc.pay.api.order.service.IZHPaymentService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.order.SafeguardUtil;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.Safeguard;
import cn.wise.sc.pay.domain.order.model.SafeguardStatus;
import cn.wise.sc.pay.domain.order.query.SafeguardDetailQuery;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.util.HashMap;
import java.util.Map;

/**
 * @description: 维权订单
 * @author: qh
 * @create: 2020-03-31 15:13
 */
@Api(tags = "维权订单(管理端)")
@CrossOrigin
@RestController
@Validated
@RequestMapping("/manage/safeguard")
public class SafeguardController {

    final ISafeguardService iSafeguardService;
    final IZHPaymentService izhPaymentService;
    final
    IOrdersService iOrdersService;

    public SafeguardController(
            ISafeguardService iSafeguardService,
            IZHPaymentService izhPaymentService,
            IOrdersService iOrdersService) {
        this.iSafeguardService = iSafeguardService;
        this.izhPaymentService = izhPaymentService;
        this.iOrdersService = iOrdersService;
    }

    @PutMapping("/audit")
    @ApiOperation("退款审核")
    @ApiImplicitParams({
            @ApiImplicitParam(
                    name = "refundNo",
                    value = "维权订单号",
                    paramType = "query",
                    dataType = "String",
                    required = true),
            @ApiImplicitParam(
                    name = "status",
                    value = "审核结果  2:不过 1:过",
                    paramType = "query",
                    dataType = "int",
                    required = true),
            @ApiImplicitParam(name = "reason", value = "原因", paramType = "query", dataType = "String")
    })
    public BaseResponse audit(
            @NotNull(message = "维权订单号为空") @RequestParam("refundNo") String refundNo,
            @NotNull(message = "审核结果不能为空") @RequestParam("status") Integer status,
            @RequestParam(value = "reason", required = false) String reason) {
        return iSafeguardService.audit(refundNo, status, reason);
    }

    @ApiOperation("退款单详情")
    @GetMapping("/{refundNo}")
    @ApiImplicitParam(
            name = "refundNo",
            value = "维权订单号",
            paramType = "query",
            dataType = "String",
            required = true)
    public BaseResponse refundDetails(
            @PathVariable("refundNo") @NotNull(message = "维权订单号不能为空") String refundNo) {
        return iSafeguardService.refundDetails(refundNo);
    }

    @ApiOperation("退款单编辑")
    @PutMapping("/status")
    @ApiImplicitParam(
            name = "safeguardDetailQuery",
            value = "维权订单",
            paramType = "body",
            dataType = "SafeguardDetailQuery")
    public BaseResponse<String> edit(@RequestBody SafeguardDetailQuery safeguardDetailQuery) {

        BaseResponse<String> response = iSafeguardService.edit(safeguardDetailQuery);

        if (response.getCode() == 400) {
            return BaseResponse.error("退款失败!", this.getClass().getName());
        }
        if (SafeguardStatus.REFUND.name()
                .equals(SafeguardUtil.getStatus(safeguardDetailQuery.getStatus()).name())) {
            // 获取退款单信息.
            BaseResponse refundDetailsResponse =
                    iSafeguardService.refundDetails(safeguardDetailQuery.getRefundNo());
            if (refundDetailsResponse.getCode() == 200) {
                Map map = (Map) refundDetailsResponse.getData();
                Safeguard safeguard = BeanUtil.mapToBean(map, Safeguard.class, true);
                // 发起退款申请
                String amount = new Double(safeguard.getAmount() * 100).intValue() + "";
                Map<String, String> map1 = new HashedMap(1);
                map1.put("unified_order_no", safeguard.getOrderNo());
                Orders data = iOrdersService.getOneOrder(JSON.toJSONString(map1)).getData();
                if (data == null) {
                    return BaseResponse.error("退款失败!", null);
                }
                return izhPaymentService.applyForRefund(
                        safeguard.getRefundNo(), safeguard.getOrderNo(), amount, amount, safeguard.getReason(), data.getSellerId() + ""
                );
            }
            return refundDetailsResponse;
        } else {
            return response;
        }
    }

    @ApiOperation("分页查询")
    @GetMapping("/page")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "refundNo", value = "维权订单号", paramType = "query", dataType = "String")
    })
    public BaseResponse page(String refundNo, PageQuery pageQuery) {

        PageWrapperQuery pageWrapperQuery = PageWrapperQueryUtil.create();
        pageWrapperQuery.setPageQuery(pageQuery);
        Map<String, Object> eq = new HashMap<>(1);
        if (StrUtil.isNotEmpty(refundNo)) {
            eq.put("refund_no", refundNo);
            pageWrapperQuery.setEq(eq);
        }

        return iSafeguardService.page(pageWrapperQuery);
    }
}
