package cn.wise.sc.pay.pass.business.manager.controller.points;

import cn.wise.sc.pay.api.points.service.IGoodRecordService;
import cn.wise.sc.pay.api.points.service.IMemberShipService;
import cn.wise.sc.pay.api.points.service.IPointDetailService;
import cn.wise.sc.pay.api.points.service.IPointPhotoService;
import cn.wise.sc.pay.api.points.service.IPointsSourceV1Service;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.PointPhoto;
import cn.wise.sc.pay.domain.point.vo.PointDetailVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

/**
 * @description: 积分控制器
 * @author: qh
 * @create: 2020-04-28 15:47
 **/
@RestController
@CrossOrigin
@RequestMapping("/points")
public class PointsController {

    final
    IGoodRecordService iGoodRecordService;
    final
    IMemberShipService iMemberShipService;
    final
    IPointDetailService iPointDetailService;
    final
    IPointsSourceV1Service iPointsSourceV1Service;
    final
    IPointPhotoService iPointPhotoService;

    public PointsController(IGoodRecordService iGoodRecordService,
                            IMemberShipService iMemberShipService,
                            IPointDetailService iPointDetailService,
                            IPointsSourceV1Service iPointsSourceV1Service,
                            IPointPhotoService iPointPhotoService) {
        this.iGoodRecordService = iGoodRecordService;
        this.iMemberShipService = iMemberShipService;
        this.iPointDetailService = iPointDetailService;
        this.iPointsSourceV1Service = iPointsSourceV1Service;
        this.iPointPhotoService = iPointPhotoService;
    }


    /**
     * 查询用户积分消费记录
     *
     * @param pageNo   起始页
     * @param pageSize 每页大小
     * @param userId   用户id
     * @return
     */
    @GetMapping("/consume/{userId}")
    @ApiOperation("查询用户积分消费记录")
    BaseResponse listByConsume(@RequestParam("pageNo") Integer pageNo,
                               @RequestParam("pageSize") Integer pageSize,
                               @PathVariable("userId") Long userId) {
        return iPointDetailService.listByConsume(pageNo, pageSize, userId);
    }

    /**
     * 查询用户积分增加记录
     *
     * @param pageNo   起始页
     * @param pageSize 每页大小
     * @param userId   用户id
     * @return
     */
    @GetMapping("/increase/{userId}")
    @ApiOperation("查询用户积分增加记录")
    BaseResponse listByIncrease(@RequestParam("pageNo") Integer pageNo,
                                @RequestParam("pageSize") Integer pageSize,
                                @PathVariable("userId") Long userId) {
        return iPointDetailService.listByIncrease(pageNo, pageSize, userId);
    }

    @ApiOperation("查询用户所有记录")
    @GetMapping("/pointDetails/{userId}")
    BaseResponse<Page<PointDetailVO>> listPointDetails(PageQuery pageQuery, @PathVariable("userId") int userId) {
        return iPointDetailService.list(pageQuery.getPageNo(), pageQuery.getPageSize(), userId);
    }
    @PutMapping("/pointPhoto/{id}/{status}")
    public BaseResponse<Boolean> audit(@NotNull(message = "id不能为空!") @PathVariable("id") Long id,
                                       @NotNull(message = "状态不能为空!") @PathVariable("status") Integer status) {
        return iPointPhotoService.audit(id, status);
    }

    @GetMapping("/pointPhoto/page")
    public BaseResponse<IPage<PointPhoto>> page(@NotNull(message = "分页参数不能为空!") PageQuery pageQuery,
                                                Integer status){
        return iPointPhotoService.page(pageQuery, status);
    }
    @GetMapping("/pointPhoto/setting")
    public BaseResponse<Boolean> getAutoAuditSetting() {
        return iPointPhotoService.getAutoAuditSetting();
    }

    @PutMapping("/pointPhoto/setting")
    public BaseResponse<Boolean> editAutoAuditSetting() {
        return iPointPhotoService.editAutoAuditSetting();
    }

}
