package cn.wise.sc.pay.paas.open.api.controller;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantProfitService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.OrderStatusUtils;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.paas.open.api.OpenApiApplication;
import cn.wise.sc.pay.paas.open.api.model.OrdersVO;
import cn.wise.sc.pay.paas.open.api.service.IPaasOrderService;
import cn.wise.sc.pay.api.order.service.IZHPaymentService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @description: 订单开放接口
 * @author: qh
 * @create: 2020-04-22 10:19
 **/
@CrossOrigin
@RestController
@RequestMapping("/v1/api/order")
public class OrderOpenApiController {

    @Autowired
    IPaasOrderService iOrderService;
    @Autowired
    IMerchantProfitService merchantProfitService;
    @Autowired
    IZHPaymentService izhPaymentService;

    @GetMapping("/test")
    public String test() {
        return "111111111";
    }

    @PostMapping("/new")
    @ApiOperation("新增一个订单")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderDetailQuery")
    public BaseResponse newOrder(@ApiParam @RequestBody OrderDetailQuery orderQuery) {
        Map<String, String> rts = new HashMap<>();
        if (StrUtil.isEmpty(orderQuery.getDeviceNo())
                || orderQuery.getTotalPrice() == 0D
                || orderQuery.getPayPrice() == 0D
                || orderQuery.getSellerId() == 0L) {
            return BaseResponse.error("创建失败,请检查参数是否正确!", OpenApiApplication.class);
        }
        orderQuery.setStatus(OrderStatus.WAIT_PAY.name());
        BaseResponse baseResponse = iOrderService.createOrder(orderQuery);
        Object obj = baseResponse.getData();
        if (baseResponse.getCode() == 200 && obj != null) {
            Orders order = (Orders) obj;
            StringBuilder sb = new StringBuilder("https://oxo.chfatech.com/mp?");
            sb.append("storeId=");
            sb.append(order.getSellerId());
            sb.append("&orderId=");
            sb.append(order.getOrderNo());
            sb.append("&price=");
            sb.append(order.getTotalPrice());
            sb.append("&popPrice=");
            sb.append(order.getDiscount() == null ? "0.00" : order.getDiscount());
            sb.append("&reallyPrice=");
            sb.append(order.getPayPrice());
            sb.append("&SN=");
            sb.append(order.getDeviceNo());
            String qrUrl = sb.toString();
            rts.put("qrUrl", qrUrl);
            rts.put("orderInfo", JSON.toJSONString(order));
            return BaseResponse.ok(rts);
        } else {
            return BaseResponse.error(baseResponse.getMsg(), OpenApiApplication.class);
        }
    }

    @PostMapping("/new/entry")
    @ApiOperation("挂单")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderQuery")
    public BaseResponse newEntryOrder(@RequestBody OrderDetailQuery orderQuery) {

        orderQuery.setStatus(OrderStatus.ENTRY_ORDER.name());

        return iOrderService.createOrder(orderQuery);
    }


    @PutMapping("/entry/")
    @ApiOperation("修改挂单信息")
    @ApiImplicitParam(name = "orderQuery", value = "订单请求", paramType = "body", dataType = "OrderQuery")
    public BaseResponse editEntryOrder(@RequestBody OrderDetailQuery orderQuery) {

        orderQuery.setStatus(OrderStatus.ENTRY_ORDER.name());
        return iOrderService.editOrder(orderQuery);
    }

    @GetMapping("list/user/{userId}/{status}")
    @ApiOperation("查询用户订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse listUsers(@PathVariable("userId") int userId, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("user_id", userId);

        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus);
        responseEntity = iOrderService.getOrders(condition);
        return responseEntity;
    }

    @GetMapping("page/user/{userId}/{status}")
    @ApiOperation("查询用户订单  分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 确定认支付,2 待评论,3 订单完成,4 申请退款,5 已退款,6 取消订单 ,7 待售后 8 挂单 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse pageUsers(PageQuery query, @PathVariable("userId") int userId, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("user_id", userId);

        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus);
        responseEntity = iOrderService.getOrdersPage(condition, query);
        return responseEntity;
    }

    @GetMapping("list/seller/{seller}/{status}")
    @ApiOperation("查询商户订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse listSeller(@PathVariable("seller") int seller, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("seller_id", seller);
        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus);
        responseEntity = iOrderService.getOrders(condition);
        return responseEntity;
    }

    @PutMapping("/confirmPayment")
    @ApiOperation("确定付款(现金)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "payPrice", value = "实付金额", paramType = "query", dataType = "Double")
    })
    public BaseResponse confirmPayment(@NotNull(message = "订单号不能为空") String orderNo,
                                       @Min(value = 0, message = "实际支付金额需大于0") Double payPrice) {
        OrderDetailQuery query = new OrderDetailQuery();
        query.setOrderNo(orderNo);
        query.setPaas("现金支付");
        query.setPayment("现金支付");
        query.setPayPrice(payPrice);
        query.setStatus(OrderStatus.COMPLETE.name());

        BaseResponse response = iOrderService.editOrder(query);

        if (response.getCode() == 200) {
            return BaseResponse.okMsg("支付成功", OpenApiApplication.class);
        } else {
            return response;
        }
    }

    @PutMapping("{orderNo}")
    @ApiOperation("支付订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号儿", paramType = "path", dataType = "String")
    public BaseResponse payOrder(@PathVariable("orderNo") String orderNo) {

        return iOrderService.payOrder(orderNo);
    }

    @PostMapping("/page")
    @ApiOperation("查询订单分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "consignee", value = "收货人", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "goodsName", value = "商品名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "userName", value = "用户名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "shopId", value = "商铺id", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "orderStatus", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]", paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = "start", value = "开始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse getPage(@RequestBody PageQuery page, String consignee, String goodsName,
                                String userName, Long shopId, Integer orderStatus, long start, long end) {

        PageWrapperQuery pageWrapperQuery = new PageWrapperQuery();
        pageWrapperQuery.setPageQuery(page);
        //构建等于条件
        Map<String, Object> eq = new HashMap();
        if (shopId != null) {
            eq.put("seller_id", shopId);
        }

        if (orderStatus != null) {
            OrderStatus status = OrderStatusUtils.getStatus(orderStatus);
            if (!OrderStatus.ALL.name().equals(status.name())) {
                eq.put("order_status", OrderStatusUtils.getStatus(orderStatus));
            }
        }
        //构建like条件
        Map<String, Object> like = new HashMap<>();
        if (StrUtil.isNotEmpty(consignee)) {
            like.put("consignee", consignee);
        }
        if (StrUtil.isNotEmpty(goodsName)) {
            like.put("goods", goodsName);
        }
        if (StrUtil.isNotEmpty(userName)) {
            like.put("username", userName);
        }
        Map<String, Object> lt = new HashMap<>();
        Map<String, Object> gt = new HashMap<>();
        if (start != 0 && end != 0) {
            gt.put("create_time", start);
            lt.put("create_time", end);
        }
        pageWrapperQuery.setLike(like);
        pageWrapperQuery.setEq(eq);
        pageWrapperQuery.setGt(gt);
        pageWrapperQuery.setLt(lt);
        BaseResponse baseResponse = iOrderService.page(pageWrapperQuery);
        if (baseResponse.getCode() == 200) {
            Page page1 = baseResponse.getPage();
            //转换成VO
            List<Orders> list = page1.getRecords();
            List orderVOs = list.stream().map(arg -> new OrdersVO(arg)).collect(Collectors.toList());
            page1.setRecords(orderVOs);
        }
        return baseResponse;
    }

    @GetMapping("/orderNo")
    @ApiOperation("获取订单号")
    public BaseResponse getOrderNo() {

        return BaseResponse.ok(iOrderService.getPreOrderNo());
    }

    @GetMapping("/page/v2/seller/{seller}/{status}")
    @ApiOperation("查询商户订单  分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            ),
            @ApiImplicitParam(name = "start", value = "起始时间", paramType = "query", dataType = "long"),
            @ApiImplicitParam(name = "end", value = "结束时间", paramType = "query", dataType = "long")
    })
    public BaseResponse pageSeller(@PathVariable("seller") int seller, @PathVariable("status") int status,
                                   long start, long end, PageQuery pageQuery) {

        return iOrderService.pageSeller(seller, status, start, end, pageQuery);
    }

    @GetMapping("/page/seller/{seller}/{status}")
    @ApiOperation("查询商户订单  分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "seller", value = "商户id", paramType = "path", dataType = "int"),
            @ApiImplicitParam(name = "status", value = "订单类型 [0 待支付,1 支付中,2 待评论,3 订单完成 ,5 已退款,6 取消订单,  8 挂单 12 待发货 13 待收货 ,其他:查询所有]",
                    paramType = "path", dataType = "int"
            )
    })
    public BaseResponse pageSeller(PageQuery query, @PathVariable("seller") int seller, @PathVariable("status") int status) {

        BaseResponse responseEntity;
        Map<String, Object> condition = new HashMap<>();
        condition.put("seller_id", seller);
        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        condition.put("order_status", orderStatus);
        responseEntity = iOrderService.getOrdersPage(condition, query);
        return responseEntity;
    }

    @DeleteMapping("{orderNo}")
    @ApiOperation("取消订单")
    @ApiImplicitParam(name = "orderNo", value = "订单号儿", paramType = "path", dataType = "String")
    public BaseResponse delOrder(@PathVariable("orderNo") String orderNo) {
        return iOrderService.cancelOrder(orderNo);
    }

    @PostMapping("/applyForRefund")
    public BaseResponse refund(String orderNo) {
        if (StrUtil.isEmpty(orderNo)) {
            return BaseResponse.error("订单号为空", "cashier");
        }
        Snowflake snowflake = IdUtil.createSnowflake(1, 1);
        String refundNo = snowflake.nextIdStr();
        Map<String, Object> condition = new HashMap<>(1);
        condition.put("order_no", orderNo);
        condition.put("pay_status", PayStatus.PAY_YES.name());
        OrderResponse response = iOrderService.getOneOrder(JSON.toJSONString(condition));
        if (response.getCode() == 200) {
            Orders order = response.getData();
            BaseResponse response1 = izhPaymentService.applyForRefund(refundNo, orderNo,
                    order.getPayPrice().multiply(new BigDecimal(100)).intValue() + "",
                    order.getPayPrice().multiply(new BigDecimal(100)).intValue() + "", "",
                    order.getSellerId() + "");
            return response1;
        } else {
            return response;
        }
    }

}
