package cn.wise.sc.pay.api.association.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.association.bean.Topic;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(value = "saas-v1-association", path = "/association/topic", fallback = TopicServiceImpl.class)
public interface ITopicService {

    /**
     * 获取话题列表
     *
     * @return
     */
    @GetMapping("/list")
    BaseResponse<List<Topic>> list();


    /**
     * 添加话题
     *
     * @param topic
     * @return
     */
    @PostMapping("/addTopic")
    BaseResponse addTopic(@RequestBody Topic topic);

    /**
     * 分页话题列表
     *
     * @param name      话题名
     * @param pageQuery 分页实体
     * @return
     */
    @GetMapping("/page")
    BaseResponse<Page<Topic>> pageTopic(@RequestParam("name") String name, @RequestBody PageQuery pageQuery);

    /**
     * 更新话题名称
     *
     * @param topic 话题请求实体
     * @return
     */
    @PutMapping("updateName")
    BaseResponse updateTopicName(@RequestBody Topic topic);


    /**
     * 更新社群分类排序
     *
     * @param tId
     * @param oId
     * @return
     */
    @PutMapping("updateOrder")
    BaseResponse updateTopicOrder(@RequestParam("tId") Long tId, @RequestParam("oId") Long oId);

    /**
     * 删除社群分类
     *
     * @param id 社区id
     * @return
     */
    @DeleteMapping("delete")
    BaseResponse deleteTopicById(@RequestParam("id") Long id);

}
