package cn.wise.sc.pay.business.association.controller;

import cn.wise.sc.pay.business.association.service.IBlacklistService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.association.bean.Blacklist;
import cn.wise.sc.pay.domain.association.query.BlacklistQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 前端控制器
 *
 * @author lqk
 * @since 2019-11-19
 */
@Api(description = "社群黑名单管理")
@RestController
@RequestMapping("/association/blacklist")
@CrossOrigin
public class BlacklistController {

    @Autowired
    IBlacklistService blacklistService;


    @GetMapping("/list")
    @ApiOperation("获取社群黑名单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userName", value = "用户名称", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "pageNo", value = "页码", paramType = "path", dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "条数", paramType = "path", dataType = "Integer")
    })
    public BaseResponse<Page<Blacklist>> list(String userName, PageQuery pageQuery) {
        QueryWrapper<Blacklist> wrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(userName)) {
            wrapper.like("user_name", userName);
        }
        Page<Blacklist> page = null;
        if (pageQuery != null) {
            page = new Page<>();
            page.setSize(pageQuery.getPageSize());
            page.setCurrent(pageQuery.getPageNo());
            page.setAsc("create_time");
        }
        page = (Page<Blacklist>) blacklistService.page(page, wrapper);

        return BaseResponse.ok(page);
    }

    @PostMapping("/addBlacklist")
    @ApiOperation("新增黑名单成员")
    @ApiImplicitParam(
            name = "blacklistQuery",
            value = "黑名单查询类",
            paramType = "body",
            dataType = "json")
    public BaseResponse addBlacklist(@RequestBody BlacklistQuery blacklistQuery) {
        if (blacklistQuery != null) {
            return blacklistService.addBlacklist(blacklistQuery);
        }
        return BaseResponse.error("添加失败", Blacklist.class);
    }

    @DeleteMapping("/deleteBlacklist")
    @ApiOperation("移除黑名单成员")
    @ApiImplicitParam(name = "id", value = "黑名单id", paramType = "path", dataType = "Long")
    public BaseResponse deleteBannerById(@RequestParam("id") Long id) {
        boolean res = blacklistService.removeById(id);
        return res ? BaseResponse.ok("移除成功") : BaseResponse.error("移除失败", Blacklist.class);
    }
}
