package cn.wise.sc.pay.business.association.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.association.configure.FastDFSUtils;
import cn.wise.sc.pay.business.association.mapper.BannerMapper;
import cn.wise.sc.pay.business.association.service.IArticleBannerService;
import cn.wise.sc.pay.business.association.service.IBannerService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.association.bean.ArticleBanner;
import cn.wise.sc.pay.domain.association.bean.Banner;
import cn.wise.sc.pay.domain.association.query.BannerQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@Service
public class BannerServiceImpl extends ServiceImpl<BannerMapper, Banner> implements
        IBannerService {

    @Autowired
    IArticleBannerService articleBannerService;

    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    public BaseResponse<List<Banner>> selectList() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.ne("name", "推荐");
        wrapper.orderByAsc("order_num");
        List<Banner> result = this.list(wrapper);
        result.stream().forEach(arg -> {
            if (StrUtil.isNotEmpty(arg.getIcon())) {
                arg.setIcon(fastDFSUtils.fullUrl(arg.getIcon()));
            }
        });
        return BaseResponse.ok(result);
    }

    @Override
    public BaseResponse getRwList() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.ne("name", "推荐");
        wrapper.orderByAsc("order_num");
        wrapper.eq("rw_flag", 1);
        List<Banner> result = this.list(wrapper);
        result.stream().forEach(arg -> {
            if (StrUtil.isNotEmpty(arg.getIcon())) {
                arg.setIcon(fastDFSUtils.fullUrl(arg.getIcon()));
            }
        });
        return BaseResponse.ok(result);
    }

    @Override
    public BaseResponse<List<Banner>> bannerList() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.orderByAsc("order_num");
        List<Banner> result = this.list(wrapper);

        result.stream().forEach(arg -> {
            if (StrUtil.isNotEmpty(arg.getIcon())) {
                arg.setIcon(fastDFSUtils.fullUrl(arg.getIcon()));
            }
        });
        return BaseResponse.ok(result);
    }

    @Override
    public BaseResponse insertBanner(BannerQuery bannerQuery) {
        QueryWrapper<Banner> wrapper = new QueryWrapper<>();
        wrapper.eq("name", bannerQuery.getName());
        List<Banner> list = this.list(wrapper);
        if (list != null && list.size() > 0) {
            return BaseResponse.error("分类已存在", Banner.class);
        }
        wrapper = new QueryWrapper<>();
        wrapper.orderByDesc("order_num");
        wrapper.last("limit 1");
        Banner last = this.getOne(wrapper);
        int order = 1;
        if (last != null) {
            order = last.getOrderNum() + 1;
        }

        Banner banner = Banner.builder()
                .name(bannerQuery.getName())
                .orderNum(order)
                .createTime(System.currentTimeMillis())
                .build();
        boolean result = this.save(banner);
        return result ? BaseResponse.ok("添加分类成功") : BaseResponse.error("添加失败", Banner.class);

    }

    @Override
    public BaseResponse updateBanner(Banner banner) {
        boolean result = this.updateById(banner);
        return result ? BaseResponse.ok("更新成功") : BaseResponse.error("更新失败", Banner.class);
    }

    @Override
    public BaseResponse updateBannerOrder(Long tId, Long oId) {
        Banner banner1 = this.getById(tId);
        Banner banner2 = this.getById(oId);
        if (banner1 == null || banner2 == null) {
            return BaseResponse.error("更新失败", Banner.class);
        }
        int order = banner1.getOrderNum();
        banner1.setOrderNum(banner2.getOrderNum());
        banner2.setOrderNum(order);
        this.updateById(banner1);
        this.updateById(banner2);
        return BaseResponse.ok("更新成功");
    }

    @Override
    public BaseResponse deleteBannerById(Long id) {
        boolean result = this.removeById(id);
        UpdateWrapper<ArticleBanner> wrapper = new UpdateWrapper<>();
        wrapper.eq("banner_id", id);
        articleBannerService.remove(wrapper);
        return result ? BaseResponse.ok("删除成功") : BaseResponse.error("删除失败", Banner.class);
    }
}
