package cn.wise.sc.pay.business.association.service.impl;

import cn.wise.sc.pay.business.association.mapper.CollectionMapper;
import cn.wise.sc.pay.business.association.service.IArticleService;
import cn.wise.sc.pay.business.association.service.ICollectionService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.association.bean.Article;
import cn.wise.sc.pay.domain.association.bean.Collection;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-25
 */
@Service
public class CollectionServiceImpl extends ServiceImpl<CollectionMapper, Collection> implements ICollectionService {

    @Autowired
    IArticleService articleService;

    @Override
    public BaseResponse addCollection(Long articleId, Long userId) {
        if(articleId == null || userId == null){
            return BaseResponse.error("参数有误",Article.class);
        }
        Collection collection = Collection.builder()
                .articleId(articleId)
                .userId(userId)
                .createTime(System.currentTimeMillis())
                .build();
        Boolean result = false;
        try {
            result = this.save(collection);
        }catch (Exception e){
            return BaseResponse.error("已收藏过",Article.class);
        }
        Article article = articleService.getById(articleId);
        if(article != null){
            article.setCollectNum(article.getCollectNum().longValue() + 1);
            articleService.updateById(article);
        }
        return result ? BaseResponse.ok("收藏成功")
                : BaseResponse.error("收藏失败",Article.class);
    }

    @Override
    public BaseResponse deleteCollection(Long id, Long userId) {
        if(userId == null || id == null){
            return BaseResponse.error("参数有误",Article.class);
        }
        Article article = articleService.getById(id);
        if(article != null){
            article.setCollectNum(article.getCollectNum().longValue() - 1);
            articleService.updateById(article);
        }
        UpdateWrapper<Collection> wrapper = new UpdateWrapper<>();
        wrapper.eq("article_id",id);
        wrapper.eq("user_id",userId);
        boolean result = this.remove(wrapper);
        return result ? BaseResponse.ok("取消收藏") : BaseResponse.error("取消收藏失败",Article.class);
    }

    @Override
    public BaseResponse judgeCollection(Long articleId, Long userId) {
        if(userId == null || articleId == null){
            return BaseResponse.error("参数有误",Article.class);
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("user_id",userId);
        wrapper.eq("article_id",articleId);
        Collection collection = this.getOne(wrapper);
        if(collection != null){
            return BaseResponse.ok(1);
        }
        else {
            return BaseResponse.ok(0);
        }
    }
}
