package cn.wise.sc.pay.business.association.service.impl;

import cn.wise.sc.pay.business.association.mapper.ConcernMapper;
import cn.wise.sc.pay.business.association.service.IArticleService;
import cn.wise.sc.pay.business.association.service.IConcernService;
import cn.wise.sc.pay.domain.association.bean.Concern;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-25
 */
@Service
public class ConcernServiceImpl extends ServiceImpl<ConcernMapper, Concern> implements IConcernService {

    @Autowired
    IArticleService articleService;

    @Override
    public ResponseEntity addConcern(Concern concern) {
        if(concern != null){
            QueryWrapper<Concern> wrapper = new QueryWrapper<>();
            wrapper.eq("me_id",concern.getMeId());
            wrapper.eq("he_id",concern.getHeId());
            wrapper.last("limit 1");
            Concern con = this.getOne(wrapper);
            if(con != null){
                con.setStatus((byte)1);
                this.updateById(con);
                concern.setStatus((byte)1);
            }else{
                concern.setStatus((byte)0);
            }
            concern.setCreateTime(System.currentTimeMillis());
            boolean result = this.save(concern);
            return result ? ResponseEntity.ok("关注成功") : ResponseEntity.badRequest().body("失败");
        }
        return ResponseEntity.badRequest().body("失败");
    }

    @Override
    public ResponseEntity deleteConcern(Long meId, Long heId) {
        if(meId != null && heId != null){
            QueryWrapper<Concern> wrapper = new QueryWrapper<>();
            wrapper.eq("me_id",heId);
            wrapper.eq("he_id",meId);
            wrapper.last("limit 1");
            Concern con = this.getOne(wrapper);
            if(con != null){
                con.setStatus((byte)0);
                this.updateById(con);
            }
            UpdateWrapper<Concern> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("me_id",meId);
            updateWrapper.eq("he_id",heId);
            boolean result = this.remove(updateWrapper);
            return result ? ResponseEntity.ok("成功取消关注") : ResponseEntity.badRequest().body("取消关注失败");
        }
        return null;
    }

    @Override
    public ResponseEntity<List<Concern>> getConcernList(Long meId,Integer type) {
        if(meId == null || type == null){
            return ResponseEntity.ok(new ArrayList<>());
        }
        QueryWrapper<Concern> wrapper = new QueryWrapper<>();
        if(type == 0){
            wrapper.eq("me_id",meId);
        }else{
            wrapper.eq("he_id", meId);
        }
        List<Concern> result = this.list(wrapper);
        return ResponseEntity.ok(result);
    }

    @Override
    public ResponseEntity<Map<String, Integer>> getMainData(Long userId) {
        if(userId != null){
            Map<String,Integer> resultMap = new HashMap<>();
            QueryWrapper<Concern> wrapper = new QueryWrapper<>();
            wrapper.eq("me_id",userId);
            int myConcerns = this.count(wrapper);
            resultMap.put("myConcern",myConcerns);
            QueryWrapper<Concern> heWrapper = new QueryWrapper<>();
            heWrapper.eq("he_id",userId);
            int concernsMe = this.count(heWrapper);
            resultMap.put("concernMe",concernsMe);
            Integer praiseSum = articleService.getMyPraiseSum(userId);
            resultMap.put("praiseSum",praiseSum);
            return ResponseEntity.ok(resultMap);
        }
        return null;
    }
}
