package cn.wise.sc.pay.api.cashier.service;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

/**
 * @description: 订单描述
 * @author: qh
 * @create: 2019-12-12 12:15
 **/
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@ApiModel
public class OrderDetailQuery<Entity> implements Serializable {

    private static final long serialVersionUID = -8605708775492489693L;

    private long id;

    /**
     * 商品对象  不同平台不同实现
     */
    @ApiModelProperty("商品列表")
    private List<Entity> goods;

    /**
     * 订单号 全局唯一
     */
    @ApiModelProperty("订单编号")
    private String orderNo;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    private long createTime;

    /**
     * 支付方式
     */
    @ApiModelProperty("支付方式")
    private String payment;

    /**
     * 实际支付金额
     */
    @ApiModelProperty("实际支付金额")
    private double payPrice;

    /**
     * 用户id
     */
    @ApiModelProperty("用户id")
    private Long userId;

    /**
     * 用户电话
     */
    @ApiModelProperty("用户电话")
    private String userPhone;

    /**
     * 备注
     */
    @ApiModelProperty("备注")
    private String remark;

    /**
     * 平台
     */
    @ApiModelProperty("平台")
    private String paas;

    /**
     * 支付状态
     */
    @ApiModelProperty("支付状态")
    private String payStatus;

    /**
     * 订单状态
     */
    @ApiModelProperty(hidden = true)
    private String status;

    /**
     * 总价格
     */
    @ApiModelProperty("订单状态")
    private double totalPrice;
    /**
     * 商户id
     */
    @ApiModelProperty("总价格")
    private Long sellerId;

    /**
     * 展览id/运费
     */
    @ApiModelProperty("展览id")
    private String field;

    /**
     * 商品名称
     */
    @ApiModelProperty("商品名称  传参选用")
    private String goodsName;

    /**
     * 卓号
     */
    @ApiModelProperty("桌号 传参选用")
    private String tableId;

    /**
     * 预留字段  收银员
     */
    @ApiModelProperty("商户id")
    private String cashier;

    /**
     * 预留字段 设备号
     */
    @ApiModelProperty("收银员")
    private String deviceNo;

    /**
     * 收货地址
     */
    @ApiModelProperty("收货地址")
    private String address;

    /**
     * 物流号
     */
    @ApiModelProperty("物流号")
    private String logisticsNo;

    /**
     * 预留字段 收货地址
     */
    @ApiModelProperty("收地址")
    private String receivePlace;

    /**
     * 统一下单使用的订单号
     */
    private String unifiedOrderNo;

    /**
     * 运费
     */
    @ApiModelProperty("运费")
    private BigDecimal freight = new BigDecimal(0);

    /**
     * 支付时间
     */
    @ApiModelProperty("支付时间")
    private Long payTime;

    @ApiModelProperty("退款状态")
    private String refund;
    /**
     * 折扣金额
     */
    @ApiModelProperty("折扣金额")
    private BigDecimal discount = new BigDecimal(0);
}
