package cn.wise.sc.pay.business.cashier.service;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.goods.service.ISpcDetailService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.Do.OrderGoodsDO;
import cn.wise.sc.pay.domain.order.OrderStatusUtils;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @description: 订单实现类
 * @author: qh
 * @create: 2019-12-04 17:12
 **/
@Service
public class IPaasOrderServiceImpl implements IPaasOrderService {

    @Autowired
    IOrdersService iOrderService;

    @Autowired
    ISpcDetailService iSpcDetailService;

    @Autowired
    IMerchantService merchantService;


    @Override
    public BaseResponse payOrder(String orderNo) {

        if (StringUtils.isEmpty(orderNo)) {
            return BaseResponse.error("订单号不能为空", new Orders());
        }
        Map<String, Object> condition = new HashMap<>();
        condition.put("order_no", orderNo);
        String conditionJson = JSON.toJSONString(condition);
        List<OrderGoodsDO> orderGoodsDOS = (List<OrderGoodsDO>) iOrderService.getOrders(conditionJson).getData();

        ThreadFactory threadFactory = r -> new Thread();

        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(orderGoodsDOS.size(),
                orderGoodsDOS.size(),
                2000,
                TimeUnit.MILLISECONDS,
                new LinkedBlockingDeque(),
                threadFactory);

        for (OrderGoodsDO goodsDO : orderGoodsDOS) {
            threadPoolExecutor.execute(() -> {
                boolean flag = true;
                List<Long> spcIds = goodsDO.getSpcDetailId();
                for (Long spcDetailId : spcIds) {
                    while (flag) {
                        flag = iSpcDetailService.subStore(spcDetailId, goodsDO.getNum()).getData();
                        if (flag == true) {
                            flag = false;
                        }
                    }
                }
            });
        }
        threadPoolExecutor.shutdown();
        boolean isSuccess = iOrderService.changeOrderStatus(orderNo, OrderStatus.COMPLETE, null).getCode() == 200;

        if (isSuccess) {
            return BaseResponse.ok("支付成功", new Orders());
        } else {
            iOrderService.changeOrderStatus(orderNo, OrderStatus.CANCELLED, null);
            return BaseResponse.ok("订单支付失败,请重新下单", new Orders());
        }
    }

    @Override
    public OrderResponse createOrder(OrderDetailQuery orderDetail) {
        return iOrderService.createOrder(orderDetail);
    }

    @Override
    public BaseResponse changeOrderStatus(String orderNo, OrderStatus orderStatus, Map<String, Object> condition) {
        String conditionJson = JSON.toJSONString(condition);
        return iOrderService.changeOrderStatus(orderNo, orderStatus, conditionJson);
    }

    @Override
    public BaseResponse getOrders(Map<String, Object> condition) {
        String conditionJson = JSON.toJSONString(condition);
        return iOrderService.getOrders(conditionJson);
    }

    @Override
    public BaseResponse delOrders(String orderNo, Map<String, Object> condition) {

        String conditionJson = JSON.toJSONString(condition);

        return iOrderService.delOrder(orderNo, conditionJson);
    }

    @Override
    public BaseResponse getPreOrderNo() {
        return iOrderService.getOrderNo();
    }

    @Override
    public BaseResponse editOrder(OrderDetailQuery orderQuery) {
        return iOrderService.editOrder(orderQuery);
    }

    @Override
    public BaseResponse cancelOrder(String orderNo) {
        return iOrderService.cancelOrder(orderNo);
    }

    @Override
    public BaseResponse getEntryOrder(String orderNo) {
        if (StrUtil.isEmpty(orderNo)) {
            return BaseResponse.error("订单号为空", "收银端订单.class");
        }
        Map<String, Object> condition = new HashMap<>();
        condition.put("order_status", OrderStatus.ENTRY_ORDER.name());
        iOrderService.changeOrderStatus(orderNo, OrderStatus.EXIT_ORDER, JSON.toJSONString(new HashMap<>()));
        condition.clear();
        condition.put("order_no", orderNo);
        condition.put("order_status", OrderStatus.EXIT_ORDER.name());
        return iOrderService.getOneOrder(JSON.toJSONString(condition));
    }


    @Override
    public BaseResponse merchantOrderByKeys(Integer pageNO, Integer PageSize, String merchantName, Long startTime, Long endTime) {
        PageQuery pageQuery = new PageQuery();
        PageWrapperQuery pageWrapperQuery = new PageWrapperQuery();
        Map<String, List<Long>> inMap = new HashMap<>(1);
        Map<String, Object> geMap = new HashMap<>(1);
        Map<String, Object> leMap = new HashMap<>(1);

        if (pageNO != null && PageSize != null) {
            pageQuery.setPageNo(pageNO);
            pageQuery.setPageSize(PageSize);
        }
        pageWrapperQuery.setPageQuery(pageQuery);

        if (StringUtils.isNotBlank(merchantName)) {
            List<Long> merchantIdByName = merchantService.getMerchantIdByName(merchantName).data;
            if (merchantIdByName == null || merchantIdByName.size() == 0) {
                return BaseResponse.error("商户不存在", this.getClass().getName());
            }
            inMap.put("seller_id", merchantIdByName);
            pageWrapperQuery.setIn(inMap);
        }

        if (startTime != null) {
            geMap.put("update_time", startTime);
            pageWrapperQuery.setGe(geMap);
        }
        if (endTime != null) {
            leMap.put("update_time", endTime);
            pageWrapperQuery.setLe(leMap);
        }

        OrderResponse page = iOrderService.page(pageWrapperQuery);
        Page<Orders> page1 = page.getPage();
        return page1 == null ? BaseResponse.error("获取数据失败", this.getClass().getName()) : BaseResponse.ok(page1);
    }


    @Override
    public BaseResponse orderDetailsByKeys(Integer pageNO, Integer PageSize, String goodsName, Long startTime, Long endTime, String orderNo) {
        PageWrapperQuery pageWrapperQuery = new PageWrapperQuery();

        Map<String, Object> geMap = new HashMap(1);

        Map<String, Object> leMap = new HashMap(1);

        Map<String, Object> likeMap = new HashMap(1);

        Map<String, Object> eqMap = new HashMap(1);

        PageQuery pageQuery = new PageQuery();
        if (pageNO != null && PageSize != null) {
            pageQuery.setPageSize(PageSize);
            pageQuery.setPageNo(pageNO);
        }
        if (StringUtils.isNotBlank(goodsName)) {
            likeMap.put("goods_name", goodsName);
            pageWrapperQuery.setLike(likeMap);
        }
        if (startTime != null) {
            geMap.put("update_time", startTime);
            pageWrapperQuery.setGe(geMap);
        }
        if (endTime != null) {
            leMap.put("update_time", endTime);
            pageWrapperQuery.setLe(leMap);
        }
        if (StringUtils.isNotBlank(orderNo)) {
            eqMap.put("order_no", orderNo);
            pageWrapperQuery.setEq(eqMap);
        }
        pageWrapperQuery.setPageQuery(pageQuery);
        OrderResponse page = iOrderService.page(pageWrapperQuery);
        return page == null ? BaseResponse.error("数据信息不存在", this.getClass().getName()) : BaseResponse.ok(page);
    }

    @Override
    public BaseResponse orderByNo(String orderNo) {
        if (StringUtils.isBlank(orderNo)) {
            return BaseResponse.error("请传入参数值", this.getClass().getName());
        }
        Map<String, Object> map = new HashMap<>(1);
        map.put("order_no", orderNo);
        String string = JSON.toJSONString(map);
        OrderResponse oneOrder = iOrderService.getOneOrder(string);
        return oneOrder == null ? BaseResponse.error("数据不存在", this.getClass().getName()) : BaseResponse.ok(oneOrder);
    }

    @Override
    public BaseResponse getSellerOrders(int seller, int status, long start, long end) {

        PageWrapperQuery pageWrapperQuery = new PageWrapperQuery();
        Map<String, Object> geMap = new HashMap(1);
        Map<String, Object> leMap = new HashMap(1);

        Map<String, Object> eqMap = new HashMap(2);

        if (seller != 0) {
            eqMap.put("seller_id", seller);
        }
        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        if (orderStatus != OrderStatus.ALL) {
            eqMap.put("order_status", orderStatus.name());
            pageWrapperQuery.setEq(eqMap);
        }
        if (start != 0 && end != 0) {
            geMap.put("update_time", start);
            leMap.put("update_time", end);
            pageWrapperQuery.setGe(geMap);
            pageWrapperQuery.setLe(leMap);
        }

        OrderResponse response = iOrderService.getOrders(pageWrapperQuery);
        Page iPage = new Page();
        iPage.setTotal(response.getList().size());
        iPage.setRecords(response.getList());
        response.setPage(iPage);
        response.setList(null);
        return response;
    }


    @Override
    public BaseResponse getSellerOrdersPage(int seller, int status, long start, long end, PageQuery pageQuery) {
        PageWrapperQuery pageWrapperQuery = new PageWrapperQuery();
        Map<String, Object> geMap = new HashMap(1);
        Map<String, Object> leMap = new HashMap(1);

        Map<String, Object> eqMap = new HashMap(2);

        if (seller != 0) {
            eqMap.put("seller_id", seller);
        }
        OrderStatus orderStatus = OrderStatusUtils.getStatus(status);
        if (orderStatus != OrderStatus.ALL) {
            eqMap.put("order_status", orderStatus.name());

        }
        if (start != 0 && end != 0) {
            geMap.put("create_time", start);
            leMap.put("update_time", end);
        }
        pageWrapperQuery.setEq(eqMap);
        pageWrapperQuery.setGe(geMap);
        pageWrapperQuery.setLe(leMap);
        pageWrapperQuery.setPageQuery(pageQuery);
        OrderResponse response = iOrderService.page(pageWrapperQuery);
        return response;
    }

}
