package cn.wise.sc.pay.business.cashier.util;

import cn.wise.sc.pay.business.goods.model.vo.GoodsExcel;
import cn.wise.sc.pay.business.goods.service.IGoodsService;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.metadata.Sheet;
import com.github.tobato.fastdfs.domain.fdfs.StorePath;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.XSSFPictureData;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: Excel表工具
 * @author: qh
 * @create: 2019-12-24 15:01
 **/
public class GoodsSpecExcelUtils {

    private static final Logger LOGGER = LoggerFactory.getLogger(GoodsSpecExcelUtils.class);


    public static Map<Integer, String> getPictureBytes(InputStream is,FastFileStorageClient client)
            throws IOException, InvalidFormatException {
        Map<Integer, String> map = new HashMap<>();
        XSSFWorkbook workbook = (XSSFWorkbook) WorkbookFactory.create(is);
        List<XSSFPictureData> allPictures = workbook.getAllPictures();
        for (int i = 0; i < allPictures.size(); i++) {
            XSSFPictureData picData = allPictures.get(i);
            InputStream stream = new BufferedInputStream(new ByteArrayInputStream(picData.getData()));
            StorePath storePath = client.uploadFile(stream,picData.getData().length,picData.suggestFileExtension(),null);
            map.put(i+1,storePath.getPath());
        }
        return map;
    }

    public static Boolean readExcel(InputStream is,Integer merchantId, IGoodsService goodsService,
                                    Map<Integer,String> picMap, Class clazz) {
        BufferedInputStream bis = null;
        try {
            bis = new BufferedInputStream(is);
            AnalysisEventListener listener = new ExcelListener(goodsService,merchantId,picMap);
            ExcelReader excelReader = EasyExcelFactory.getReader(bis, listener);
            excelReader.read(new Sheet(1, 1, clazz));
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return true;
    }

}

class ExcelListener extends AnalysisEventListener<GoodsExcel> {

    public ExcelListener(IGoodsService goodsService,Integer merchantId, Map<Integer,String> picMap){
        this.goodsService = goodsService;
        this.picMap = picMap;
        this.merchantId = merchantId;
    }

    IGoodsService goodsService;

    Integer merchantId;

    Map<Integer,String> picMap;

    private static final int BATCH_COUNT = 50;
    //自定义用于暂时存储data。
    //可以通过实例获取该值
    private List<GoodsExcel> datas = new ArrayList<>();

    @Override
    public void invoke(GoodsExcel object, AnalysisContext context) {
        System.out.println("当前行：" + context.getCurrentRowNum());
        //数据存储到list，供批量处理，或后续自己业务逻辑处理。
        object.setCover(picMap.get(context.getCurrentRowNum()));
        object.setSellerId(merchantId);
        if("有".equals(object.getSpc())){
            object.setSpc("1");
        }else{
            object.setSpc("0");
        }
        datas.add(object);
        if (datas.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            datas.clear();
        }
    }
    /**
     * 加上存储数据库
     */
    public void saveData() {
        goodsService.saveGoods(datas);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        saveData();
        datas.clear();//解析结束销毁不用的资源
    }

    public List<GoodsExcel> getDatas() {
        return datas;
    }

    public void setDatas(List<GoodsExcel> datas) {
        this.datas = datas;
    }
}