package cn.wise.sc.pay.api.eshopping.service;

import cn.wise.sc.pay.api.eshopping.service.hystrix.EshopTrolleyServiceImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.query.TrolleyGoodsDetail;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.Min;
import java.util.List;

/**
 * @description: 网上商城购物车
 * @author: qh
 * @create: 2020-03-27 15:46
 **/
@FeignClient(value = "saas-v1-eshopping", path = "/eshop/shoppingTrolley", fallback = EshopTrolleyServiceImpl.class)
public interface IEshopTrolleyService {


    /**
     * 加入购物车
     *
     * @param trolleyGoodsDetail 购物车商品信息
     * @return List<TrolleyGoodsDetail>
     */
    @PostMapping
    BaseResponse<List<TrolleyGoodsDetail>> joinTrolley(@RequestBody TrolleyGoodsDetail trolleyGoodsDetail);

    /**
     * 移除购物车
     *
     * @param goodId 商品id
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @DeleteMapping
    BaseResponse<List<TrolleyGoodsDetail>> removeTrolley(@RequestParam("goodId") @Min(value = 1, message = "无效的商品号") Long goodId,
                                                         @RequestParam("userId") @Min(value = 1, message = "无效的用户id") Long userId,
                                                         @RequestParam("skuId") Long skuId);

    /**
     * 获取购物车里的所有商品
     *
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @GetMapping
    BaseResponse<List<TrolleyGoodsDetail>> getGoods(@RequestParam("userId") @Min(value = 1, message = "无效的用户id") Long userId);

    /**
     * 编辑购物车
     *
     * @param list   购物车信息
     * @param userId 用户id
     * @return List<TrolleyGoodsDetail>
     */
    @PutMapping
    BaseResponse<List<TrolleyGoodsDetail>> edit(@RequestBody List<TrolleyGoodsDetail> list, @RequestParam("userId") Long userId);
}