package cn.wise.sc.pay.business.eshopping.controller;

import cn.wise.sc.pay.business.eshopping.service.IEshopBrandService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopBrand;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopBrandQuery;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.DecimalMin;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@RestController
@Api(tags = "商品品牌")
@RequestMapping("/eshop/brand")
@CrossOrigin
public class EshopBrandController {

    final
    IEshopBrandService eshopBrandService;

    public EshopBrandController(IEshopBrandService eshopBrandService) {
        this.eshopBrandService = eshopBrandService;
    }

    @PostMapping
    @ApiImplicitParams({
            @ApiImplicitParam(value = "名称", name = "name", paramType = "body", dataType = "String", required = true),
            @ApiImplicitParam(value = "logo地址", name = "logo", paramType = "body", dataType = "String")
    })
    public BaseResponse addBrand(@RequestBody EshopBrandQuery eshopBrand){
        return eshopBrandService.addBrand(eshopBrand);
    }

    @PutMapping
    @ApiImplicitParams({
            @ApiImplicitParam(value = "品牌标识", name = "brandId", paramType = "body", dataType = "String", required = true),
            @ApiImplicitParam(value = "名称", name = "name", paramType = "body", dataType = "String", required = true),
            @ApiImplicitParam(value = "logo地址", name = "logo", paramType = "body", dataType = "String")
    })
    public BaseResponse updateBrand(@RequestBody EshopBrandQuery eshopBrand){
        if(eshopBrand.getBrandId() == null){
            return BaseResponse.error("id为空",EshopBrand.class);
        }
        return eshopBrandService.updateBrand(eshopBrand);
    }

    @GetMapping("/brands")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "品牌名称", name = "name", paramType = "query", dataType = "String"),
            @ApiImplicitParam(value = "页码", name = "pageNo", paramType = "query", dataType = "Integer", required = true),
            @ApiImplicitParam(value = "每页条数", name = "pageSize", paramType = "query", dataType = "Integer", required = true)
    })
    public BaseResponse brandPage(String name,
                                  Integer pageNo,
                                  Integer pageSize){
        return eshopBrandService.brandPage(name, pageNo, pageSize);
    }

    @DeleteMapping
    public BaseResponse deleteBrand(Integer id){
        UpdateWrapper wrapper = new UpdateWrapper();
        wrapper.eq("brand_id",id);
        wrapper.set("disabled",1);
        return BaseResponse.ok(eshopBrandService.update(wrapper));
    }
}

