package cn.wise.sc.pay.business.eshopping.controller;

import cn.wise.sc.pay.business.eshopping.service.IEshopGoodsService;
import cn.wise.sc.pay.business.eshopping.service.ILogisticsTemplateService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.eshopping.query.LogisticsTemplateQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;

/**
 * 物流管理模板控制器
 */
@RestController
@RequestMapping("/logisticsTemplate")
@CrossOrigin
@Api(tags = "物流管理模板")
public class LogisticsTemplateController {

    @Autowired
    ILogisticsTemplateService logisticsTemplateService;
    @Autowired
    IEshopGoodsService eshopGoodsService;

    @ApiOperation("新增物流模板记录")
    @PostMapping
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "logisticsTemplateQuery", value = "参数封装对象", dataType = "LogisticsTemplateQuery", paramType = "body")
            }
    )
    public BaseResponse add(@RequestBody LogisticsTemplateQuery logisticsTemplateQuery) {
        BaseResponse response = logisticsTemplateService.add(logisticsTemplateQuery);
        return response;
    }

    @ApiOperation("根据id修改物流模板信息")
    @PutMapping
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "logisticsTemplateQuery", value = "参数封装对象", dataType = "LogisticsTemplateQuery", paramType = "body")
            }
    )
    public BaseResponse edit(@RequestBody LogisticsTemplateQuery logisticsTemplateQuery) {
        BaseResponse response = logisticsTemplateService.edit(logisticsTemplateQuery);
        return response;
    }

    @ApiOperation("根据id获取物流模板信息")
    @GetMapping("/{id}")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "path")
            }
    )
    public BaseResponse getInfoById(@NotNull(message = "id值不能为空") @PathVariable Integer id) {
        BaseResponse response = logisticsTemplateService.getInfoById(id);
        return response;
    }

    @ApiOperation("分页查询物流模板记录")
    @GetMapping("/page")
    @ApiImplicitParams(
            {
            }
    )
    public BaseResponse pageInfo(PageQuery pageQuery) {
        BaseResponse response = logisticsTemplateService.pageInfo(pageQuery);
        return response;
    }

    @ApiOperation("根据id删除物流模板信息")
    @DeleteMapping("/{id}")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "path")
            }
    )
    public BaseResponse deleteById(@NotNull(message = "id值不能为空") @PathVariable Integer id) {
        BaseResponse response = logisticsTemplateService.deleteById(id);
        return response;
    }

    @ApiOperation("根据id删除子物流模板信息")
    @DeleteMapping("/child/{id}")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "int", paramType = "path")
            }
    )
    public BaseResponse deleteChildById(@NotNull(message = "id值不能为空") @PathVariable Integer id) {
        BaseResponse response = logisticsTemplateService.deleteChildById(id);
        return response;
    }

    @ApiOperation("获取物流信息")
    @GetMapping("/logisticsInfo")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "logisticsNo", value = "物流号", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "com", value = "快递平台编号(拼音)", dataType = "String", paramType = "query")
            }
    )
    public BaseResponse getLogisticsInfo(@NotNull(message = "物流号不能为空") @RequestParam("logisticsNo") String logisticsNo,
                                         @NotNull(message = "快递平台") @RequestParam("com") String com) {
        return logisticsTemplateService.getLogisticsInfo(logisticsNo, com);
    }
}
