package cn.wise.sc.pay.business.eshopping.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopGoods;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsQuery;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsSpec;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopGoodsSpecification;
import cn.wise.sc.pay.domain.eshopping.model.vo.GoodsSpecExcel;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
public interface IEshopGoodsService extends IService<EshopGoods> {

    /**
     * 添加商品
     *
     * @param eshopGoodsQuery
     * @return
     */
    BaseResponse addGoods(EshopGoodsQuery eshopGoodsQuery);

    /**
     * 编辑商品
     *
     * @param eshopGoodsQuery
     * @return
     */
    BaseResponse updateGoods(EshopGoodsQuery eshopGoodsQuery);

    /**
     * 删除商品
     *
     * @param id
     * @return
     */
    BaseResponse deleteGoods(Integer id);

    /**
     * 商品分页列表
     *
     * @param name
     * @param marketEnable
     * @param pageNo
     * @param pageSize
     * @return
     */
    BaseResponse goodsPage(String name, Integer marketEnable, Integer pageNo, Integer pageSize);

    /**
     * 更改商品上下架状态，审核状态
     *
     * @param goodsId
     * @param marketEnable
     * @param underMessage
     * @return
     */
    BaseResponse updateStatus(Integer goodsId, Integer marketEnable, String underMessage);

    /**
     * 小程序端商品列表
     *
     * @param categoryId
     * @param price
     * @param multiple
     * @param createTime
     * @param pageNo
     * @param pageSize
     * @return
     */
    BaseResponse appGoodsPage(Integer categoryId, Integer price, Integer createTime, Integer multiple, Integer pageNo, Integer pageSize);

    /**
     * 根据商品id获取商品详情
     *
     * @param goodsId
     * @return
     */
    BaseResponse getGoodsDetail(Integer goodsId);

    /**
     * 获取商品及规格
     *
     * @param goodsList
     * @return
     */
    List<EshopGoodsSpecification> eshopGoodsInfo(List<EshopGoodsSpec> goodsList);

    /**
     * 设置导入商品的模板
     *
     * @return
     */
    List<GoodsSpecExcel> getOneGoodsSpec();

    /**
     * 批量导入商品
     *
     * @param goodsSpecList
     */
    void saveGoods(List<GoodsSpecExcel> goodsSpecList);

    /**
     * 根据id添加商品的评论数量
     * tianjia  商品评论数量
     * @param id
     * @return
     */
    BaseResponse incrCommentNumById(Integer id);

}
