package cn.wise.sc.pay.business.eshopping.service.impl;

import cn.wise.sc.pay.business.eshopping.configure.FastDFSUtils;
import cn.wise.sc.pay.business.eshopping.mapper.EshopBrandMapper;
import cn.wise.sc.pay.business.eshopping.service.IEshopBrandService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopBrand;
import cn.wise.sc.pay.domain.eshopping.model.query.EshopBrandQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@Service
public class EshopBrandServiceImpl extends ServiceImpl<EshopBrandMapper, EshopBrand> implements IEshopBrandService {

    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse addBrand(EshopBrandQuery eshopBrandQuery) {
        EshopBrand eshopBrand = EshopBrand.builder()
                .name(eshopBrandQuery.getName())
                .logo(eshopBrandQuery.getLogo())
                .disabled(1)
                .build();
        this.save(eshopBrand);
        return BaseResponse.ok(eshopBrand);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse updateBrand(EshopBrandQuery eshopBrandQuery) {
        EshopBrand eshopBrand = EshopBrand.builder()
                .brandId(eshopBrandQuery.getBrandId())
                .name(eshopBrandQuery.getName())
                .logo(eshopBrandQuery.getLogo())
                .disabled(1)
                .build();
        this.updateById(eshopBrand);
        return BaseResponse.ok(eshopBrand);
    }

    @Override
    public BaseResponse brandPage(String name, Integer pageNo, Integer pageSize) {
        Page page = new Page(pageNo,pageSize);
        QueryWrapper wrapper = new QueryWrapper();
        if (StringUtils.isNotBlank(name)){
            wrapper.likeLeft("name",name);
        }
        page = (Page) this.page(page, wrapper);
        List<EshopBrand> list = page.getRecords();
        if(list != null && list.size() >0){
            list.stream().forEach(arg->{
                arg.setLogo(fastDFSUtils.fullUrl(arg.getLogo()));
            });
        }
        return BaseResponse.ok(page);
    }
}
