package cn.wise.sc.pay.business.eshopping.service.impl;

import cn.wise.sc.pay.business.eshopping.configure.FastDFSUtils;
import cn.wise.sc.pay.business.eshopping.mapper.EshopGoodsMapper;
import cn.wise.sc.pay.business.eshopping.service.*;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.eshopping.model.bean.*;
import cn.wise.sc.pay.domain.eshopping.model.query.*;
import cn.wise.sc.pay.domain.eshopping.model.vo.EshopGoodsVO;
import cn.wise.sc.pay.domain.eshopping.model.vo.GoodsSpecExcel;
import cn.wise.sc.pay.domain.eshopping.model.vo.LogisticsDetailVo;
import cn.wise.sc.pay.domain.eshopping.model.vo.SpecificationVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@Service
public class EshopGoodsServiceImpl extends ServiceImpl<EshopGoodsMapper, EshopGoods> implements IEshopGoodsService {

    @Autowired
    ISpecificationService specificationService;
    @Autowired
    EshopGoodsMapper goodsMapper;
    @Autowired
    ILogisticsParentService iLogisticsParentService;
    @Autowired
    ILogisticsTemplateService iLogisticsTemplateService;
    @Autowired
    IEshopBrandService brandService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse addGoods(EshopGoodsQuery eshopGoodsQuery) {

        EshopGoods eshopGoods = new EshopGoods();
        BeanUtils.copyProperties(eshopGoodsQuery, eshopGoods);
        //是否上架
        eshopGoods.setMarketEnable(1);
        //是否删除
        eshopGoods.setDisabled(1);
        eshopGoods.setCreateTime(System.currentTimeMillis());
        eshopGoods.setLastModify(System.currentTimeMillis());

        //判断运费模板  没指定默认卖家承担
        if (eshopGoodsQuery.getTemplateId() == null) {
            eshopGoods.setTemplateId(-1);
        }

        this.save(eshopGoods);
        List<SpecificationQuery> spcs = eshopGoodsQuery.getSpcQuery();
        if (eshopGoodsQuery.getHaveSpec() == 1) {
            for (SpecificationQuery spc : spcs) {
                spc.setGoodsId(eshopGoods.getGoodsId());
                specificationService.newSpecification(spc);
            }
        }

        return BaseResponse.ok(eshopGoods);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse updateGoods(EshopGoodsQuery eshopGoodsQuery) {
        EshopGoods eshopGoods = new EshopGoods();
        BeanUtils.copyProperties(eshopGoodsQuery, eshopGoods);
        eshopGoods.setLastModify(System.currentTimeMillis());
        this.updateById(eshopGoods);
        List<SpecificationQuery> spcs = eshopGoodsQuery.getSpcQuery();
        if (eshopGoodsQuery.getHaveSpec() == 1 && spcs == null) {
            return BaseResponse.error("规格有误!", this.getClass().getName());
        }
        if (eshopGoodsQuery.getHaveSpec() == 1 && spcs != null){
            for (SpecificationQuery spc : spcs) {
                specificationService.editSpecification(spc);
            }
        }

        return BaseResponse.ok(eshopGoods);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse deleteGoods(Integer id) {
        if (id != null) {
            UpdateWrapper wrapper = new UpdateWrapper();
            wrapper.eq("goods_id", id);
            wrapper.set("disabled", 0);
            return BaseResponse.ok(this.update(wrapper));
        }
        return BaseResponse.error("id不能为空", EshopGoods.class);
    }

    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    public BaseResponse goodsPage(String name, Integer marketEnable, Integer pageNo, Integer pageSize) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.orderByDesc("last_modify");
        wrapper.eq("disabled", 1);
        if (StringUtils.isNotBlank(name)) {
            wrapper.like("goods_name", name);
        }
        if (marketEnable != null) {
            wrapper.eq("market_enable", marketEnable);
        }
        Page page = new Page<>(pageNo, pageSize);
        page = (Page) this.page(page, wrapper);
        List<EshopGoods> goods = page.getRecords();
        if (goods != null && goods.size() > 0) {
            goods.stream().forEach(arg -> {
                arg.setOriginal(fastDFSUtils.fullUrl(arg.getOriginal()));
            });
        }
        return BaseResponse.ok(page);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse updateStatus(Integer goodsId, Integer marketEnable, String underMessage) {
        UpdateWrapper wrapper = new UpdateWrapper();
        wrapper.eq("goods_id", goodsId);
        wrapper.set("market_enable", marketEnable);
        wrapper.set("under_message", underMessage);
        this.update(wrapper);
        return BaseResponse.ok("状态已改变");
    }

    @Autowired
    IEshopTypeService eshopTypeService;

    @Override
    public BaseResponse appGoodsPage(Integer categoryId, Integer price, Integer createTime, Integer multiple,
                                     Integer pageNo, Integer pageSize) {
        QueryWrapper wrapper = new QueryWrapper();
        Page page = new Page();
        page.setCurrent(pageNo);
        page.setSize(pageSize);
        //父/子分类下所有商品，所以添加条件分类id集合；categoryId只能是一级分类
        List<Integer> typeIds = new ArrayList<>();
        if (categoryId != null) {
            List<EshopTypeTree> types = (List<EshopTypeTree>) eshopTypeService.getGoodsTypeTree().getData();
            List<TreeNode> type = new ArrayList<>();
            types.stream().forEach(arg -> {
                if (arg.getTypeId().equals(categoryId)) {
                    type.add(arg);
                    getTypeIds(type, categoryId, typeIds);
                }
            });
        }
        wrapper.ne("market_enable", 0);
        wrapper.eq("disabled", 1);
        if (typeIds.size() > 0) {
            wrapper.in("category_id", typeIds);
        }
        if (price != null) {
            if (price == 0) {
                wrapper.orderByAsc("mktprice");
            } else {
                wrapper.orderByDesc("mktprice");
            }
        } else if (createTime != null) {
            if (createTime == 0) {
                wrapper.orderByAsc("create_time");
            } else {
                wrapper.orderByDesc("create_time");
            }
        } else if (multiple != null) {
            wrapper.orderByDesc("buy_count");
        }
        page = (Page) this.page(page, wrapper);
        List<EshopGoods> goods = page.getRecords();
        if (goods != null && goods.size() > 0) {
            goods.stream().forEach(arg -> {
                arg.setOriginal(fastDFSUtils.fullUrl(arg.getOriginal()));
            });
        }
        return BaseResponse.ok(page);
    }

    @Override
    public BaseResponse getGoodsDetail(Integer goodsId) {

        EshopGoods good = this.getById(goodsId);
        if (good == null) {
            return BaseResponse.error("没找到当前商品!", EshopGoods.class);
        } else {
            good.setOriginal(fastDFSUtils.fullUrl(good.getOriginal()));
            EshopGoodsVO eshopGoodsVO = new EshopGoodsVO();
            BeanUtils.copyProperties(good, eshopGoodsVO);

            if (good.getTemplateId() != null && good.getTemplateId() != null) {
                //获取当前商品得运费信息
                if (good.getGoodsTransfeeCharge() == 1 && good.getTemplateId() == -1) {
                    eshopGoodsVO.setFreight("免运费");
                } else {
                    BaseResponse response = iLogisticsTemplateService.getInfoById(good.getTemplateId());
                    if (response.getCode() == 400) {
                        return BaseResponse.error("没找到运费模板或绑定模板被删除!", EshopGoods.class);
                    }
                    LogisticsDetailVo logisticsDetailVo = (LogisticsDetailVo) response.getData();
                    if (logisticsDetailVo.getLogisticsTemplates() == null || logisticsDetailVo.getLogisticsTemplates().size() == 0) {
                        return BaseResponse.error("没找到运费模板或绑定模板被删除!", EshopGoods.class);
                    }
                    LogisticsTemplate logisticsTemplate = logisticsDetailVo.getLogisticsTemplates().get(0);
                    //按重量
                    if (logisticsDetailVo.getFeeWay() == 1 && good.getWeight() != null && good.getWeight() != 0) {
                        Double goodWeight = good.getWeight();
                        int continueCount = (new BigDecimal(goodWeight).subtract(new BigDecimal(logisticsTemplate.getFirstWeight()))
                                .divide(new BigDecimal(logisticsTemplate.getContinueWeight()), 0, RoundingMode.UP)).intValue();
                        eshopGoodsVO.setFreight(continueCount * logisticsTemplate.getContinueFee() + logisticsTemplate.getFee() + "");
                    } else {//件数
                        eshopGoodsVO.setFreight(logisticsTemplate.getFee() + "");
                    }
                }
            }
            //当有规格时
            if (good.getHaveSpec() == 1) {
                BaseResponse spcResponse = specificationService.getGoodsSpc(goodsId);
                if (spcResponse.getCode() == 200) {
                    eshopGoodsVO.setSku((List<SpecificationVO>) spcResponse.getData());
                } else {
                    return BaseResponse.error("没找到当前商品得规格!", EshopGoods.class);
                }
            }
            return BaseResponse.ok(eshopGoodsVO);
        }
    }

    public void getTypeIds(List<TreeNode> types, Integer categoryId, List<Integer> typeIds) {
        if (types == null || types.size() == 0) {
            return;
        } else {
            types.stream().forEach(arg -> {
                typeIds.add(arg.getId());
                getTypeIds(arg.getChildren(), categoryId, typeIds);
            });
        }
    }

    @Override
    public List<EshopGoodsSpecification> eshopGoodsInfo(List<EshopGoodsSpec> goodsList) {
        if (goodsList == null || goodsList.size() == 0) {
            return new ArrayList<>();
        }
        return goodsMapper.goodsSpecList(goodsList);
    }

    @Override
    public List<GoodsSpecExcel> getOneGoodsSpec() {
        GoodsSpecExcel goodsSpecExcel = GoodsSpecExcel.builder()
                .goodsName("时尚风衣")
                .brandName("play-boy")
                .categoryName("男装--衣服")
                .weight(12.0)
                .descr("you not know")
                .price(120.0)
                .cost(100.0)
                .mktprice(150.0)
                .havaSpec("有")
                .quantity(100)
                .templateName("运费模板")
                .goodsTransfeeCharge("买家承担运费")
                .name("颜色")
                .descr("红色-L")
                .specPrice(160)
                .specWeight(1.2)
                .store(50)
                .build();
        List<GoodsSpecExcel> list = new ArrayList<>();
        list.add(goodsSpecExcel);
        return list;
    }

    @Override
    public void saveGoods(List<GoodsSpecExcel> goodsSpecList) {
        //品牌列表
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("disabled", 1);
        List<EshopBrand> brands = brandService.list(wrapper);
        //分类列表
        QueryWrapper categoryWrapper = new QueryWrapper();
        categoryWrapper.eq("parent_id", 0);
        List<EshopType> types = eshopTypeService.list(categoryWrapper);
        //运费模板列表
        List<LogisticsParent> list = iLogisticsParentService.list();

//        List<EshopGoods> goods = new ArrayList<>();
        for (GoodsSpecExcel excelRow : goodsSpecList) {
            EshopGoods arg = new EshopGoods();
            BeanUtils.copyProperties(excelRow, arg);
            //设置品牌id
            for (EshopBrand brand : brands) {
                if (brand.getName().equals(excelRow.getBrandName())) {
                    arg.setBrandId(brand.getBrandId());
                    break;
                }
            }
            //设置分类id
            for (EshopType type : types) {
                if (type.getTypeName().equals(excelRow.getCategoryName())) {
                    arg.setCategoryId(type.getTypeId());
                    break;
                }
            }
            //设置运费模板id
            for (LogisticsParent parent : list) {
                if (parent.getName().equals(excelRow.getTemplateName())) {
                    arg.setTemplateId(parent.getId());
                    break;
                }
            }
            if ("买家承担运费".equals(excelRow.getGoodsTransfeeCharge())) {
                arg.setGoodsTransfeeCharge(0);
            } else {
                arg.setGoodsTransfeeCharge(1);
            }
            arg.setCreateTime(System.currentTimeMillis());
            arg.setLastModify(System.currentTimeMillis());
            arg.setDisabled(1);
            arg.setMarketEnable(1);
            arg.setOriginal("M00/00/02/rBEAEF8j00-AdIcKAAALfVm4HKQ084.png");
            if ("有".equals(excelRow.getHavaSpec())) {
                arg.setHaveSpec(1);
                this.save(arg);
                Integer goodsId = arg.getGoodsId();
                Specification specification = Specification.builder()
                        .name(excelRow.getName())
                        .descr(excelRow.getDescr())
                        .price(excelRow.getSpecPrice())
                        .store(excelRow.getStore())
                        .weight(excelRow.getSpecWeight())
                        .goodsId(goodsId.longValue())
                        .status(1)
                        .build();
                specificationService.save(specification);
            } else {
                arg.setHaveSpec(0);
                this.save(arg);
            }
        }
    }


    @Override
    /**
     * tianjia  商品评论数量
     */
    public BaseResponse incrCommentNumById(Integer id) {
        if (id == null || id < 0) {
            return BaseResponse.error("参数错误", this.getClass().getName());
        }
        EshopGoods eshopGoods = this.getById(id);
        if (eshopGoods == null) {
            return BaseResponse.error("商品不存在", this.getClass().getName());
        }
        if (eshopGoods.getCommentNum() == null) {
            eshopGoods.setCommentNum(0);
        }
        eshopGoods.setCommentNum(eshopGoods.getCommentNum() + 1);
        boolean update = this.updateById(eshopGoods);

        return update ? BaseResponse.ok(eshopGoods) : BaseResponse.error("增加商品评论数量错误", this.getClass().getName());
    }
}
