package cn.wise.sc.pay.business.eshopping.utils;

import cn.wise.sc.pay.business.eshopping.service.IEshopGoodsService;
import cn.wise.sc.pay.domain.eshopping.model.vo.GoodsSpecExcel;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.metadata.Sheet;
import com.github.tobato.fastdfs.domain.fdfs.StorePath;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import org.apache.poi.POIXMLDocumentPart;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.*;
import org.openxmlformats.schemas.drawingml.x2006.spreadsheetDrawing.CTMarker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.*;

/**
 * @description: Excel表工具
 * @author: qh
 * @create: 2019-12-24 15:01
 **/
public class GoodsSpecExcelUtils {

    private static final Logger LOGGER = LoggerFactory.getLogger(GoodsSpecExcelUtils.class);


    public static Map<Integer, String> getPictureBytes(InputStream is,FastFileStorageClient client) throws IOException, InvalidFormatException {
        //todo 应急处理 不严谨
        Map<Integer, String> map = new HashMap<>();
        XSSFWorkbook workbook = (XSSFWorkbook) WorkbookFactory.create(is);
        List<XSSFPictureData> allPictures = workbook.getAllPictures();
        for (int i = 0; i < allPictures.size(); i++) {
            XSSFPictureData picData = allPictures.get(i);
            InputStream stream = new BufferedInputStream(new ByteArrayInputStream(picData.getData()));
            StorePath storePath = client.uploadFile(stream,picData.getData().length,picData.suggestFileExtension(),null);
            map.put(i+1,storePath.getPath());
        }
        //todo 正常逻辑处理
        //获取图片HSSFPictureData集合
//        XSSFSheet sheetAt = workbook.getSheetAt(0);
//        for(XSSFShape shape : sheetAt.getDrawingPatriarch().getShapes()) {
//            XSSFPicture pic = (XSSFPicture) shape;
//            XSSFClientAnchor anchor = pic.getPreferredSize();
//            XSSFPictureData pictureData = pic.getPictureData();
//            byte[] data = pictureData.getData();
//            String ext = pictureData.suggestFileExtension();
//            InputStream stream = new BufferedInputStream(new ByteArrayInputStream(data));
//            StorePath storePath = client.uploadFile(stream,data.length,ext,null);
//            CTMarker ctMarker = anchor.getFrom();
//            map.put(ctMarker.getRow(), storePath.getPath());
//        }
        return map;
    }

    public static Boolean readExcel(InputStream is, IEshopGoodsService goodsService,
                                    Map<Integer,String> picMap, Class clazz) {
        BufferedInputStream bis = null;
        try {
            bis = new BufferedInputStream(is);
            AnalysisEventListener listener = new ExcelListener(goodsService,picMap);
            ExcelReader excelReader = EasyExcelFactory.getReader(bis, listener);
            excelReader.read(new Sheet(1, 1, clazz));
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return true;
    }

}

class ExcelListener extends AnalysisEventListener<GoodsSpecExcel> {

    public ExcelListener(IEshopGoodsService goodsService,Map<Integer,String> picMap){
        this.eshopGoodsService = goodsService;
        this.picMap = picMap;
    }

    IEshopGoodsService eshopGoodsService;

    Map<Integer,String> picMap;

    private static final int BATCH_COUNT = 50;
    //自定义用于暂时存储data。
    //可以通过实例获取该值
    private List<GoodsSpecExcel> datas = new ArrayList<>();

    @Override
    public void invoke(GoodsSpecExcel object, AnalysisContext context) {
        System.out.println("当前行：" + context.getCurrentRowNum());
        datas.add(object);
        if (datas.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            datas.clear();
        }
    }

    /**
     * 这里会一行行的返回头
     *
     * @param headMap
     * @param context
     */
    @Override
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
        System.out.println();
    }
    /**
     * 加上存储数据库
     */
    public void saveData() {
        eshopGoodsService.saveGoods(datas);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        saveData();
        datas.clear();//解析结束销毁不用的资源
    }

    public List<GoodsSpecExcel> getDatas() {
        return datas;
    }

    public void setDatas(List<GoodsSpecExcel> datas) {
        this.datas = datas;
    }
}