package cn.wise.sc.pay.business.exhibition.controller;


import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.exhibition.configure.FastDFSUtils;
import cn.wise.sc.pay.business.exhibition.service.IEntranceQRcodeService;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.exhibition.bean.EntranceQrcode;
import cn.wise.sc.pay.domain.exhibition.query.ExhibitionGroundQuery;
import cn.wise.sc.pay.domain.exhibition.query.ExhibitionQuery;
import cn.wise.sc.pay.domain.exhibition.query.QRCodeGiftQuery;
import cn.wise.sc.pay.domain.exhibition.query.TicketNoticeQuery;
import cn.wise.sc.pay.domain.exhibition.query.TicketQuery;
import cn.wise.sc.pay.domain.exhibition.vo.RQCodeInfoVo;
import cn.wise.sc.pay.domain.exhibition.vo.TicketVO;
import cn.wise.sc.pay.domain.order.model.Orders;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author qh
 * @since 2019-11-18
 */
@Api(description = "展览详情")
@RestController
@RequestMapping("/exhibition")
@CrossOrigin
public class ExhibitionController {

    @Autowired
    IExhibitionService iExhibitionService;
    @Autowired
    IEntranceQRcodeService iEnQRCode;
    @Autowired
    FastDFSUtils fastDFSUtils;
    @Autowired
    IEntranceQRcodeService iEXQRCode;
    @Autowired
    IOrdersService iOrdersService;

    @PostMapping("/new")
    @ApiImplicitParam(name = "exhibitionQuery", value = "展览查询类", paramType = "body", dataType = "ExhibitionQuery")
    @ApiOperation("添加一个新的展览")
    public BaseResponse addExhibition(@RequestBody ExhibitionQuery exhibitionQuery) {

        return iExhibitionService.addExhibition(exhibitionQuery);
    }

    public static void main(String[] args) {

        ExhibitionQuery exhibitionQuery = new ExhibitionQuery();
        exhibitionQuery.setAddress("地址");
        exhibitionQuery.setAdvertisingCover("图片");
        exhibitionQuery.setAdvertisingType(1);
        exhibitionQuery.setCover("图片");
        exhibitionQuery.setDescribe("描述");
        exhibitionQuery.setDiscounPrice(200D);
        exhibitionQuery.setPrice(200D);
        exhibitionQuery.setRefundRate(70D);
        exhibitionQuery.setTitle("标题");
        exhibitionQuery.setTag("标签");

        //场次
        List<ExhibitionGroundQuery> list = new ArrayList<>();
        ExhibitionGroundQuery exhibitionGroundQuery = new ExhibitionGroundQuery();
        exhibitionGroundQuery.setStartTime(System.currentTimeMillis());
        exhibitionGroundQuery.setEndTime(System.currentTimeMillis() + 1000000);
        list.add(exhibitionGroundQuery);
        exhibitionQuery.setExhibitionGroundQueries(list);

        List<TicketQuery> list1 = new ArrayList<>();
        TicketQuery ticketQuery = new TicketQuery();
        ticketQuery.setName("单人");
        ticketQuery.setPrice(200D);
        ticketQuery.setTotalPublish(200);
        ticketQuery.setSingleSold(1);
        list1.add(ticketQuery);
        TicketQuery ticketQuery2 = new TicketQuery();
        ticketQuery2.setName("联人");
        ticketQuery2.setPrice(200D);
        ticketQuery2.setTotalPublish(200);
        ticketQuery2.setSingleSold(1);
        list1.add(ticketQuery2);
        TicketQuery ticketQuery3 = new TicketQuery();
        ticketQuery3.setName("yu单人");
        ticketQuery3.setPrice(200D);
        ticketQuery3.setTotalPublish(200);
        ticketQuery3.setSingleSold(1);
        list1.add(ticketQuery3);
        TicketQuery ticketQuery4 = new TicketQuery();
        ticketQuery4.setName("yu联人");
        ticketQuery4.setPrice(200D);
        ticketQuery4.setTotalPublish(200);
        ticketQuery4.setSingleSold(1);
        list1.add(ticketQuery4);

        exhibitionQuery.setTicketQueries(list1);

        List<TicketNoticeQuery> list2 = new ArrayList<>();
        TicketNoticeQuery ticketNoticeQuery = new TicketNoticeQuery();
        ticketNoticeQuery.setTitle("票务须知");
        ticketNoticeQuery.setDescribe("描述");
        list2.add(ticketNoticeQuery);
        exhibitionQuery.setTicketNoticesQuerys(list2);
        System.out.println(JSON.toJSONString(exhibitionQuery));
    }

    @DeleteMapping("{id}")
    @ApiImplicitParam(name = "id", value = "id", paramType = "ath", dataType = "int")
    @ApiOperation("删除展览")
    public BaseResponse delExhibition(@PathVariable("id") Integer id) {
        return iExhibitionService.delById(id);
    }

    @PutMapping
    @ApiImplicitParam(name = "exhibitionQuery", value = "展览查询类", paramType = "body", dataType = "ExhibitionQuery")
    @ApiOperation("编辑展览")
    public BaseResponse delExhibition(@RequestBody ExhibitionQuery exhibitionQuery) {
        return iExhibitionService.updateExhibition(exhibitionQuery);
    }


    @PostMapping("/upload")
    @ApiOperation("上传图片")
    public BaseResponse upload(MultipartFile file) {

        Map<String, String> map = new HashMap<>();
        String path = "";
        if (file != null && StringUtils.isNotEmpty(file.getOriginalFilename())) {
            try {
                path = fastDFSUtils.uploadFile(file).getPath();
                map.put("host", fastDFSUtils.fullUrl(path));
            } catch (IOException e) {
                return BaseResponse.error("图片上传失败", map);
            }
        }
        map.put("uri", path);
        return BaseResponse.ok(map);
    }

    @PutMapping("/{id}/{type}")
    @ApiOperation("上新/推荐")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "展览id", paramType = "path", dataType = "long"),
            @ApiImplicitParam(name = "type", value = "展示类型 0:上新(默认) 1:推荐", paramType = "path", dataType = "int")
    })
    public BaseResponse editShowType(@PathVariable("id") Long id, @PathVariable("type") Integer type) {

        return iExhibitionService.editShowType(id, type);
    }

    @GetMapping("/list")
    @ApiOperation("获取所有(条件)展览,不分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型(字符串)", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "showType", value = "展示类型 0:上新(默认) 1:推荐", paramType = "query", dataType = "string")
    })
    public BaseResponse getExhibitionList(String type, Integer showType, TimeBucket timeBucket) {

        return iExhibitionService.getExhibitionList(type, showType, timeBucket);
    }

    @GetMapping("/advertising/list")
    @ApiOperation("获取所有(条件)展览广告位,不分页")
    public BaseResponse getAdvertising() {
        return iExhibitionService.getAdvertising();
    }

    @PostMapping("/page")
    @ApiOperation("获取所有(条件)展览,分页")
    public BaseResponse getExhibitionPage(@RequestBody PageWrapperQuery pageQuery) {

        return iExhibitionService.getExhibitionPage(pageQuery);
    }

    @PutMapping("/store/tickets")
    public BaseResponse updateTicketStore(@RequestBody List<TicketVO> ticketVOS) {
        return iExhibitionService.updateStore(ticketVOS);
    }

    @GetMapping("/store/exhibition/{id}")
    public BaseResponse getExhibitionStore(@PathVariable("id") Long id) {
        return iExhibitionService.getExhibitionStore(id);
    }

    @GetMapping("/{id}")
    @ApiOperation("获取展览详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "path", dataType = "long")
    })
    public BaseResponse getExhibitionDetail(@PathVariable("id") long id) {

        return iExhibitionService.getExhibitionDetail(id);
    }


    @PutMapping("/QRCode")
    @ApiOperation("核销入场二维码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "enQCodeJSON", value = "enQCodeJSON", paramType = "query", dataType = "String")
    })
    public BaseResponse entranceCheck(@RequestParam("enQCodeJSON") String enQCodeJSON) {

        if (StringUtils.isEmpty(enQCodeJSON)) {
            return BaseResponse.error("参数异常", new EntranceQrcode());
        }

        return iEnQRCode.scanEntranceQRcode(enQCodeJSON);
    }

    @GetMapping("/QRCode")
    @ApiOperation("获取入场二维码")
    @ApiImplicitParam(name = "orderNo", value = "订单号", paramType = "query", dataType = "String")
    public BaseResponse getQRCode(@RequestParam("orderNo") String orderNo) {
        Orders orders = iOrdersService.getOneOrder(JSON.toJSONString(new HashMap() {
            {
                put("order_no", orderNo);
            }
        })).getData();
        return iEXQRCode.getQRCode(orders);
    }

    @GetMapping("/QRCode/list/{userId}")
    @ApiOperation("获取用户的所有票据信息")
    public BaseResponse<List<RQCodeInfoVo>> getAllQRCode(@PathVariable("userId") String userId) {
        return iEXQRCode.getQRCodeInfoPo(userId);
    }

    @PutMapping("/gift")
    @ApiOperation("票务赠与")
    @ApiImplicitParam(name = "entranceQrcode", value = "原票据信息", paramType = "body", dataTypeClass = QRCodeGiftQuery.class)
    public BaseResponse<EntranceQrcode> shareQRCode(@RequestBody QRCodeGiftQuery qrCodeGiftQuery) {
        return  iEXQRCode.shareQRCode(qrCodeGiftQuery);
    }
}

