package cn.wise.sc.pay.business.exhibition.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.exhibition.mapper.OrderMapper;
import cn.wise.sc.pay.business.exhibition.service.IEntranceQRcodeService;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionGroundService;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionService;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionStoreService;
import cn.wise.sc.pay.business.exhibition.service.IOrderService;
import cn.wise.sc.pay.business.exhibition.service.ITicketService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.exhibition.po.Commodity;
import cn.wise.sc.pay.domain.exhibition.bean.Exhibition;
import cn.wise.sc.pay.domain.exhibition.bean.ExhibitionStore;
import cn.wise.sc.pay.domain.exhibition.bean.Order;
import cn.wise.sc.pay.domain.exhibition.bean.Ticket;
import cn.wise.sc.pay.domain.exhibition.query.GoodsQuery;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * <p>
 * 订单类 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-11-22
 */
@Service
public class ExhibitionOrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {

    @Autowired
    IExhibitionStoreService iExStoreService;
    @Autowired
    ITicketService iTicketService;
    @Autowired
    IExhibitionService iExhibitionService;
    @Autowired
    IOrdersService iOrdersService;
    @Autowired
    IExhibitionGroundService iExhibitionGroundService;
    @Autowired
    IEntranceQRcodeService iEntranceQRcodeService;

    /**
     * @param orderQuery
     * @return
     */
    @Override
    public BaseResponse placeOrder(OrderDetailQuery<GoodsQuery> orderQuery) {

        List<GoodsQuery> goodsQueries = orderQuery.getGoods();

        if (!vaildTickets(orderQuery)) {
            return BaseResponse.error("规格不正确", new Orders());
        }

        List<Long> ticketIds = goodsQueries.stream()
                .map(arg -> arg.getId())
                .collect(Collectors.toList());
        //获取票信息
        QueryWrapper ewTicket = new QueryWrapper();
        ewTicket.in("id", ticketIds);
        List<Ticket> tickets = iTicketService.list(ewTicket);
        //获取展览
        Exhibition exhibition = iExhibitionService.getById(orderQuery.getField());
        if (exhibition == null || exhibition.getStatus() == 0 || exhibition.getDelFlag() == 1) {
            return BaseResponse.error("当前展览已关闭", new Orders());
        }
        //获取库存信息
        for (Ticket ticket : tickets) {
            ExhibitionStore exhibitionStore = iExStoreService.getStoreByTicketId(ticket.getId());
            int bugNum = goodsQueries.stream()
                    .filter(arg -> arg.getId() == ticket.getId())
                    .map(arg -> arg.getNum())
                    .findFirst().get().intValue();
            if (exhibitionStore == null || bugNum > exhibitionStore.getRemaining()) {
                return BaseResponse.error("库存不足:" + ticket.getName(), Exhibition.class);
            }
        }

        //构建订单中商品信息集合   转换成JSON存储
        List<Commodity> commodities = new ArrayList<>(goodsQueries.size());
        for (GoodsQuery goodsQuery : goodsQueries) {
            //关联票价
            Ticket ticket = tickets.stream().filter(arg -> arg.getId().longValue() == goodsQuery.getId()).findFirst().get();
            //判断是否为预售票
            Commodity commodity = Commodity.builder()
                    .title(exhibition.getTitle())
                    .goodsId(goodsQuery.getId())
                    .goodsNum(goodsQuery.getNum())
                    .price(ticket.getPrice())
                    .type(ticket.getType())
                    .address(exhibition.getAddress())
                    .cover(exhibition.getCoverUrl())
                    .validity(ticket.getEndTime())
                    .build();
            commodities.add(commodity);
        }

        //库存充足 可以下单(待支付) 但库存不增减 支付完减
        OrderDetailQuery orderDetailQuery = new OrderDetailQuery();
        BeanUtil.copyProperties(orderQuery, orderDetailQuery);
        orderDetailQuery.setPaas("EXHIBITION");
        orderDetailQuery.setStatus(OrderStatus.WAIT_PAY.name());
        orderDetailQuery.setOrderNo(IdUtil.createSnowflake(1, 1).nextIdStr());
        orderDetailQuery.setFreight(new BigDecimal(0));
        orderDetailQuery.setSellerId(exhibition.getId());
        orderDetailQuery.setGoods(commodities);
        BaseResponse eventResult;
        try {
            eventResult = iOrdersService.createOrder(orderDetailQuery);
        } catch (Exception e) {
            return BaseResponse.error("下单失败,请稍后再试！", new Orders());
        }
        BaseResponse baseResponse = eventResult;

        return baseResponse;
    }


    @Override
    @Transactional
    public BaseResponse payOrder(String orderNo) {

        if (StringUtils.isEmpty(orderNo)) {
            return BaseResponse.error("参数有误", new Orders());
        }

        Map<String, Object> map = new HashMap<>();
        map.put("order_no", orderNo);
        BaseResponse<Orders> result = iOrdersService.getOneOrder(JSON.toJSONString(map));
        Orders order = result.getData();

        if (order == null || !OrderStatus.WAIT_PAY.name().equals(order.getOrderStatus())
        ) {

            return BaseResponse.error("订单已失效", new Orders());
        }

        /*
         * 1.获取库存 判断库存是否充足
         * 2.获取version版本
         * 3.带上version去更新 version+1 库存减
         * 4.如果不成功 补偿
         * */
        List<Commodity> commodities = JSON.parseArray(order.getGoods(), Commodity.class);
        if (commodities == null || commodities.size() == 0) {
            return BaseResponse.error("订单中商品为空!", Exhibition.class);
        }
        AtomicBoolean flag = new AtomicBoolean(true);
        //循环减库存 失败记录失败的商品
        int decStoreFail = -1;
        for (int i = 0; i < commodities.size(); i++) {
            Commodity commodity = commodities.get(i);
            //判断是否达到限购数
            QueryWrapper queryWrapper = new QueryWrapper();
            queryWrapper.eq("ticket_id", commodity.getGoodsId());
            ExhibitionStore exhibitionStore = iExStoreService.getOne(queryWrapper);
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("ticket_type", commodity.getType());
            wrapper.eq("association_id", commodity.getGoodsId());
            wrapper.eq("user_id", order.getUserId());
            int count = iEntranceQRcodeService.count(wrapper);
            if (count + commodity.getGoodsNum() > exhibitionStore.getSingleSold()) {
                return BaseResponse.error("超过允许购买的最大数!", this.getClass());
            }

            BaseResponse response = iExStoreService.decreaseStore(commodity.getGoodsId(), commodity.getGoodsNum());
            if (response.getCode() == 400) {
                decStoreFail = i;
                break;
            }
        }
        //大于-1 证明是失败了 回退库存
        if (decStoreFail != -1) {
            for (int i = 0; i <= decStoreFail; i++) {
                Commodity commodity = commodities.get(i);
                iExStoreService.addStore(commodity.getGoodsId(), commodity.getGoodsNum());
            }
            return BaseResponse.error("支付失败 库存不足:" + commodities.get(decStoreFail).getTitle(),
                    Exhibition.class);
        }
        return BaseResponse.ok("支付成功");
    }

    @Override
    public BaseResponse payOrderSuccess(Orders order) {

        //当前订单状态 已支付
        if (!OrderStatus.CONFIRM.name().equals(order.getOrderStatus())) {
            return BaseResponse.error("当前订单状态不为支付中", Exhibition.class);
        }
        //修改订单状态为待发货  已支付
        OrderDetailQuery orderDetailQuery = OrderDetailQuery.builder()
                .orderNo(order.getOrderNo())
                .payStatus(PayStatus.PAY_YES.name())
                .status(OrderStatus.WAIT_RECEIVE.name())
                .paas("EXHIBITION")
                .build();
        return iOrdersService.editOrder(orderDetailQuery);
    }

    @Override
    public void payOrderFail(String orderNo) {

        Map condition = new HashMap();
        condition.put("order_no", orderNo);
        condition.put("status", OrderStatus.CONFIRM);

        BaseResponse result = iOrdersService.getOneOrder(JSON.toJSONString(condition));

        if (result.getData() == null) {
            return;
        }

        OrderDetailQuery orderDetailQuery = OrderDetailQuery.builder()
                .orderNo(orderNo)
                .status(OrderStatus.CANCELLED.name())
                .payStatus(PayStatus.PAY_ON.name())
                .build();

        iOrdersService.editOrder(orderDetailQuery);
    }

    @Override
    public BaseResponse<Orders> getOrderStatus(String orderNo) {

        if (StrUtil.isEmpty(orderNo)) {

            return BaseResponse.error("参数有误", new Orders());

        } else {

            Map condition = new HashMap();
            condition.put("order_no", orderNo);
            BaseResponse<Orders> result = iOrdersService.getOneOrder(JSON.toJSONString(condition));
            Orders order = result.getData();

            if (order == null) {
                return BaseResponse.error("订单未被创建", new Orders());
            } else {
                return BaseResponse.ok(order);
            }
        }
    }

    @Override
    public BaseResponse getPreOrderNo() {
        return iOrdersService.getOrderNo();
    }

    /**
     * 检查库存和限购
     *
     * @param eStores
     * @param goodsQueries
     * @return
     */
    private boolean checkStore(List<ExhibitionStore> eStores, List<GoodsQuery> goodsQueries) {

        boolean isStoreFull = false;

        //是否库存和限购
        for (GoodsQuery goodsQuery : goodsQueries) {
            ExhibitionStore exStore = eStores.stream()
                    .filter(arg -> arg.getTicketId().longValue() == goodsQuery.getId())
                    .findFirst().get();
            if (exStore.getRemaining() >= goodsQuery.getNum() && exStore.getSingleSold() >= goodsQuery.getNum()) {
                isStoreFull = true;
            }
        }

        return isStoreFull;
    }

    /**
     * 判断参数是否有效
     */
    private boolean vaildTickets(OrderDetailQuery<GoodsQuery> orderQuery) {

        if (StrUtil.isEmpty(orderQuery.getField())) {
            return false;
        }

        List<GoodsQuery> goodsQueries = orderQuery.getGoods();

        if (goodsQueries.size() == 0) {
            return false;
        }

        long nullGoodsId = goodsQueries.stream()
                .filter(arg -> arg.getId() == 0 || arg.getNum() == 0)
                .count();
        return nullGoodsId == 0;
    }

    /**
     * 检查库存
     *
     * @return
     */
    private boolean checkStore(long id, int numm) {

        QueryWrapper ewStore = new QueryWrapper();
        ewStore.eq("ticket_id", id);
        ExhibitionStore exStore = iExStoreService.getOne(ewStore);

        //判断库存和单人限购
        return exStore.getRemaining() >= numm && exStore.getSingleSold() >= numm;
    }

//    public boolean cancelOrder() {
//
//        QueryWrapper ew = new QueryWrapper();
//        ew.ne("order_status", OrderStatus.COMPLETE.name());
//        ew.ne("order_status", OrderStatus.WAITING.name());
//        long time = System.currentTimeMillis() + 5 * 1000;
//        ew.ge("create_time", time);
//
//        long count = iOrdersService.getCount(ew).getData();
//
//        getOrder(ew, count, 2000);
//
//        return false;
//    }

//    /**
//     * 获取订单 分页
//     *
//     * @param ew
//     * @param total
//     * @return
//     */
//    private boolean getOrder(QueryWrapper ew, long total, int size) {
//
//        try {
//            Page page;
//            if (total <= size) {
//                page = new Page(0, size);
//                returnStorePage(page);
//            } else {
//                double getCount = Math.ceil(total / size);
//                for (int i = 0; i < getCount; i++) {
//                    page = new Page(i, size);
//                    PageWrapperQuery pageWrapperQuery = PageWrapperQuery.builder()
//                            .pageQuery(page)
//                            .qw(ew)
//                            .build();
//                    IPage pageNoice = iOrdersService.page(pageWrapperQuery).getPage();
//                    returnStorePage(pageNoice);
//                }
//            }
//        } catch (Exception e) {
//            return false;
//        }
//
//        return true;
//    }
//
//    /**
//     * 分页回退库存
//     *
//     * @param page
//     * @return
//     */
//    private IPage returnStorePage(IPage page) {
//
//        List<Orders> orders = page.getRecords();
//        //更新订单状态->取消
//        List<OrderDetailQuery> ordersCancels = new ArrayList<>(orders.size());
//        Map<Long, Integer> goodsStoreMap = new HashMap<>();
//        //遍历每个订单回退库存
//        for (Orders order : orders) {
//            OrderDetailQuery orderDetailQuery = OrderDetailQuery.builder()
//                    .orderNo(order.getOrderNo())
//                    .status(OrderStatus.CANCELLED.name())
//                    .build();
//
//            ordersCancels.add(orderDetailQuery);
//            List<Commodity> commodities = JSON.parseArray(order.getGoods(), Commodity.class);
//            for (Commodity commodity : commodities) {
//                //组装回退的 HashMap<goodId,store>
//                if (goodsStoreMap.containsKey(commodity.getGoodsId())) {
//                    int count = goodsStoreMap.get(commodity.getGoodsId());
//                    goodsStoreMap.put(commodity.getGoodsId(), count + commodity.getGoodsNum());
//                } else {
//                    goodsStoreMap.put(commodity.getGoodsId(), commodity.getGoodsNum());
//                }
//            }
//        }
//        //回退库存
//        boolean isSuccess = true;
//        for (Long key : goodsStoreMap.keySet()) {
//            if (isSuccess) {
//                isSuccess = returnStore(key, goodsStoreMap.get(key));
//            }
//        }
//        if (isSuccess) {
//            iOrdersService.updateBatch(ordersCancels);
//        }
//
//        return page;
//    }
//
//    /**
//     * 返回库存
//     *
//     * @param id
//     * @param storeSize
//     * @return
//     */
//    private boolean returnStore(long id, int storeSize) {
//
//        ExhibitionStore store = iExStoreService.getById(id);
//        QueryWrapper ew = new QueryWrapper();
//        ew.eq("version", store.getVersion());
//        ExhibitionStore updateStore = ExhibitionStore.builder()
//                .id(store.getId())
//                .remaining(store.getRemaining() + storeSize)
//                .version(store.getVersion() + 1)
//                .build();
//        return iExStoreService.update(updateStore, ew);
//    }

}
