package cn.wise.sc.pay.business.exhibition.service.impl;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.exhibition.bean.ExhibitionType;
import cn.wise.sc.pay.business.exhibition.mapper.ExhibitionTypeMapper;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionTypeService;
import cn.wise.sc.pay.domain.exhibition.query.ExhibitionTypeQuery;
import cn.wise.sc.pay.domain.exhibition.vo.ExhibitionTypeVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-11-18
 */
@Service
public class ExhibitionTypeTypeServiceImpl extends ServiceImpl<ExhibitionTypeMapper, ExhibitionType> implements IExhibitionTypeService {

    @Autowired
    ExhibitionTypeMapper exhibitionTypeMapper;

    @Override
    public BaseResponse insertExhibitionType(ExhibitionTypeQuery exhibitionTypeQuery) {

        ExhibitionType exhibitionType = ExhibitionType
                .builder()
                .name(exhibitionTypeQuery.getName())
                .parentId(exhibitionTypeQuery.getParentId())
                .sort(exhibitionTypeQuery.getSort())
                .build();

        QueryWrapper qw = new QueryWrapper();
        qw.eq("name", exhibitionTypeQuery.getName());
        qw.eq("parent_id", exhibitionTypeQuery.getParentId());

        int count = this.count(qw);
        if (count > 0) {
            return BaseResponse.error("展览类型不存在!", this.getClass().getName());
        }

        boolean isSuccess = this.save(exhibitionType);
        if (isSuccess) {
            return BaseResponse.ok("新增成功!", exhibitionType);
        } else {
            return BaseResponse.ok("添加失败!", exhibitionType);
        }
    }

    @Override
    public BaseResponse<List<ExhibitionTypeVO>> selectExhibitionTypes() {

        List<ExhibitionType> exhibitionTypes = this.list(null);

        List<ExhibitionTypeVO> exhibitionTypeVOS = new ArrayList<>();


        for (ExhibitionType exhibitionType : exhibitionTypes) {

            if (exhibitionType.getParentId() == 0) {
                ExhibitionTypeVO exhibitionTypeVO = ExhibitionTypeVO.builder()
                        .id(exhibitionType.getId())
                        .name(exhibitionType.getName())
                        .parentId(exhibitionType.getParentId())
                        .exhibitionTypeVOS(new ArrayList<>())
                        .build();
                treeExhibitionType(exhibitionTypes, exhibitionTypeVO);

                exhibitionTypeVOS.add(exhibitionTypeVO);
            }
        }

        return BaseResponse.ok(exhibitionTypeVOS);
    }

    @Override
    public BaseResponse delById(Long id) {

        if (id == null) {
            return BaseResponse.error("参数错误", new ExhibitionType());
        }
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("parent_id", id);
        long count = this.count(queryWrapper);
        if (count > 0) {
            return BaseResponse.error("该分类下有子分类", ExhibitionType.class);
        }
        boolean isSuccess = this.removeById(id);
        if (isSuccess) {
            return BaseResponse.ok("删除成功", new ExhibitionType());
        }

        return BaseResponse.error("删除失败", new ExhibitionType());
    }

    @Override
    public BaseResponse eidtExhibitionType(Integer id, ExhibitionTypeQuery exhibitionTypeQuery) {

        if (id == null) {
            return BaseResponse.error("参数错误", new ExhibitionType());
        }
        ExhibitionType exhibitionType = ExhibitionType.builder()
                .sort(exhibitionTypeQuery.getSort())
                .parentId(exhibitionTypeQuery.getParentId())
                .name(exhibitionTypeQuery.getName())
                .id(Integer.toUnsignedLong(id))
                .build();

        boolean isSuccess = this.updateById(exhibitionType);
        if (isSuccess) {
            return BaseResponse.ok("编辑成功", new ExhibitionType());
        }

        return BaseResponse.error("编辑失败", new ExhibitionType());

    }

    /**
     * 递归获取展览类型树
     *
     * @param exhibitionTypes
     * @param parentTypeVo
     */
    private void treeExhibitionType(List<ExhibitionType> exhibitionTypes, ExhibitionTypeVO parentTypeVo) {

        //过滤子类型
        List<ExhibitionType> childTypes = exhibitionTypes
                .stream()
                .filter(arg -> arg.getParentId().longValue() == parentTypeVo.getId())
                .collect(Collectors.toList());
        if (childTypes.size() == 0) {
            return;
        }

        for (ExhibitionType childType : childTypes) {
            ExhibitionTypeVO childTypeVo = new ExhibitionTypeVO();
            childTypeVo.setName(childType.getName());
            childTypeVo.setId(childType.getId());
            childTypeVo.setParentId(childType.getParentId());
            parentTypeVo.exhibitionTypeVOS.add(childTypeVo);
            treeExhibitionType(exhibitionTypes, childTypeVo);
        }
    }

}
