package cn.wise.sc.pay.domain.exhibition.vo;

import cn.wise.sc.pay.domain.exhibition.bean.Exhibition;
import cn.wise.sc.pay.domain.exhibition.bean.ExhibitionStore;
import cn.wise.sc.pay.domain.exhibition.bean.Ticket;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * @description:
 * @author: qh
 * @create: 2019-11-20 10:03
 **/
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class ExhibitionManagerVO implements Serializable {

    private static final long serialVersionUID = -1633214868623867199L;
    /**
     * 展览id
     */
    private Long id;

    /**
     * 分类
     */
    private String type;

    /**
     * 标题
     */
    private String title;

    /**
     * 封面
     */
    private String coverUrl;

    /**
     * 场次时间
     */
    private String grounds;

    /**
     * 地址
     */
    private String address;

    /**
     * 想看人数
     */
    private Integer wantWatch;

    /**
     * 总评论数
     */
    private Integer totalComment;

    /**
     * 详情
     */
    private String describe;

    /**
     * 属性
     */
    private String attribute;
    /**
     * 状态
     */
    private String status;

    private List<TicketVO> ticketVOS;

    public ExhibitionManagerVO(Exhibition exhibition, List<Ticket> tickets, List<ExhibitionStore> exStores) {

        this.id = exhibition.getId();
        this.title = exhibition.getTitle();
        this.address = exhibition.getAddress();
        this.type = exhibition.getType();
        this.coverUrl = exhibition.getCoverUrl();
        this.totalComment = exhibition.getTotalComment();
        this.wantWatch = exhibition.getWantWatch();
        int showType = exhibition.getShowType();
        this.attribute = showType == 0 ? "上新" : "推荐";
        int status = exhibition.getStatus();
        this.describe = exhibition.getDescribe();

        if (status == 0) {
            this.status = "已结束";
        }
        if (status == 1) {
            this.status = "进行中";
        }
        if (status == 2) {
            this.status = "预售中";
        }
        this.grounds = exhibition.getGround();

        List<TicketVO> voList = new ArrayList<>();
        for (Ticket ticket : tickets) {
            ExhibitionStore exhibitionStore = exStores.stream()
                    .filter(arg -> arg.getTicketId().longValue() == ticket.getId())
                    .findFirst().get();
            TicketVO ticketVO = new TicketVO(ticket, exhibitionStore);
            voList.add(ticketVO);
        }
        this.ticketVOS = voList;
    }

}
