package cn.wise.sc.pay.api.merchant.service;

import cn.wise.sc.pay.api.merchant.service.hystrix.MerchantImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.merchant.query.MerchantQuery;
import cn.wise.sc.pay.domain.merchant.vo.StoreVo;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletResponse;
import java.util.List;


/**
 * <p>
 * 点单实体 服务类
 * </p>
 *
 * @author lqk
 * @since 2019-12-25
 */
@FeignClient(value = "saas-v1-merchant", path = "/merchant/", fallback = MerchantImpl.class)
public interface IMerchantService {

    @PostMapping
    BaseResponse loginMerchant(@RequestParam("merchAppid") String merchantAppid, @RequestParam("password") String password);

    @GetMapping
    BaseResponse getDeviceUser(@RequestParam("deviceCode") String deviceCode, @RequestParam("username") String username);

    @PostMapping("getMerchNames")
    BaseResponse getMerchNameByIds(@RequestBody List<Long> merchIds);

    @GetMapping("getMerchNameByIds")
    BaseResponse getMerchantByIds(@RequestParam("ids") List<Long> ids);

    /**
     * 根据商户名称获取id
     *
     * @param merchantName
     * @return
     */
    @GetMapping("name")
    BaseResponse<List<Long>> getMerchantIdByName(@RequestParam("merchantName") String merchantName);

    @GetMapping("getMerchantInfo")
    BaseResponse<Merchant> getMerchantInfo(@RequestParam("id") Long id);

    @GetMapping("like")
    BaseResponse getMerchList(@RequestParam(value = "merchName",required = false) String merchName);

    @PostMapping("addMerchant")
    BaseResponse addMerchant(@RequestBody MerchantQuery merchantQuery);

    @DeleteMapping("deleteMerchant")
    BaseResponse deleteMerchant(@RequestParam("merchantId") Long merchantId);

    @PutMapping("updateMerchant")
    BaseResponse updatemerchant(@RequestBody Merchant merchant);

    @PostMapping("getMerchantPage")
    BaseResponse<Page<Merchant>> getMerchantPage(@RequestParam("status") Integer status,
                                                 @RequestParam("merchantName") String merchantName,
                                                 @RequestParam("pageNo") Integer pageNo,
                                                 @RequestParam("pageSize") Integer pageSize,
                                                 @RequestParam("start") Long start,
                                                 @RequestParam("end") Long end);

    @GetMapping("getMerchOrders")
    public BaseResponse getMerchantOrders(@RequestParam("merchAppid") String merchAppid, @RequestParam("start") long start,
                                          @RequestParam("end") long end, String status);

    @GetMapping("merchStaff")
    public BaseResponse getMerchStaff(@RequestParam("merchId") Long merchId, @RequestParam("username") String username,
                                      @RequestParam("pageNo") Integer pageNo,
                                      @RequestParam("pageSize") Integer pageSize);

    @PostMapping("/userStaff")
    public BaseResponse getMerchantStaff(@RequestParam("merchId") Long merchId, @RequestParam("roleId") Integer roleId,
                                         @RequestParam("username") String username,
                                         @RequestParam("pageNo") Integer pageNo,
                                         @RequestParam("pageSize") Integer pageSize);

    @GetMapping("merchantExport")
    public void exportMerchantList(@RequestParam("status") Integer status, HttpServletResponse response);

    //根据商户id取消商家白名单
    @PutMapping("cancelWhiteList/{id}")
    public BaseResponse cancelWhiteList(@PathVariable("id") Long id);

    //根据商户id添加商家白名单
    @PutMapping("addWhiteList/{id}")
    public BaseResponse addWhiteList(@PathVariable("id") Long id);

    /**
     * 获取店铺详情和额外的点评信息
     *
     * @param id 店铺id
     * @return
     */
    @GetMapping("storeInfo/{id}")
    BaseResponse getStoreInfo(@PathVariable("id") Long id);

    @PostMapping("list")
    BaseResponse list(@RequestBody PageWrapperQuery pageWrapperQuery);
    /**
     * 获取店铺详细信息 分页
     * @param pageWrapperQuery 分页参数 条件
     * @return
     */
    @PostMapping("storeInfo/page")
    BaseResponse getStoreInfoPage(@RequestBody PageWrapperQuery pageWrapperQuery);

    /**
     * 更新店铺详情
     * @param storeVo 店铺信息
     * @return
     */
    @PutMapping("storeInfo")
    BaseResponse updateStore(@RequestBody StoreVo storeVo);

    @PostMapping("/check/pwd")
    BaseResponse<Boolean> checkPwd(@RequestParam("seller") String seller,
                                   @RequestParam("pwd")  String pwd);
}
