package cn.wise.sc.pay.business.merchant.controller;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.merchant.service.IMerchantService;
import cn.wise.sc.pay.business.comment.bean.Comment;
import cn.wise.sc.pay.business.comment.controller.CommonCommentController;
import cn.wise.sc.pay.business.merchant.service.IStoreExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.StoreExtra;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * @description: 店铺评论
 * @author: qh
 * @create: 2020-06-15 09:58
 **/
@CrossOrigin
@RestController
@Api(tags = "评论的回复")
@RequestMapping("/merchant/comment")
public class MerchantCommentController extends CommonCommentController {

    @Autowired
    IMerchantService iMerchantService;
    @Autowired
    IStoreExtraService iStoreExtraService;

    public static void main(String[] args) {
        Comment comment = new Comment();
        comment.setTargetId(91L);
        comment.setComBack1("5");
        comment.setContent("吃吃吃此吃吃吃此迟迟吃吃和!");
        comment.setCriticId(1L);
        comment.setCriticName("评论人");
        List<String> url = new ArrayList<>();
        url.add("图片.png");
        url.add("视频.mp4");
        comment.setSources(url);
        System.out.println(JSON.toJSONString(comment));
    }

    @Override
    @PostMapping("/addComment")
    @ApiOperation("评论")
    @ApiImplicitParam(name = "comment", value = "评论实体", paramType = "body", dataType = "Comment")
    public BaseResponse addComment(@RequestBody Comment comment) {

        if (comment.getTargetId() == 0) {
            return BaseResponse.error("targetId必填!", this.getClass().getName());
        }
        //星级为空 默认5星
        if (StrUtil.isEmpty(comment.getComBack1())) {
            comment.setComBack1("5");
        }
        super.addComment(comment);

        //获取评估的星级
        String starStr = comment.getComBack1();
        int star = Integer.parseInt(starStr);

        //获取店铺已有的星级
        Long targetId = comment.getTargetId();
        QueryWrapper<StoreExtra> qw = new QueryWrapper<>();
        qw.eq("merchant_id", targetId);
        StoreExtra storeExtra = iStoreExtraService.getOne(qw);
        QueryWrapper cqw = new QueryWrapper();
        cqw.eq("target_id", targetId);
        Integer commentCount = super.getCommentCount(cqw);

        double newStar = (commentCount * storeExtra.getStar() + star) / (commentCount + 1);
        storeExtra.setStar(newStar);
        iStoreExtraService.updateById(storeExtra);

        return BaseResponse.ok();
    }


}
