package cn.wise.sc.pay.business.merchant.controller;

import cn.hutool.core.date.DateUtil;
import cn.wise.sc.pay.business.merchant.service.IMerchantProfitService;
import cn.wise.sc.pay.business.merchant.service.IProfitAllocationService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.merchant.bean.MerchantProfit;
import cn.wise.sc.pay.domain.merchant.bean.ProfitAllocation;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;

/**
 * 商户店铺控制器
 */
@RestController
@CrossOrigin
@RequestMapping("/merchant/money")
@Slf4j
public class MerchantProfitController {

    @Autowired
    IMerchantProfitService merchantProfitService;

    @Autowired
    IProfitAllocationService profitAllocationService;

    @GetMapping("/moneyByStrategy")
    @ApiOperation(value = "根据店铺id查询相关分润额")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "merchantId",value = "商户id",paramType = "query",dataType = "Long"),
            @ApiImplicitParam(name = "strategy",value = "今天 1，本月 2，本季度 3，本年 4，自定义时间 5",paramType = "query",dataType = "Integer"),
            @ApiImplicitParam(name = "startTime",value = "自定义其实时间",paramType = "query",dataType = "Long"),
            @ApiImplicitParam(name = "endTime",value = "自定义结束时间",paramType = "query",dataType = "Long")
    })
    public BaseResponse getMoneyByStrategy(Long merchantId,Integer strategy, Long startTime, Long endTime){
        if(strategy == null){
            return BaseResponse.error("参数错误",MerchantProfit.class);
        }
        Long sTime = System.currentTimeMillis();
        if(strategy == 1){
            startTime = DateUtil.beginOfDay(DateUtil.date(sTime)).getTime();
        }else if( strategy == 2){
            startTime = DateUtil.beginOfMonth(DateUtil.date(sTime)).getTime();
        }else if(strategy == 3){
            startTime = DateUtil.beginOfQuarter(DateUtil.date(sTime)).getTime();
        }else if(strategy == 4){
            startTime = DateUtil.beginOfYear(DateUtil.date(sTime)).getTime();
        }else{
        }
        endTime = System.currentTimeMillis();
        QueryWrapper wrapper = new QueryWrapper();
        if(merchantId != null){
            wrapper.eq("merchant_id", merchantId);
        }
        if(startTime == null || endTime == null){
            return BaseResponse.error("参数错误", MerchantProfit.class);
        }
        wrapper.between("create_time",startTime,endTime);
        List<MerchantProfit> profitList = merchantProfitService.list(wrapper);
        BigDecimal aosMoney = new BigDecimal(0);
        BigDecimal cmbMoney = new BigDecimal(0);
        for (MerchantProfit profit : profitList) {
            aosMoney = aosMoney.add(profit.getAosMoney()).setScale(3, BigDecimal.ROUND_HALF_DOWN);
            cmbMoney = cmbMoney.add(profit.getCmbMoney()).setScale(3, BigDecimal.ROUND_HALF_DOWN);
        }
        HashMap<String,Double> result = new HashMap<>();
        result.put("aosMoney",aosMoney.doubleValue());
        result.put("cmbMoney",cmbMoney.doubleValue());
        return BaseResponse.ok(result);
    }


    @PostMapping("/addProfit")
    @ApiOperation(value = "交易成功分润额")
    public BaseResponse addMoneyData(Long merchantId,Long orderId, String payment, BigDecimal money, Long createTime){
        if(merchantId == null || money == null || StringUtils.isBlank(payment)){
            return BaseResponse.error("参数错误",MerchantProfit.class);
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("merch_id",merchantId);
        wrapper.eq("payment",payment);
        wrapper.last("limit 1");
        ProfitAllocation profit = profitAllocationService.getOne(wrapper);
        if(profit == null){
            wrapper = new QueryWrapper();
            wrapper.eq("merch_id", 0);
            wrapper.eq("payMent",payment);
            profit = profitAllocationService.getOne(wrapper);
        }
        if(profit == null){
            return BaseResponse.error("分润配置未设置", MerchantProfit.class);
        }
        BigDecimal aosRate = new BigDecimal(profit.getAosRate()).divide(new BigDecimal(1000));
        BigDecimal cmbRate = new BigDecimal(profit.getCmbRate()).divide(new BigDecimal(1000));
        MerchantProfit merchantProfit = MerchantProfit.builder()
                .aosMoney(aosRate.multiply(money))
                .cmbMoney(cmbRate.multiply(money))
                .merchantId(merchantId)
                .createTime(createTime)
                .orderId(orderId)
                .build();
        Boolean result = merchantProfitService.save(merchantProfit);
        return result ? BaseResponse.ok("添加分润数据成功") : BaseResponse.error("添加分润数据失败",MerchantProfit.class) ;
    }

}
