package cn.wise.sc.pay.business.merchant.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.merchant.service.IMerchantService;
import cn.wise.sc.pay.business.merchant.service.IProfitAllocationService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.merchant.bean.ProfitAllocation;
import cn.wise.sc.pay.domain.merchant.excelvo.ProfitAllocationVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.ArrayList;
import java.util.List;

/**
 * @author lqk
 * @since 2019/12/26
 */
@RestController
@CrossOrigin
@RequestMapping("/merchant/profit/")
@Slf4j
public class ProfitAllocationController {

    @Autowired
    IProfitAllocationService profitAllocationService;

    @Autowired
    IMerchantService merchantService;

    @PostMapping("addOrUpdateProfit")
    public BaseResponse addProfit(@RequestBody ProfitAllocation profit) {
        boolean result = profitAllocationService.saveProfit(profit);
        return result ? BaseResponse.ok("成功") : BaseResponse.error("失败",ProfitAllocation.class);
    }

    @GetMapping("{merchId}")
    public BaseResponse getMerchProfit(@PathVariable("merchId") Long merchId){
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("merch_id",merchId);
        List<ProfitAllocation> list = profitAllocationService.list(wrapper);
        return BaseResponse.ok(list);
    }

    @GetMapping("pageProfit")
    public BaseResponse merchProfitPage(Long merchId, String payment, Integer pageNo,Integer pageSize){
        Page page = new Page(pageNo, pageSize);
        QueryWrapper wrapper = new QueryWrapper();
        if(merchId != null){
            wrapper.eq("merch_id",merchId);
        }else {
            wrapper.ne("merch_id",0);//默认分润设置不计入列表
        }
        if(StringUtils.isNotBlank(payment)){
            wrapper.eq("payment",payment);
        }
        page = (Page) profitAllocationService.page(page,wrapper);
        List<ProfitAllocation> list = page.getRecords();
        List<Long> merchIds = new ArrayList<>();
        List<ProfitAllocationVO> result = new ArrayList<>();
        if(list != null && list.size() > 0){
            list.stream().forEach(arg->{
                merchIds.add(arg.getMerchId());
            });
            wrapper = new QueryWrapper();
            wrapper.in("id",merchIds);
            List<Merchant> merchants = merchantService.list(wrapper);
            list.stream().forEach(arg->{
                merchants.stream().forEach(para->{
                    if(arg.getMerchId().longValue() == para.getId().longValue()){
                        ProfitAllocationVO vo = new ProfitAllocationVO();
                        BeanUtil.copyProperties(arg,vo);
                        vo.setMerchName(para.getMerchName());
                        result.add(vo);
                    }
                });
            });
            page.setRecords(result);
        }
        return BaseResponse.ok(page);
    }

    @GetMapping("getPageProfit")
    public BaseResponse getPageProfit(Long merchId, String payment, PageQuery pageQuery){
        Page<List<ProfitAllocationVO>> page = new Page();
        if(pageQuery != null){
            page.setCurrent(pageQuery.getPageNo());
            page.setSize(pageQuery.getPageSize());
        }
        return BaseResponse.ok(profitAllocationService.page(page,merchId,payment));
    }

    @PostMapping("addProfit")
    public BaseResponse addProfitConfig(@RequestBody List<ProfitAllocation> profits){
        if(profits == null || profits.size() == 0){
            return BaseResponse.error("参数错误",ProfitAllocation.class);
        }
        boolean result = profitAllocationService.saveProfits(profits);
        return result ? BaseResponse.ok(profits) : BaseResponse.error("配置已存在,新增失败！", ProfitAllocation.class);
    }

    @DeleteMapping("deleteProfit")
    public BaseResponse deleteProfit(Long id){
        if(id == null){
            return BaseResponse.error("参数错误",ProfitAllocation.class);
        }
        boolean result = profitAllocationService.removeById(id);
        return result ? BaseResponse.ok("删除成功") : BaseResponse.error("删除失败",ProfitAllocation.class);
    }

    public BaseResponse getProfitByName(String shopName){
        QueryWrapper wrapper = new QueryWrapper();
//        if(StringUtils.isBlank(shopName)){
//            wrapper.eq("")
//        }
        return BaseResponse.ok(profitAllocationService.list(wrapper));
    }

}
