package cn.wise.sc.pay.business.merchant.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.business.merchant.configure.FastDFSUtils;
import cn.wise.sc.pay.business.merchant.mapper.MerchantMapper;
import cn.wise.sc.pay.business.merchant.service.IDeviceService;
import cn.wise.sc.pay.business.merchant.service.IMerchantService;
import cn.wise.sc.pay.business.merchant.service.IStoreExtraService;
import cn.wise.sc.pay.business.merchant.utils.AosMerchantUtil;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.common.core.util.DateTimeUtils;
import cn.wise.sc.pay.domain.merchant.bean.Device;
import cn.wise.sc.pay.domain.merchant.bean.Merchant;
import cn.wise.sc.pay.domain.merchant.bean.StoreExtra;
import cn.wise.sc.pay.domain.merchant.query.MerchantQuery;
import cn.wise.sc.pay.domain.merchant.vo.StoreVo;
import cn.wise.sc.pay.domain.merchant.vo.UserMerchantVo;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.laas.userbac.business.service.UserService;
import cn.wise.sc.pay.laas.userbac.domain.dto.UserDTO;
import cn.wise.sc.pay.laas.userbac.domain.entity.SysUser;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lqk
 * @since 2019-11-19
 */
@Service
public class MerchantServiceImpl extends ServiceImpl<MerchantMapper, Merchant>
        implements IMerchantService {

    @Autowired
    IOrdersService iOrdersService;

    @Autowired
    UserService userService;

    @Autowired
    IDeviceService deviceService;
    @Autowired
    IStoreExtraService iStoreExtraService;
    @Autowired
    MerchantMapper merchantMapper;
    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse registerMerchant(MerchantQuery merchantQuery) {
        if (StringUtils.isBlank(merchantQuery.getMerchAddress())) {
            return BaseResponse.error("请添加地址", Merchant.class);
        }
        if (StringUtils.isBlank(merchantQuery.getMerchPhone())) {
            return BaseResponse.error("请添加电话", Merchant.class);
        }
        if (merchantQuery.getStaffNum() == null) {
            return BaseResponse.error("请添加员工数量", Merchant.class);
        }
        if (merchantQuery.getRegisterMoney() == null) {
            return BaseResponse.error("请添加注册资金", Merchant.class);
        }
        if (StringUtils.isBlank(merchantQuery.getLinkPerson())) {
            return BaseResponse.error("请添加联系名称", Merchant.class);
        }
        if (StringUtils.isBlank(merchantQuery.getLinkEmail())) {
            return BaseResponse.error("请添加邮箱", Merchant.class);
        }
        QueryWrapper wrapperQuery = new QueryWrapper();
        wrapperQuery.eq("merch_name", merchantQuery.getMerchName());
        Merchant merchantRes = this.getOne(wrapperQuery);
        if (merchantRes != null) {
            return BaseResponse.error("商户名称已存在！", Merchant.class);
        }
        Merchant merchant = Merchant.builder()
                .merchAppid(AosMerchantUtil.aosMerchAppid())
                .merchName(merchantQuery.getMerchName())
                .merchAddress(merchantQuery.getMerchAddress())
                .described(merchantQuery.getDescribe())
                .merchPhone(merchantQuery.getMerchPhone())
                .linkPerson(merchantQuery.getLinkPerson())
                .linkPhone(merchantQuery.getLinkPhone())
                .linkEmail(merchantQuery.getLinkEmail())
                .staffNum(merchantQuery.getStaffNum())
                .establishTime(merchantQuery.getEstablishTime())
                .registerMoney(merchantQuery.getRegisterMoney())
                .attached(merchantQuery.getAttached())
                .merchRate(merchantQuery.getMerchRate())
                .aosRate(merchantQuery.getAosRate())
                .manageStatus((byte) 0)
                .delStatus((byte) 0)
                .createTime(System.currentTimeMillis())
                .updateTime(System.currentTimeMillis())
                .build();
        merchant.setMerchSecret(AosMerchantUtil.secretEncryption(merchantQuery.getMerchSecret()));
        //设置白名单
        merchant.setWhiteList(0);
        boolean result = this.save(merchant);
        if (result) {
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.orderByDesc("id");
            wrapper.last("limit 1 ");
            merchant = this.getOne(wrapper);
            UserDTO userDTO = new UserDTO();
            userDTO.setUsername(merchant.getMerchAppid());
            List<Integer> roles = new ArrayList<>();
            roles.add(1);
            userDTO.setPassword("123456");
            userDTO.setRole(roles);
            userDTO.setMerchId(merchant.getId());
            userDTO.setLevel("ADMIN");//admin代表商户管理员；可登录商户所有收银端
            result = userService.saveUser(userDTO);
        }

        //添加店铺额外信息
        StoreExtra storeExtra = new StoreExtra();
        storeExtra.setMerchantId(merchant.getId());
        storeExtra.setStar(5D);
        storeExtra.setAvgExpenditure(merchantQuery.getAvgExpenditure());
        storeExtra.setLabel(merchantQuery.getLabel());
        storeExtra.setImg(merchantQuery.getImg());
        storeExtra.setType(merchantQuery.getType());
        storeExtra.setDistance(merchantQuery.getDistance());
        storeExtra.setDetailImgs(merchantQuery.getDetailImgs());
        storeExtra.setOpenTime(merchantQuery.getOpenTime());
        iStoreExtraService.save(storeExtra);

        return result ? BaseResponse.ok("商户添加成功") : BaseResponse.error("商户添加失败", Merchant.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse deleteMerchant(Long merchantId) {
        Merchant merchant = this.getById(merchantId);
        merchant.setDelStatus((byte) 1);
        merchant.setUpdateTime(System.currentTimeMillis());
        boolean result = this.saveOrUpdate(merchant);
        return result ? BaseResponse.ok("成功删除商户") : BaseResponse.error("删除商户失败", Merchant.class);
    }

    @Override
    public BaseResponse updateMerchant(Merchant merchant) {
        merchant.setUpdateTime(System.currentTimeMillis());
        boolean result = this.updateById(merchant);
        return result ? BaseResponse.ok("成功修改商户") : BaseResponse.error("修改商户失败", Merchant.class);
    }

    @Override
    public BaseResponse<Page<Merchant>> getMerchantPage(Integer status, String merchantName,
                                                        PageQuery pageQuery, TimeBucket timeBucket) {
        QueryWrapper<Merchant> wrapper = new QueryWrapper<>();
        if (status != null) {
            wrapper.eq("manage_status", status);
        }
        if (StringUtils.isNotBlank(merchantName)) {
            wrapper.like("merch_name", merchantName);
        }
        if (DateTimeUtils.validTimeBucket(timeBucket)) {
            wrapper.between("create_time", timeBucket.getStartTime(), timeBucket.getEndTime());
        }
        wrapper.ne("del_status", 1);
        Page<Merchant> page = new Page<>();
        page.setSize(pageQuery.getPageSize());
        page.setCurrent(pageQuery.getPageNo());
        page = (Page<Merchant>) this.page(page, wrapper);

        return BaseResponse.ok(page);
    }

    @Override
    public BaseResponse getMerchantOrders(String merchAppid, TimeBucket timeBucket, String status) {
        Map map = new HashMap();
        map.put("seller_id", merchAppid);
        map.put("pay_status", status);
        OrderResponse orderResponse = iOrdersService.getOrders(JSON.toJSONString(map));
        Object obj = orderResponse.getList();
        return BaseResponse.ok(obj);
    }

    @Override
    public BaseResponse loginCahier(String merchantAppid, String password) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("merchAppid", merchantAppid);
        wrapper.eq("merch_secret", AosMerchantUtil.secretEncryption(password));
        Merchant merchant = this.getOne(wrapper);
        if (merchant != null) {
            return BaseResponse.ok(true);
        }
        return BaseResponse.error("登录失败", false);
    }

    @Override
    public BaseResponse<List<Long>> getMerchantIdByName(String merchantName) {
        QueryWrapper<Merchant> merchantQueryWrapper = new QueryWrapper<>();
        merchantQueryWrapper.like(StringUtils.isNotBlank(merchantName), "merch_name", merchantName);
        List<Merchant> list = this.list(merchantQueryWrapper);
        if (list == null && list.size() == 0) {
            return BaseResponse.ok(null);
        }
        List<Long> collect = list.stream().map(s -> s.getId()).collect(Collectors.toList());
        return BaseResponse.ok(collect);
    }

    @Override
    public BaseResponse cancelWhiteList(Long id) {

        Merchant merchantById = this.getById(id);
        if (merchantById == null) {
            return BaseResponse.error("商户不存在", Merchant.class);
        }
        Integer whiteList = merchantById.getWhiteList();
        if (whiteList == 1) {
            UpdateWrapper<Merchant> merchantUpdateWrapper = new UpdateWrapper<>();
            merchantUpdateWrapper.set("white_list", 0).eq("id", id);
            boolean update = this.update(merchantUpdateWrapper);
            return update ? BaseResponse.ok("操作成功") : BaseResponse.error("操作失败", Merchant.class);
        }
        return BaseResponse.error("当前商户不是白名单状态", Merchant.class);
    }

    @Override
    public BaseResponse addWhiteList(Long id) {
        Merchant merchantById = this.getById(id);
        if (merchantById == null) {
            return BaseResponse.error("商户不存在", Merchant.class);
        }
        Integer whiteList = merchantById.getWhiteList();
        if (whiteList == 0) {
            UpdateWrapper<Merchant> merchantUpdateWrapper = new UpdateWrapper<>();
            merchantUpdateWrapper.set("white_list", 1).eq("id", id);
            boolean update = this.update(merchantUpdateWrapper);
            return update ? BaseResponse.ok("操作成功") : BaseResponse.error("操作失败", Merchant.class);
        }
        return BaseResponse.error("当前商户已是白名单状态", Merchant.class);
    }

    @Override
    public BaseResponse userMerchant(PageQuery pageQuery, String username, String shopName) {
        //分页对象
        Page userPage = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
        sysUserQueryWrapper.eq("del_flag", "0");
        if (StringUtils.isNotBlank(username)) {
            sysUserQueryWrapper.like("username", username);
        }
        if (StringUtils.isNotBlank(shopName)) {
            QueryWrapper<Merchant> merchantQueryWrapper = new QueryWrapper<>();
            merchantQueryWrapper.like("merch_name", shopName);
            List<Merchant> list = this.list(merchantQueryWrapper);
            if (list == null || list.size() < 1) {
                return BaseResponse.ok(userPage.setRecords(null));
            }
            List<Long> collect = list.stream().map(s -> s.getId()).collect(Collectors.toList());
            sysUserQueryWrapper.in("merch_id", collect);
        }
        IPage<SysUser> userMerchantPage = userService.page(userPage, sysUserQueryWrapper);
        //用户
        List<SysUser> userList = userMerchantPage.getRecords();
        if (userList == null || userList.size() < 1) {
            return BaseResponse.ok(userPage.setRecords(null));
        }
        List<Long> merchantId = userList.stream().map(u -> u.getMerchId()).collect(Collectors.toList());
        //商户
        Collection<Merchant> merchants = this.listByIds(merchantId);
        //商户
        List<Merchant> merchantList = new ArrayList(merchants);
        //merchantIds
        List<Long> merchantIds = merchantList.stream().map(s -> s.getId()).collect(Collectors.toList());
        QueryWrapper<Device> deviceQueryWrapper = new QueryWrapper<>();
        deviceQueryWrapper.in("merch_id", merchantIds);
        //设备记录
        List<Device> deviceList = deviceService.list(deviceQueryWrapper);
        List<UserMerchantVo> UserMerchantVos = new ArrayList<>();

        List<Merchant> finalMerchantList = merchantList;
        userList.stream().forEach(u -> {
                    UserMerchantVo userMerchantVo = new UserMerchantVo();
                    userMerchantVo.setUsername(u.getUsername());
                    userMerchantVo.setUserPhone(u.getPhone());
                    userMerchantVo.setUserLock(u.getLockFlag());
                    userMerchantVo.setId(u.getUserId());
                    if (finalMerchantList != null && finalMerchantList.size() > 0) {
                        finalMerchantList.stream().forEach(m -> {
                            if (u.getMerchId().equals(m.getId())) {
                                userMerchantVo.setMerchantName(m.getMerchName());
                                userMerchantVo.setMerchantPhone(m.getMerchPhone());
                                userMerchantVo.setMerchantLinkPhone(m.getLinkPhone());
                            }
                        });
                    }
                    if (deviceList != null && deviceList.size() > 0) {
                        List<String> devCodes = new ArrayList<>();
                        List<String> devTypes = new ArrayList<>();
                        deviceList.stream().forEach(d -> {
                            if (u.getMerchId().equals(d.getMerchId())) {
                                devCodes.add(d.getDeviceCode());
                                devTypes.add(d.getTypeName());
                            }
                        });
                        userMerchantVo.setDeviceCodes(devCodes);
                        userMerchantVo.setDeviceTypeNames(devTypes);
                    }
                    UserMerchantVos.add(userMerchantVo);
                }
        );

        Page<UserMerchantVo> userMerchantVoPage = new Page<>();
//        //店铺名过滤
//        if (StringUtils.isNotBlank(shopName)) {
//            List<UserMerchantVo> collect = UserMerchantVos.stream().filter(s -> s.getMerchantName().contains(shopName)).collect(Collectors.toList());
//            userMerchantVoPage.setRecords(collect);
//            userMerchantVoPage.setCurrent(userMerchantPage.getCurrent());
//            userMerchantVoPage.setSize(userMerchantPage.getSize());
//            userMerchantVoPage.setTotal(userMerchantPage.getTotal());
//            return BaseResponse.ok(userMerchantVoPage);
//        }
        userMerchantVoPage.setRecords(UserMerchantVos);
        userMerchantVoPage.setCurrent(userMerchantPage.getCurrent());
        userMerchantVoPage.setSize(userMerchantPage.getSize());
        userMerchantVoPage.setTotal(userMerchantPage.getTotal());
        return BaseResponse.ok(userMerchantVoPage);
    }

    @Override
    /**
     * 获取所有店铺
     * @return
     */
    public BaseResponse<IPage<Merchant>> listMerchant(Integer pageNo, Integer pageSize, String name) {
        if (pageNo == null || pageSize == null || pageNo <= 0 || pageSize <= 0) {
            return BaseResponse.error("参数不正确", this.getClass().getName());
        }
        Page<Merchant> page = new Page<>(pageNo, pageSize);
        QueryWrapper<Merchant> wrapper = new QueryWrapper<>();
        wrapper.eq("del_status", 0);
        //店铺筛选
        if (StringUtils.isNotBlank(name)) {
            wrapper.like("merch_name", name);
        }
        IPage<Merchant> merchantIPage = this.page(page, wrapper);
        return BaseResponse.ok(merchantIPage);
    }

    @Override
    public BaseResponse<StoreVo> getStoreInfo(Long id) {

        if (id == 0) {
            return BaseResponse.error("店铺id是必须的!", StoreVo.class.getName());
        }

        QueryWrapper qw = new QueryWrapper();
        qw.eq("id", id);
        Merchant store = this.getOne(qw);
        assert store != null;
        QueryWrapper qw2 = new QueryWrapper();
        qw2.eq("merchant_id", id);
        StoreExtra storeExtra = iStoreExtraService.getOne(qw2);
//        assert storeExtra != null;
        StoreVo storeVo = new StoreVo();
        BeanUtil.copyProperties(store, storeVo);
        BeanUtil.copyProperties(storeExtra, storeVo, "id");
        if (StrUtil.isNotEmpty(storeExtra.getDetailImgs())) {
            String[] split = storeExtra.getDetailImgs().split("\\|");
            String newDetailImgs = "";
            for (String img : split) {
                newDetailImgs += fastDFSUtils.fullUrl(img) + "|";
            }
            storeVo.setDetailImgs(newDetailImgs);
        }
        storeVo.setImg(fastDFSUtils.fullUrl(storeVo.getImg()));
        return BaseResponse.ok(storeVo);
    }


    @Override
    public BaseResponse getStoreInfoPage(PageWrapperQuery pageWrapperQuery) {

        QueryWrapper qw = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);

        IPage page = new Page(pageWrapperQuery.getPageQuery().getPageNo(), pageWrapperQuery.getPageQuery().getPageSize());

        page = this.page(page, qw);

        List<Merchant> list = page.getRecords();
        List<StoreVo> storeVos = new ArrayList<>(list.size());
        if (list != null && list.size() != 0) {
            List<Long> merchantIds = list.stream().map(arg -> arg.getId()).collect(Collectors.toList());
            QueryWrapper<StoreExtra> qw2 = new QueryWrapper();
            qw2.in("merchant_id", merchantIds);
            List<StoreExtra> storeExtras = iStoreExtraService.list(qw2);

            storeExtras.forEach(arg -> {
                Merchant anElse = list.stream()
                        .filter(merchant -> merchant.getId().longValue() == arg.getMerchantId())
                        .findFirst().orElse(null);

                if (anElse != null) {
                    StoreVo storeVo = new StoreVo();
                    BeanUtil.copyProperties(anElse, storeVo);
                    BeanUtil.copyProperties(arg, storeVo, "id");
                    storeVo.setImg(fastDFSUtils.fullUrl(arg.getImg()));
                    storeVos.add(storeVo);
                }
            });
        }

        return BaseResponse.ok(storeVos);
    }

    @Override
    @Transactional
    public BaseResponse updateStore(StoreVo storeVo) {

        Merchant updateMerchant = new Merchant();
        BeanUtil.copyProperties(storeVo, updateMerchant);

        boolean isSuccess = this.updateById(updateMerchant);
        if (!isSuccess) {
            return BaseResponse.error("更新失败!", this.getClass().getName());
        }
        StoreExtra storeExtra = new StoreExtra();
        BeanUtil.copyProperties(storeVo, storeExtra, "id");
        UpdateWrapper uw = new UpdateWrapper();
        uw.eq("merchant_id", storeVo.getId());
        storeExtra.setMerchantId(storeVo.getId());
        isSuccess = iStoreExtraService.update(storeExtra, uw);
        if (!isSuccess) {
            throw new RuntimeException("更新失败");
        }
        return BaseResponse.okMsg("更新成功", this.getClass());
    }

    @Override
    public BaseResponse<Boolean> checkPwd(String seller, String pwd) {

        QueryWrapper query = new QueryWrapper();
        query.eq("id", seller);
        query.eq("sensitive_pwd", pwd);

        int count = this.count(query);
        if (count > 0) {
            return BaseResponse.ok(true);
        } else {
            return BaseResponse.ok(false);
        }
    }
}
