package cn.wise.sc.pay.api.order.service;

import cn.wise.sc.pay.api.order.service.impl.OrderCallExServiceImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.order.model.OrderCallException;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import java.util.List;

/**
 * @description: 订单调用招行接口异常 服务接口
 * @author: qh
 * @create: 2020-01-13 17:55
 **/
@FeignClient(value = "saas-v1-order", path = "/OrderCallException", fallback = OrderCallExServiceImpl.class)
public interface IOrderCallExService {

    /**
     * 构建一条  异常调用消息记录
     *
     * @param orderNo 回调异常的订单号  必填
     * @return
     */
    @PostMapping
    BaseResponse createExceptionInfo(@RequestParam("orderNo") String orderNo);

    /**
     * 构建多条  异常调用消息记录
     *
     * @param orderCallExceptions 回调异常的订单集合  必填
     * @return
     */
    @PostMapping("/batch")
    BaseResponse createExceptionInfoBatch(@RequestBody List<OrderCallException> orderCallExceptions);


    /**
     * 获取回调异常的所有记录,status为未结束的
     *
     * @param exceptionStatus 异常的订单类型  0:支付异常  1:退款异常  其他:ALL
     * @return
     */
    @GetMapping("/list/{exceptionStatus}")
    BaseResponse getList(@PathVariable("exceptionStatus") Integer exceptionStatus);

    /**
     * 批量修改
     *
     * @param list 招行调用异常对象集合
     * @return
     */
    @PutMapping("/batch")
    BaseResponse editBatchById(@RequestBody List<OrderCallException> list);
}
