package cn.wise.sc.pay.api.order.service;

import cn.wise.sc.pay.api.order.service.impl.SafeguardServiceHystrixImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.model.Safeguard;
import cn.wise.sc.pay.domain.order.query.SafeguardDetailQuery;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * @description: 退款服务
 * @author: qh
 * @create: 2020-03-31 14:57
 */
@FeignClient(
        value = "saas-v1-order",
        path = "/safeguard",
        fallback = SafeguardServiceHystrixImpl.class)
public interface ISafeguardService {

    /**
     * 创建维权订单
     *
     * @param safeguardDetailQuery 维权订单请求描述对象
     * @return BaseResponse
     */
    @PostMapping
    BaseResponse create(@RequestBody SafeguardDetailQuery safeguardDetailQuery);

    /**
     * 维权订单审核
     *
     * @param refundNo 维权订单号
     * @param status   审核状态
     * @param reason   原因
     * @return BaseResponse
     */
    @PutMapping("/audit")
    BaseResponse audit(
            @RequestParam("refundNo") String refundNo,
            @RequestParam("status") int status,
            @RequestParam("reason") String reason);

    /**
     * 维权订单详情
     *
     * @param refundNo 维权订单号
     * @return BaseResponse
     */
    @GetMapping("/{refundNo}")
    BaseResponse refundDetails(@PathVariable("refundNo") String refundNo);

    /**
     * 编辑维权订单
     *
     * @param safeguardDetailQuery 维权订单请求描述
     * @return String
     */
    @PutMapping("/status")
    BaseResponse<String> edit(@RequestBody SafeguardDetailQuery safeguardDetailQuery);

    /**
     * 维权订单列表
     *
     * @param pageWrapperQuery 维权订单请求描述对象
     * @return BaseResponse
     */
    @PostMapping("/page")
    BaseResponse<Safeguard> page(@RequestBody PageWrapperQuery pageWrapperQuery);

    /**
     * 位退款订单关联物流号
     *
     * @param refundNo    退款单号
     * @param LogisticsNo 物流号
     * @return
     */
    @PutMapping("/connectLogisticsNo")
    BaseResponse<Safeguard> connectLogisticsNo(@RequestParam("refundNo") String refundNo,
                                               @RequestParam("LogisticsNo") String LogisticsNo);
}
