package cn.wise.sc.pay.business.order.consumer;

import cn.hutool.core.lang.Assert;
import cn.wise.sc.pay.business.order.consumer.model.EshopGoodsMq;
import cn.wise.sc.pay.business.order.consumer.model.ExhibitionMq;
import cn.wise.sc.pay.business.order.consumer.model.ExhibitionXpenseMq;
import cn.wise.sc.pay.business.order.consumer.model.GoodsMq;
import cn.wise.sc.pay.business.order.service.IXpenseTrackerService;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.XpenseTracker;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
@Slf4j
public class XpenseTrackerListener {

    final
    IXpenseTrackerService iXpenseTrackerService;

    public XpenseTrackerListener(IXpenseTrackerService iXpenseTrackerService) {
        this.iXpenseTrackerService = iXpenseTrackerService;
    }

    /**
     * 监听create_xpense队列 处理完成订单
     *
     * @param content
     */
    @RabbitListener(queues = "create_xpense")
    public void processMessage(String content) {
        Assert.notEmpty(content);
        Orders order = new Orders();
        try {
            order = JSON.parseObject(content, Orders.class);
        } catch (Exception ex) {
            log.error("JSON对象解析失败  =====  XpenseTrackerListener.processMessage()  =====  异常:{}", ex.getMessage());
        }
        List<XpenseTracker> xpenseTrackers = getXpense(order);
        iXpenseTrackerService.createBatch(xpenseTrackers);

    }

    /**
     * 根据订单平台解析商品信息
     *
     * @param order
     * @return
     */
    private List<XpenseTracker> getXpense(Orders order) {
        List<XpenseTracker> xpenseTrackers = new ArrayList<>();
        Assert.notNull(order, "订单为空");
        //订单来自展览
        if ("EXHIBITION".equals(order.getPaas())) {
            List<ExhibitionXpenseMq> list = JSON.parseArray(order.getGoods(), ExhibitionXpenseMq.class);
            for (ExhibitionXpenseMq ex : list
            ) {
                XpenseTracker xpenseTracker = XpenseTracker.builder()
                        .name(order.getPaymentMethodName() + " — " + ex.getTitle() + "(" + ex.getType() + ")")
                        .price(ex.getPrice())
                        .orderNo(order.getOrderNo())
                        .userId(order.getUserId().longValue())
                        .createTime(order.getUpdateTime())
                        .num(ex.getGoodsNum())
                        .build();
                xpenseTrackers.add(xpenseTracker);
            }
        }
        //订单来自收银系统
        if ("cashier".equals(order.getPaas())) {
            List<GoodsMq> list = JSON.parseArray(order.getGoods(), GoodsMq.class);
            for (GoodsMq ex : list
            ) {
                if (ex.getPrice() == null || ex.getTotalPrice() == null) {
                    continue;
                }
                XpenseTracker xpenseTracker = XpenseTracker.builder()
                        .name(order.getPaymentMethodName() + " — " + ex.getName())
                        .userId(order.getUserId().longValue())
                        .createTime(order.getCreateTime())
                        .orderNo(order.getOrderNo())
                        .price(ex.getTotalPrice().doubleValue())
                        .num(ex.getBuyCount())
                        .build();
                xpenseTrackers.add(xpenseTracker);
            }
        }
        if ("ESHOP".equals(order.getPaas())) {
            List<EshopGoodsMq> list = JSON.parseArray(order.getGoods(), EshopGoodsMq.class);
            for (EshopGoodsMq goods : list) {
                XpenseTracker xpenseTracker = XpenseTracker.builder()
                        .name(order.getPaymentMethodName() + " — " + goods.getGoodsName())
                        .userId(order.getUserId().longValue())
                        .createTime(order.getCreateTime())
                        .price(goods.getPrice())
                        .orderNo(order.getOrderNo())
                        .num(goods.getBuyCount())
                        .build();
                xpenseTrackers.add(xpenseTracker);
            }
        }
        return xpenseTrackers;
    }
}
