package cn.wise.sc.pay.business.order.controller;

import cn.wise.sc.pay.business.order.service.IOrderCallExceptionService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.order.OrderCallExStatusUtil;
import cn.wise.sc.pay.domain.order.model.OrderCallException;
import cn.wise.sc.pay.domain.order.model.OrderCallExceptionStatus;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @description: 订单调用第三方异常 控制器
 * @author: qh
 * @create: 2020-01-13 16:58
 **/
@CrossOrigin
@RestController
@RequestMapping("/OrderCallException")
@Api(tags = "OrderCallException", description = "招行支付/退款未响应异常处理")
public class OrderCallExController {

    @Autowired
    IOrderCallExceptionService orderCallExService;

    /**
     * 构建一条  异常调用消息记录
     *
     * @param orderNo 回调异常的订单号  必填
     * @return
     */
    @PostMapping
    public BaseResponse createExceptionInfo(@RequestParam("orderNo") String orderNo) {
        return orderCallExService.createExceptionInfo(orderNo);
    }

    /**
     * 构建多条  异常调用消息记录
     * @param orderCallExceptions 支付异常订单集合
     * @return
     */
    @PostMapping("/batch")
    BaseResponse createExceptionInfoBatch(@RequestBody List<OrderCallException> orderCallExceptions){

        return orderCallExService.createExceptionInfoBatch(orderCallExceptions);
    }

    /**
     * 获取回调异常的所有记录,status为未结束的
     *
     * @param exceptionStatus 异常的订单类型
     * @return
     */
    @GetMapping("/list/{exceptionStatus}")
    public BaseResponse getList(@PathVariable("exceptionStatus") Integer exceptionStatus) {


        OrderCallExceptionStatus callExceptionStatus = OrderCallExStatusUtil.getExceptionStatus(exceptionStatus);

        return orderCallExService.getList(callExceptionStatus);
    }

    /**
     * 批量修改
     *
     * @param list 招行调用异常对象集合
     * @return
     */
    @PutMapping("/batch")
    public BaseResponse editBatchById(@RequestBody List<OrderCallException> list) {
        return orderCallExService.editBatchById(list);
    }
}
