package cn.wise.sc.pay.business.order.model;

import cn.wise.sc.payment.core.model.BaseModel;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * @description: 支付结果通知参数
 * @author: qh
 * @create: 2019-12-18 14:53
 **/
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class PayResultNoticeParams extends BaseModel {

    /**
     * 版本号
     */
    private String version;
    /**
     * 编码方式
     */
    private String encoding;
    /**
     * 签名
     */
    private String sign;
    /**
     * 签名方法 01：RSA2
     */
    private String signMethod;

    /**
     * 商户号 商户在招行完成商户进件后分配的招行商户号
     */
    private String merId;

    /**
     * 商户订单号 商户端生成，要求此订单号在整个商户下唯一
     */
    private String orderId;

    /**
     * 平台订单号  招行生成
     */
    private String cmbOrderId;

    /**
     * 招行系统生成的收银员，需和被查询交易的收银员一致。
     */
    private String userId;

    /**
     * 交易金额 单位分
     */
    private String txnAmt;

    /**
     * 优惠金额 分
     */
    private String dscAmt;

    /**
     * 交易币种
     */
    private String currencyCode;

    /**
     * 目前支持：支付宝/微信/银联
     * ZF：支付宝
     * WX：微信
     * YL：银联
     */
    private String payType;

    /**
     * 支付订单生成的时间，格式为yyyyMMddHHmmss
     */
    private String txnTime;

    /**
     * 优惠信息
     */
    private String couponInfo;

    /**
     * 支付宝支付：用户在支付宝系统的唯一标识
     * 微信支付：商户上送sub_appid时返回用户在sub_appid下的sub_openid
     */
    private String openId;

    /**
     * 第三方系统返回的交易完成日期，此日期仅在交易成功是返回，
     * 不保证是实际清算日期，格式为yyyyMMdd
     */
    private String endDate;

    /**
     * 第三方系统返回的交易完成时间，此时间仅在交易成功是返回，格式为HHmmss
     * 支付宝和微信通道的成功交易返回此字段
     * 银联通道的交易无论成功与否，均不返回此字段
     */
    private String endTime;

    /**
     * 若原交易有上送此字段，则返回此字段
     */
    private String mchReserved;
}
