package cn.wise.sc.pay.business.order.service;

import cn.wise.sc.pay.business.order.service.strategy.IOrderStatisticsStrategy;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.StatisticsGoods;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 点单实体 服务类
 * </p>
 *
 * @author qh
 * @since 2019-12-12
 */
public interface IOrdersService extends IService<Orders> {

    /**
     * 通过统一下单号取消订单
     * @param unOrderNo 统一下单号
     * @return
     */
    boolean cancelStatusByUnOrderNo(List<String> unOrderNo);

    /**
     * 创建订单
     *
     * @param orderDetail 订单描述
     * @return
     */
    OrderResponse createOrder(OrderDetailQuery orderDetail);

    /**
     * 改变订单状态
     *
     * @param orderNo     订单号
     * @param orderStatus 要改变的状态
     * @param condition   关键条件 <field,value> 如:卖家id
     * @return
     */
    OrderResponse changeOrderStatus(String orderNo, OrderStatus orderStatus, Map<String, Object> condition);

    /**
     * 改变订单状态 同时修改用户id
     *
     * @param orderNo     订单号
     * @param orderStatus 要改变的状态
     * @param condition   关键条件 <field,value> 如:卖家id
     * @return
     */
    OrderResponse changeOrderStatusByUserId(String orderNo, String userId, OrderStatus orderStatus, Map<String, Object> condition);


    /**
     * 统计一个时间段的(状态)订单总数
     *
     * @param start    起始时间
     * @param end      结束时间
     * @param status   订单类型
     * @param sellerId 商户id
     * @return
     */
    BaseResponse countOrdersByDate(Long start, Long end, OrderStatus status, String sellerId);

    /**
     * 根据条件查询
     *
     * @param condition 条件 map集合  <field,value>
     * @return
     */
    OrderResponse getOrders(Map<String, Object> condition);

    /**
     * 根据用户id获取订单
     *
     * @param userId 用户id
     * @param status 订单状态
     * @return
     */
    OrderResponse getOrdersByUserId(String userId, OrderStatus status);

    /**
     * 根据key删除
     *
     * @param orderNo   订单号儿
     * @param condition 删除条件
     * @return
     */
    OrderResponse delOrders(String orderNo, Map<String, Object> condition);

    /**
     * 获取Orderid
     *
     * @return
     */
    String getPreOrderNo();

    /**
     * 编辑订单
     *
     * @param orderQuery
     * @return
     */
    OrderResponse editOrder(OrderDetailQuery orderQuery);

    /**
     * 取消订单
     *
     * @param orderNo 订单号
     * @return
     */
    OrderResponse cancelOrder(String orderNo);

    /**
     * 支付订单
     *
     * @param orderNo
     * @return
     */
    OrderResponse payOrder(String orderNo);

    /**
     * 根据条件查询
     *
     * @param conditionJson
     * @return
     */
    OrderResponse getOneOrder(Map<String, Object> conditionJson);

    /**
     * 满足条件的条数
     *
     * @param ew
     * @return
     */
    long getCount(QueryWrapper ew);

    /**
     * 分页查询
     *
     * @param pageWrapperQuery
     * @return
     */
    OrderResponse page(PageWrapperQuery pageWrapperQuery);

    /**
     * 批量更新 更具rderNo
     *
     * @param ordersCancels orderNo必填
     * @return
     */
    boolean updateBatch(List<OrderDetailQuery> ordersCancels);

    /**
     * 获取统计 头部数据
     *
     * @param iOrderStatisticsStrategy
     * @return
     */
    BaseResponse statisticsTop(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end);

    /**
     * 获取统计 折线图
     *
     * @param iOrderStatisticsStrategy
     * @return
     */
    BaseResponse statisticsByChart(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end);

    /**
     * 获取统计笔数 根据支付方式
     *
     * @param iOrderStatisticsStrategy
     * @return
     */
    BaseResponse statisticsCountPayMethod(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end);

    /**
     * 获取统计金额 根据支付方式
     *
     * @param iOrderStatisticsStrategy
     * @return
     */
    BaseResponse statisticsMoneyPayMethod(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end);

    /**
     * 获取统计下的订单
     *
     * @return
     */
    BaseResponse statisticsList(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end, int payment);

    /**
     * 获取集合的复杂查询
     *
     * @param pageWrapperQuery 条件
     * @return
     */
    OrderResponse getListcomplex(PageWrapperQuery pageWrapperQuery);

    /**
     * 获取时间端的聚合支付金额&现金支付的总数
     *
     * @param pageWrapperQuery 查询条件构造 {@link PageWrapperQuery}
     * @return
     */
    BaseResponse statisticsAggregate(PageWrapperQuery pageWrapperQuery);

    /**
     * 统计商品售卖的金额  通过卖家id
     * @param pageWrapperQuery 条件
     * @return
     */
    BaseResponse<List<StatisticsGoods>> statisticsGoodsMarketMoneyGroupSellerGood(PageWrapperQuery pageWrapperQuery);
}
