package cn.wise.sc.pay.business.order.service.impl;

import cn.wise.sc.pay.business.order.mapper.BillsMapper;
import cn.wise.sc.pay.business.order.service.IBillsService;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.order.excel.BillsExcel;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.order.model.Bills;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 账单实体 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-12-24
 */
@Service
public class BillsServiceImpl extends ServiceImpl<BillsMapper, Bills> implements IBillsService {

    private final RestTemplate restTemplate;


    private List<String> datas = new ArrayList<>();

    public BillsServiceImpl(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    @Override
    public BaseResponse readExcelAndInsert(MultipartFile file) {

        BaseResponse baseResponse;
        InputStream inputStream = null;
        try {
            //获取MultipartFile的InputStream
            inputStream = file.getInputStream();

            //读Excel 后续处理交给BillsDataListener对象
            EasyExcel.read(inputStream, BillsExcel.class, new BillsDataListener(this)).sheet().doRead();
            baseResponse = BaseResponse.ok("导入成功", "");
            baseResponse.setClassName(Orders.class.getName());
        } catch (IOException e) {

            baseResponse = BaseResponse.error("对账单导入失败,请重试", "");
            baseResponse.setClassName(Orders.class.getName());
        }
        return baseResponse;
    }

    @Override
    public BaseResponse getBills(PageWrapperQuery queryWrapper) {

        BaseResponse baseResponse;
        try {
            QueryWrapper qw = PageWrapperQueryUtil.getQueryWrapper(queryWrapper);

            List<Bills> bills = this.list(qw);
            baseResponse = BaseResponse.ok(null);
            baseResponse.setClassName(Orders.class.getName());
            baseResponse.setList(bills);

        } catch (Exception ex) {
            baseResponse = BaseResponse.error("获取失败", null);
        }

        return baseResponse;
    }

    @Override
    public BaseResponse getBillsPage(PageWrapperQuery queryWrapper) {

        BaseResponse baseResponse;
        IPage page = new Page(queryWrapper.getPageQuery().getPageNo(), queryWrapper.getPageQuery().getPageSize());

        try {

            QueryWrapper qw = PageWrapperQueryUtil.getQueryWrapper(queryWrapper);
            qw.orderByDesc("transaction_date");
            page = this.page(page, qw);
            baseResponse = BaseResponse.ok(new Bills());
            baseResponse.setClassName(Orders.class.getName());
            baseResponse.setPage((Page) page);

        } catch (Exception ex) {
            baseResponse = BaseResponse.error("获取失败", null);
        }

        return baseResponse;
    }

    /**
     * 从Url解析账单 并导入数据库
     *
     * @param fileUrl
     * @return
     */
    @Override
    public BaseResponse readExcelAndInsert(String fileUrl) {

        BaseResponse baseResponse;
        InputStream inputStream = null;
        try {
            //获取MultipartFile的InputStream
            inputStream = downLoadFromUrl(fileUrl);

            //读Excel 后续处理交给BillsDataListener对象
            EasyExcel.read(inputStream, BillsExcel.class, new BillsDataListener(this)).sheet().doRead();
            baseResponse = BaseResponse.ok("导入成功", "");
            baseResponse.setClassName(Orders.class.getName());
        } catch (IOException e) {

            baseResponse = BaseResponse.error("对账单导入失败,请重试", "");
            baseResponse.setClassName(Orders.class.getName());
        }
        return baseResponse;
    }

    /**
     * 从 URL下载文件
     *
     * @return
     * @throws IOException
     */
    private InputStream downLoadFromUrl(String Url) throws IOException {

        ResponseEntity<byte[]> responseEntity = restTemplate.getForEntity(Url, byte[].class, "");
        byte[] bytes = responseEntity.getBody();
        InputStream inputStream = new ByteArrayInputStream(bytes);

        return inputStream;
    }

}
