package cn.wise.sc.pay.business.order.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.order.mapper.SafeguardMapper;
import cn.wise.sc.pay.business.order.model.GoodUtil;
import cn.wise.sc.pay.business.order.service.IOrdersService;
import cn.wise.sc.pay.business.order.service.ISafeguardService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.order.SafeguardUtil;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.model.Safeguard;
import cn.wise.sc.pay.domain.order.model.SafeguardStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.domain.order.query.SafeguardDetailQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 维权订单  退款订单 服务实现类
 * </p>
 *
 * @author qh
 * @since 2020-02-10
 */
@Service
@Slf4j
public class SafeguardServiceImpl extends ServiceImpl<SafeguardMapper, Safeguard> implements ISafeguardService {

    @Autowired
    IOrdersService iOrdersService;
    @Autowired
    GoodUtil goodUtil;

    @Override
    @Transactional
    public BaseResponse creat(SafeguardDetailQuery safeguardDetailQuery) {

        String orderNo = safeguardDetailQuery.getOrderNo();

        if (StrUtil.isEmpty(orderNo)) {
            return BaseResponse.error("订单号为空", Safeguard.class);
        }
        Map<String, Object> condition = new HashMap<>(1);
        condition.put("order_no", orderNo);
        Orders order = iOrdersService.getOneOrder(condition).getData();
        if (order == null) {
            return BaseResponse.error("订单不存在", Safeguard.class);
        }
        if (PayStatus.PAY_ON.name().equals(order.getPayStatus())) {
            return BaseResponse.error("订单未支付", Safeguard.class);
        }

        //更新订单状态未退款中
        OrderDetailQuery query = new OrderDetailQuery();
        query.setOrderNo(orderNo);
        query.setRefund(OrderStatus.REFUNDING.name());
        iOrdersService.editOrder(query);
        //新增维权订单
        Safeguard safeguard = Safeguard.builder()
                .originalOrder(order.getOrderNo())
                .orderNo(order.getUnifiedOrderNo())
                .refundNo(iOrdersService.getPreOrderNo())
                .refundType(safeguardDetailQuery.getRefundType())
                .reason(safeguardDetailQuery.getReason())
                .amount(order.getPayPrice().doubleValue())
                .status(SafeguardStatus.APPLY.name())
                .createTime(System.currentTimeMillis())
                .updateTime(System.currentTimeMillis())
                .paymentMethodName(order.getPaymentMethodName())
                .goods(order.getGoods())
                .paas(order.getPaas())
                .shopId(order.getSellerId())
                .userId(order.getUserId())
                .evidenceUrl(safeguardDetailQuery.getSafeguardUrl())
                .build();
        try {
            this.save(safeguard);
            return BaseResponse.ok(safeguard);
        } catch (Exception ex) {
            log.error("添加维权订单失败 ===== SafeguardServiceImpl.creat() ===== 异常{}", ex.getMessage());
            return BaseResponse.error("添加失败", Safeguard.class);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse audit(String refundNo, int status, String reason) {
        if (StrUtil.isEmpty(refundNo)) {
            return BaseResponse.error("订单号为空", Safeguard.class);
        }
        if (status != 1 && status != 2) {
            return BaseResponse.error("无权改变其他状态", Safeguard.class);
        }
        //查询数据，判断退款类型
        BaseResponse oneByRefundNo = this.getOneByRefundNo(refundNo);
        if (oneByRefundNo.getCode() == 400) {
            return oneByRefundNo;
        }
        Safeguard refundNoData = (Safeguard) oneByRefundNo.getData();
        if (refundNoData == null) {
            return BaseResponse.error("当前订单号没有数据", SafeguardServiceImpl.class.getName());
        }
        //退款类型
        SafeguardStatus safeguardStatus = SafeguardUtil.getStatus(status);

        //找到当前退款单
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("refund_no", refundNo);
        Safeguard one = this.getOne(queryWrapper);

        //如果拒绝 订单状态为可继续退款
        if (safeguardStatus.equals(SafeguardStatus.AUDIT_NO)) {
            //更新订单状态未退款中
            OrderDetailQuery query = new OrderDetailQuery();
            query.setOrderNo(one.getOriginalOrder());
            query.setRefund(SafeguardStatus.APPLY.name());
            iOrdersService.editOrder(query);
        }
        Safeguard safeguard = Safeguard.builder()
                .status(safeguardStatus.name())
                .remake(reason)
                .build();
        try {
            this.update(safeguard, queryWrapper);
            return BaseResponse.okMsg("操作成功", Safeguard.class);
        } catch (Exception ex) {
            log.error("更新操作失败 ===== SafeguardServiceImpl.audit() ===== 异常: {}", ex
                    .getMessage());
            return BaseResponse.error("审核操作失败", Safeguard.class);
        }
    }

    @Override
    public BaseResponse getOneByRefundNo(String refundNo) {
        if (StrUtil.isEmpty(refundNo)) {
            return BaseResponse.error("订单号为空", Safeguard.class);
        }
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("refund_no", refundNo);
        try {
            Safeguard safeguard = this.getOne(queryWrapper);
            safeguard.setGoods(goodUtil.fullImgUrl(safeguard.getGoods(), safeguard.getPaas()));
            return BaseResponse.ok(safeguard);
        } catch (Exception ex) {
            log.error("查询操作失败 ===== SafeguardServiceImpl.getOneByRefundNo()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("根据退款订单号查询失败", Safeguard.class);
        }
    }

    @Override
    public BaseResponse<Safeguard> page(PageWrapperQuery pageWrapperQuery) {
        IPage<Safeguard> ipage = new Page(pageWrapperQuery.getPageQuery().getPageNo(), pageWrapperQuery.getPageQuery().getPageSize());
        QueryWrapper queryWrapper = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);
        queryWrapper.orderByDesc("create_time");
        try {
            this.page(ipage, queryWrapper);
            BaseResponse baseResponse = BaseResponse.ok(new Safeguard());
            List<Safeguard> list = ipage.getRecords();
            for (Safeguard safeguard : list) {
                safeguard.setGoods(goodUtil.fullImgUrl(safeguard.getGoods(), safeguard.getPaas()));
                if (StrUtil.isNotBlank(safeguard.getEvidenceUrl())){
                    safeguard.setEvidenceUrl(goodUtil.fullUrl(safeguard.getEvidenceUrl()));
                }
            }
            ipage.setRecords(list);
            baseResponse.setPage((Page) ipage);
            return baseResponse;
        } catch (Exception ex) {
            log.error("查询操作失败 ===== SafeguardServiceImpl.page()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("分页查询失败", IPage.class.getName());
        }
    }

    @Override
    public BaseResponse<String> editSafeguard(SafeguardDetailQuery safeguardDetailQuery) {

        Safeguard safeguardUp = new Safeguard();
        BeanUtil.copyProperties(safeguardDetailQuery, safeguardUp);
        safeguardUp.setStatus(SafeguardUtil.getStatus(safeguardDetailQuery.getStatus()).name());
        QueryWrapper<Safeguard> uw = new QueryWrapper<>();
        uw.eq("refund_no", safeguardDetailQuery.getRefundNo());
        boolean result = this.update(safeguardUp, uw);
        if (result) {
            return BaseResponse.okMsg("更新成功!", Safeguard.class);
        } else {
            return BaseResponse.error("更新失败!", Safeguard.class.getName());
        }
    }

    @Override
    public BaseResponse<Safeguard> connectLogisticsNo(String refundNo, String logisticsNo) {

        if (StrUtil.isEmpty(refundNo) || StrUtil.isEmpty(logisticsNo)) {
            return BaseResponse.error("refundNo和logisticsNo是必须的!", Safeguard.class.getName());
        }
        QueryWrapper<Safeguard> qw = new QueryWrapper<>();
        qw.eq("refund_no", refundNo);
        Safeguard safeguard = this.getOne(qw);
        if (safeguard == null) {
            return BaseResponse.error("没有找到当前退款单!", Safeguard.class.getName());
        }
        safeguard.setLogisticsNo(logisticsNo);
        this.updateById(safeguard);
        return BaseResponse.okMsg("关联物流号成功!", Safeguard.class);
    }
}
