package cn.wise.sc.pay.business.order.utils;

import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.codec.binary.Base64;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.nio.charset.StandardCharsets;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @ClassName SignDataUtil
 * @Description :数据加密
 * @Author lqk
 * @Date 2019/12/9
 * @Version V1.0
 **/
public class SignDataUtil {

    private static final String SIGN_TYPE_RSA = "RSA";

    private static final String SIGN_SHA256RSA_ALGORITHMS = "SHA256WithRSA";

    private static final int DEFAULT_BUFFER_SIZE = 8192;

    static char[] hexDigits = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

    public static String bytesToHex(byte[] bytes) {
        int k = 0;
        char[] hexChars = new char[bytes.length * 2];
        for (int i = 0; i < bytes.length; i++) {
            byte byte0 = bytes[i];
            hexChars[k++] = hexDigits[byte0 >>> 4 & 0xf];
            hexChars[k++] = hexDigits[byte0 & 0xf];
        }
        return new String(hexChars);
    }

    public static String rsaSign(Map map, String privateKey){
        try{
            String content = getSignContent(map);
            PrivateKey priKey = getPrivateKeyFromPKCS8(SIGN_TYPE_RSA, new ByteArrayInputStream(privateKey.getBytes()));
            Signature signature = Signature.getInstance(SIGN_SHA256RSA_ALGORITHMS);

            signature.initSign(priKey);
            signature.update(content.getBytes(StandardCharsets.UTF_8));

            byte[] signed = signature.sign();
            return new String(Base64.encodeBase64(signed));
        }catch (Exception e){
            e.printStackTrace();
        }
        return null;
    }

    public static String getApiSign(Map map){
        try {
            String signStr = getSignContent(map);
            System.out.println("signStr" + signStr);
            MessageDigest messageDigest = MessageDigest.getInstance("MD5");
            // 使用指定的字节更新摘要
            messageDigest.update(signStr.getBytes());
            // 得到二进制的密文
            byte[] encryptData = messageDigest.digest();
            // 把密文转换成十六进制的字符串形式
            String encryptDataStr = bytesToHex(encryptData);
            System.out.println("md5 = " + encryptDataStr.toLowerCase());
            return encryptDataStr.toLowerCase();
        }catch (Exception e){
            e.printStackTrace();
        }
        return null;
    }

    public static boolean rsaCheck(Map map, String sign, String publicKey){
        try{
            Signature signature = null;
            String content = getSignContent(map);

            System.out.println(">>验证的签名为:" + sign);
            System.out.println(">>生成签名的参数为:" + content);
            PublicKey pubKey = getPublicKeyFromX509("RSA", new ByteArrayInputStream(publicKey.getBytes()));

            signature = Signature.getInstance(SIGN_SHA256RSA_ALGORITHMS);
            signature.initVerify(pubKey);
            signature.update(content.getBytes(StandardCharsets.UTF_8));

            return signature.verify(Base64.decodeBase64(sign.getBytes()));
        }catch (Exception e){
            e.printStackTrace();
        }
        return false;
    }

    public static String getSignContent(Map<String, String> sortedParams) {
        System.out.println("signData = " + JSON.toJSONString(sortedParams));
        Set<String> keySet = sortedParams.keySet();
        StringBuffer content = new StringBuffer();
        List<String> keys = new ArrayList<String>();
        for (String key : keySet) {
            keys.add(key);
        }
        Collections.sort(keys);

         int index = 0;
         for (int i = 0; i < keys.size(); i++) {
             String key = keys.get(i);
             String value = sortedParams.get(key);
             if (StringUtils.isNotEmpty(key) && StringUtils.isNotEmpty(value)) {
                 content.append((index == 0 ? "" : "&") + key + "=" + value);
                 index++;
             }
         }
         return content.toString();
    }

    public static PrivateKey getPrivateKeyFromPKCS8(String algorithm, InputStream ins) throws Exception {
        if (ins == null || StringUtils.isEmpty(algorithm)) {
             return null;
        }

        KeyFactory keyFactory = KeyFactory.getInstance(algorithm);

        byte[] encodedKey = readText(ins).getBytes();

        encodedKey = Base64.decodeBase64(encodedKey);

        return keyFactory.generatePrivate(new PKCS8EncodedKeySpec(encodedKey));
    }

    public static PublicKey getPublicKeyFromX509(String algorithm, InputStream ins) throws Exception {
        KeyFactory keyFactory = KeyFactory.getInstance(algorithm);

        StringWriter writer = new StringWriter();
        io(new InputStreamReader(ins), writer, -1);

        byte[] encodedKey = writer.toString().getBytes();

        encodedKey = Base64.decodeBase64(encodedKey);

        return keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey));
    }

    private static String readText(InputStream ins) throws IOException {
         Reader reader = new InputStreamReader(ins);
         StringWriter writer = new StringWriter();

         io(reader, writer, -1);
         return writer.toString();
    }

    private static void io(Reader in, Writer out, int bufferSize) throws IOException {
        if (bufferSize == -1) {
             bufferSize = DEFAULT_BUFFER_SIZE >> 1;
        }
        char[] buffer = new char[bufferSize];
        int amount;
        while ((amount = in.read(buffer)) >= 0) {
             out.write(buffer, 0, amount);
        }
    }
}
