package cn.wise.sc.payment.cmb.service;

import cn.wise.sc.payment.cmb.model.WxUser;
import cn.wise.sc.payment.cmb.weichat.method.HttpClientConnectionManager;
import cn.wise.sc.payment.cmb.weichat.method.SSLHttpClient;
import cn.wise.sc.payment.cmb.weichat.model.OAuthAccessToken;
import cn.wise.sc.payment.cmb.weichat.model.UseValue;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;

import java.net.URLEncoder;
import java.security.MessageDigest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Logger;


public class WeiXinDaoImpl implements WeiXinDao {
    public static DefaultHttpClient httpclient;
    private static Logger log = Logger.getLogger(WeiXinDaoImpl.class.getName());


    static {
        httpclient = new DefaultHttpClient();
        // 接受任何证书的浏览器客户端
        httpclient = (DefaultHttpClient) HttpClientConnectionManager.getSSLInstance(httpclient);
    }

    /**
     * 微信OAuth2.0授权（目前微信只支持在微信客户端发送连接，实现授权）
     */
    @Override
    public String getCodeUrl(String appid, String redirect_uri, String scope, String state) throws Exception {
        redirect_uri = URLEncoder.encode(redirect_uri, "utf-8");
        String getCodeUrl = UseValue.getCodeUrl.replace("APPID", appid)
                .replace("REDIRECT_URI", redirect_uri)
                .replace("SCOPE", scope)
                .replace("STATE", state);
        return getCodeUrl;
    }


    /**
     * 微信OAuth2.0授权-获取accessToken(这里通过code换取的网页授权access_token,与基础支持中的access_token不同）
     */
    public synchronized OAuthAccessToken getNomalToken(String appid, String secret) throws Exception {
        String getOAuthAccessToken = UseValue.getTokenUrl
                .replace("APPID", appid)
                .replace("APPSECRET", secret);
        HttpGet get = HttpClientConnectionManager.getGetMethod(getOAuthAccessToken);
        HttpResponse response = httpclient.execute(get);
        String jsonStr = EntityUtils.toString(response.getEntity(), "utf-8");
        Map<String, Object> jsonObject = new ObjectMapper()
                .configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true)
                .readValue(jsonStr, TreeMap.class);
        OAuthAccessToken accessToken = new OAuthAccessToken();
        accessToken.setAccessToken(jsonObject.get("access_token").toString());
        accessToken.setExpiresIn(Integer.valueOf(jsonObject.get("expires_in").toString()));
        return accessToken;
    }

    /**
     * 微信OAuth2.0授权-获取accessToken(这里通过code换取的网页授权access_token,与基础支持中的access_token不同）
     */
    @Override
    public synchronized OAuthAccessToken getOAuthAccessToken(String appid, String secret, String code) throws Exception {
        String getOAuthAccessToken = UseValue.getOAuthAccessToken
                .replace("APPID", appid)
                .replace("SECRET", secret)
                .replace("CODE", code);
        HttpGet get = HttpClientConnectionManager.getGetMethod(getOAuthAccessToken);
        HttpResponse response = httpclient.execute(get);
        HttpEntity data = response.getEntity();
        String jsonStr = EntityUtils.toString(data, "utf-8");
        OAuthAccessToken accessToken = new OAuthAccessToken();
        Map<String, Object> jsonObject = new ObjectMapper().configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true)
                .readValue(jsonStr, TreeMap.class);

        if (jsonObject != null && jsonObject.get("openid") != null) {
            accessToken.setOpenid(jsonObject.get("openid").toString());
            accessToken.setAccessToken(jsonObject.get("access_token").toString());
            accessToken.setExpiresIn(Integer.valueOf(jsonObject.get("expires_in").toString()));
            accessToken.setRefreshToken(jsonObject.get("refresh_token").toString());
            accessToken.setScope(jsonObject.get("scope").toString());
            return accessToken;
        } else {
            return null;
        }
    }

    /**
     * 小程序获取openid
     *
     * @param appid
     * @param secret
     * @param code
     * @return
     * @throws Exception
     */
    @Override
    public synchronized OAuthAccessToken getOpenIdByCode(String appid, String secret, String code) throws Exception {
        String getOAuthAccessToken = UseValue.getJscode2Session
                .replace("APPID", appid)
                .replace("SECRET", secret)
                .replace("CODE", code);
        HttpGet get = HttpClientConnectionManager.getGetMethod(getOAuthAccessToken);
        HttpResponse response = httpclient.execute(get);
        HttpEntity data = response.getEntity();
        String jsonStr = EntityUtils.toString(data, "utf-8");
        OAuthAccessToken accessToken = new OAuthAccessToken();
        Map<String, Object> jsonObject = new ObjectMapper().configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true)
                .readValue(jsonStr, TreeMap.class);

        if (jsonObject != null && jsonObject.get("openid") != null) {
            accessToken.setOpenid(jsonObject.get("openid").toString());
            if (jsonObject.get("session_key") != null) {
                accessToken.setSessionKey(jsonObject.get("session_key").toString());
            }
            return accessToken;
        } else {
            return null;
        }
    }

    /**
     * 微信OAuth2.0授权-刷新access_token（如果需要）
     */
    @Override
    public synchronized OAuthAccessToken refershOAuthAccessToken(String appid, String refresh_token) throws Exception {
        String getreferAccessUrl = UseValue.getreferAccessUrl
                .replace("APPID", appid)
                .replace("REFRESH_TOKEN", refresh_token);
        HttpGet get = HttpClientConnectionManager.getGetMethod(getreferAccessUrl);
        HttpResponse response = httpclient.execute(get);
        String jsonStr = EntityUtils.toString(response.getEntity(), "utf-8");
        Map<String, Object> jsonObject = new ObjectMapper()
                .configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true)
                .readValue(jsonStr, TreeMap.class);
        OAuthAccessToken accessToken = new OAuthAccessToken();
        accessToken.setAccessToken(jsonObject.get("access_token").toString());
        accessToken.setExpiresIn(Integer.valueOf(jsonObject.get("expires_in").toString()));
        accessToken.setRefreshToken(jsonObject.get("refresh_token").toString());
        accessToken.setOpenid(jsonObject.get("openid").toString());
        accessToken.setScope(jsonObject.get("scope").toString());
        return accessToken;
    }

    /**
     * 微信OAuth2.0授权-检验授权凭证（access_token）是否有效
     */
    @Override
    public synchronized boolean isAccessTokenValid(String accessToken, String openId) throws Exception {
        String isOAuthAccessToken = UseValue.isOAuthAccessToken
                .replace("ACCESS_TOKEN", accessToken)
                .replace("OPENID", openId);
        HttpGet get = HttpClientConnectionManager.getGetMethod(isOAuthAccessToken);
        HttpResponse response = httpclient.execute(get);
        String jsonStr = EntityUtils.toString(response.getEntity(), "utf-8");
        Map<String, Integer> jsonObject = new ObjectMapper()
                .configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true)
                .readValue(jsonStr, TreeMap.class);
        int returnNum = jsonObject.get("errcode");
        return returnNum == 0;
    }

    /**
     * 微信OAuth2.0授权-获取用户信息（网页授权作用域为snsapi_userinfo，则此时开发者可以通过access_token和openid拉取用户信息）
     */
    @Override
    public synchronized WxUser acceptOAuthUserNews(String accessToken, String openId) throws Exception {
        String getOAuthUserNews = UseValue.getOAuthUserNews.replace("ACCESS_TOKEN", accessToken).replace("OPEN_ID", openId);
        HttpGet get = SSLHttpClient.getGetMethod(getOAuthUserNews);
        HttpResponse response = SSLHttpClient.getSSLHttpClient().execute(get);
        String jsonStr = EntityUtils.toString(response.getEntity(), "utf-8");
        Map<String, Object> jsonObject = JSON.parseObject(jsonStr, new TypeReference<Map<String, Object>>() {
        });
        WxUser entity = new WxUser();

        entity.setOpenId(jsonObject.get("openid").toString());
        entity.setNickName(jsonObject.get("nickname").toString());
        entity.setSex(jsonObject.get("sex").toString());
        entity.setProvince(jsonObject.get("province").toString());
        entity.setCity(jsonObject.get("city").toString());
        entity.setCountry(jsonObject.get("country").toString());
        entity.setHeadimgurl(jsonObject.get("headimgurl").toString());

        return entity;
    }

    /**
     * 微信分享地址
     */
    public synchronized Map<String, Object> getSignature(String accessToken, String tempUrl) throws Exception {
        Map<String, Object> jsonObject = null;
        String signature = null;
        String jsapi_ticket = "";

        if (UseValue.jsapi_ticket == null || UseValue.jsapi_ticket.length() < 3) {
            String url = "https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token=" + accessToken + "&type=jsapi";
            HttpGet get = HttpClientConnectionManager.getGetMethod(url);
            HttpResponse response = httpclient.execute(get);
            String jsonStr = EntityUtils.toString(response.getEntity(), "utf-8");
            jsonObject = new ObjectMapper()
                    .configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true)
                    .readValue(jsonStr, TreeMap.class);
            if (jsonObject.get("ticket") != null) {
                UseValue.jsapi_ticket = jsonObject.get("ticket").toString();
                jsapi_ticket = UseValue.jsapi_ticket;
            }
        } else {
            jsapi_ticket = UseValue.jsapi_ticket;
        }

        Map<String, Object> endObj = null;
        String nonceStr = "qwertyuiopqwe";
        if (jsapi_ticket != null && jsapi_ticket.length() > 3) {
            endObj = new HashMap<>();
            String toUrl = tempUrl;
            Long timeS = (new Date()).getTime() / 1000;
            String sq = "jsapi_ticket=" + jsapi_ticket + "&noncestr=" + "qwertyuiopqwe" + "&timestamp=" + timeS + "&url=" + toUrl;
            MessageDigest md = null;
            md = MessageDigest.getInstance("SHA-1");
            //md.digest()方法必须作用于字节数组
            byte[] digest = md.digest(sq.getBytes());
            //将字节数组弄成字符串
            signature = byteToStr(digest);

            endObj.put("appId", UseValue.AppId);
            endObj.put("timestamp", timeS);
            endObj.put("noncestr", nonceStr);
            endObj.put("signature", signature.toLowerCase());
        }
        return endObj;
    }

//
//    /**
//     * 传入三个参数以及微信的token（静态自己设定）验证，
//     * @param signature 签名用来核实最后的结果是否一致
//     * @param timestamp 时间标记
//     * @param nonce 随机数字标记
//     * @return 一个布尔值确定最后加密得到的是否与signature一致
//     */
//    public static String checkSignature(String token,
//                                         String timestamp,String nonce){
//        //将传入参数变成一个String数组然后进行字典排序
//        String[] arr=new String[]{token,timestamp,nonce};
//        Arrays.sort(arr);
//        //创建一个对象储存排序后三个String的结合体
//        StringBuilder content=new StringBuilder();
//        for(int i=0;i<arr.length;i++){
//            content.append(arr[i]);
//        }
//
//
//        //启动sha1加密法的工具
//        MessageDigest md=null;
//        String tmpStr=null;
//        try {
//
//        } catch (NoSuchAlgorithmException e) {
//            //Auto-generated catch block
//            e.printStackTrace();
//        }
//        content=null;
//
////        return tmpStr!=null?tmpStr.equals(signature.toUpperCase()):false;
//     return tmpStr;
//    }


    /**
     * 将字节加工然后转化成字符串
     *
     * @param digest
     * @return
     */
    private static String byteToStr(byte[] digest) {
        String strDigest = "";
        for (int i = 0; i < digest.length; i++) {
            //将取得字符的二进制码转化为16进制码的的码数字符串
            strDigest += byteToHexStr(digest[i]);
        }
        return strDigest;
    }

    /**
     * 把每个字节加工成一个16位的字符串
     *
     * @param b
     * @return
     */
    public static String byteToHexStr(byte b) {
        //转位数参照表
        char[] Digit = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

        //位操作把2进制转化为16进制
        char[] tempArr = new char[2];
        //XXXX&1111那么得到的还是XXXX
        tempArr[0] = Digit[(b >>> 4) & 0X0F];
        //XXXX&1111那么得到的还是XXXX
        tempArr[1] = Digit[b & 0X0F];

        //得到进制码的字符串
        String s = new String(tempArr);
        return s;
    }


}
