package cn.wise.sc.pay.api.points.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.query.GoodExchangeQuery;
import cn.wise.sc.pay.domain.point.query.GoodRecordAddressQuery;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @description: 商品兑换记录服务接口
 * @author: qh
 * @create: 2020-04-28 15:08
 **/
@FeignClient(value = "saas-v1-points", path = "/goodsRecord", fallback = GoodRecordServiceImpl.class)
public interface IGoodRecordService {

    /**
     * 根据id获取用户兑换记录
     *
     * @param userId    用户id
     * @param pageQuery 分页对象
     * @return
     */
    @GetMapping("/page/{userId}")
    BaseResponse pageByUserId(@PathVariable("userId") Long userId,
                              @SpringQueryMap PageQuery pageQuery);

    /**
     * 设置地址
     *
     * @param goodRecordAddressQuery
     * @return
     */
    @PutMapping("/add/address")
    BaseResponse addAddressById(@RequestBody GoodRecordAddressQuery goodRecordAddressQuery);

    /**
     * 商品兑换
     *
     * @param goodExchangeQuery
     * @return
     */
    @PutMapping("/exchangeGoods")
    BaseResponse exchangeGoods(@RequestBody GoodExchangeQuery goodExchangeQuery);

    /**
     * 分页显示积分可兑换商品列表
     *
     * @param pageQuery
     * @return
     */
    @GetMapping("/exchangeRecord")
    BaseResponse pageExchange(@SpringQueryMap PageQuery pageQuery);

    /**
     * 获取最新的兑换记录
     *
     * @param size
     * @return
     */
    @GetMapping("/latestRecords/{size}")
    BaseResponse getLatestRecords(@PathVariable("size") Integer size);
}
