package cn.wise.sc.pay.api.points.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.MemberInterestsQuery;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @description: 会员权益服务接口
 * @author: qh
 * @create: 2020-01-08 10:20
 **/
@FeignClient(value = "saas-v1-points", path = "/member/Interests", fallback = MemberInterestsImpl.class)
public interface IMemberInterestsService {

    /**
     * 新增一个会员权益
     *
     * @param MSQuery 权益请求对象
     * @return
     */
    @PostMapping
    BaseResponse addNewInterests(@RequestBody MemberInterestsQuery MSQuery);

    /**
     * 编辑会员权益 id必传
     *
     * @param interestsQuery 权益请求对象
     * @return
     */
    @PutMapping
    BaseResponse editInterests(@RequestBody MemberInterestsQuery interestsQuery);

    /**
     * 根据id删除一个权益  id必传
     *
     * @param id 权益id
     * @return
     */
    @DeleteMapping("/{id}")
    BaseResponse delInterests(@PathVariable("id") Long id);

    /**
     * 分页查询 会员权益
     *
     * @param pageQuery 分页查询对象
     * @return
     */
    @PostMapping("/page")
    BaseResponse getPage(@RequestBody PageQuery pageQuery);

}
