package cn.wise.sc.pay.business.points.consumer;

import cn.wise.sc.pay.business.points.consumer.model.PointsOrderMsgModel;
import cn.wise.sc.pay.business.points.service.IPointDetailService;
import cn.wise.sc.pay.business.points.service.IPointSourceV1Service;
import cn.wise.sc.pay.business.points.service.IUserExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.model.TransactionMessage;
import cn.wise.sc.pay.domain.point.bean.PointSourceV1;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.enums.PointSourceEnum;
import cn.wise.sc.pay.domain.point.query.MemberScoreQuery;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * @description: 订单  增加相应积分和经验
 * @author: qh
 * @create: 2020-04-28 16:36
 **/
@Component
@Slf4j
public class OrderPointsExpListener {

    @Autowired
    IUserExtraService iUserExtraService;
    @Autowired
    IPointDetailService pointDetailService;
    @Autowired
    IPointSourceV1Service iPointSourceV1Service;

    @RabbitListener(queues = "pointAndExpQue")
    public void processMessage(String content) {

        TransactionMessage transactionMessage = JSON.parseObject(content, TransactionMessage.class);
        PointsOrderMsgModel model = JSON.parseObject(transactionMessage.getMessage(), PointsOrderMsgModel.class);

        //根据用户等级获取积分
        QueryWrapper<PointSourceV1> queryWrapper = new QueryWrapper<>();
        BigDecimal money = model.getPayPrice();
        queryWrapper.eq("way", PointSourceEnum.ONLINE.getCode());
        PointSourceV1 pointSource = iPointSourceV1Service.getOne(queryWrapper);
        BigDecimal points = model.getPoints();
        Long userId = model.getUserId();
        if (pointSource != null) {
            //统一积分设置
            if (pointSource.getSetType() == 0) {
                double conversion = pointSource.getUnionScore().doubleValue();
                points = money
                        .multiply(new BigDecimal(conversion))
                        .setScale(2, RoundingMode.HALF_DOWN);
            } else if (pointSource.getSetType() == 1) {
                String json = pointSource.getMemberScores();
                List<MemberScoreQuery> list = JSON.parseArray(json, MemberScoreQuery.class);
                //用户传过来不为空时
                QueryWrapper<UserExtra> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("user_id", userId);
                UserExtra userExtra = iUserExtraService.getOne(queryWrapper1);
                MemberScoreQuery memberScoreQuery = list.stream().filter(arg -> arg.getMemberShipId().longValue() == userExtra.getMembershipId())
                        .findFirst().orElse(null);
                //用户的会员等级 匹配积分获取比例
                points = money
                        .multiply(new BigDecimal(memberScoreQuery.getMemberScore()))
                        .setScale(2, RoundingMode.HALF_DOWN);
            }
            BaseResponse pointResponse = iUserExtraService.changePoints(model.getUserId(), points.doubleValue(), true);
            if (pointResponse.getCode() == 400) {
                log.error("processMessage>>pointAndExpQue添加积分错误: {}", pointResponse.getMsg());
            }
            BaseResponse expResponse = iUserExtraService.changeExperience(model.getUserId(), points.doubleValue(), true);
            if (expResponse.getCode() == 400) {
                log.error("processMessage>>pointAndExpQue添加经验值错误: {}", expResponse.getMsg());
            }
            //添加积分记录
            PointDetailQuery pointDetailQuery = PointDetailQuery.builder()
                    .userId(model.getUserId())
                    .score(points.doubleValue())
                    .inOrOut(1)
                    .pathWay(PointSourceEnum.ONLINE.getCode())
                    .build();
            BaseResponse pointDetailResponse = pointDetailService.insertPointDetailInfo(pointDetailQuery);
            if (pointDetailResponse.getCode() == 400) {
                log.error("processMessage>>pointAndExpQue添加积分记录错误: {}", pointDetailResponse.getMsg());
            }

        }
    }
}
