package cn.wise.sc.pay.business.points.controller;


import cn.wise.sc.pay.business.points.service.ICashPointRecordService;
import cn.wise.sc.pay.business.points.service.IMembershipService;
import cn.wise.sc.pay.business.points.service.IPointDetailService;
import cn.wise.sc.pay.business.points.service.IPointSourceV1Service;
import cn.wise.sc.pay.business.points.service.IUserExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.CashPointRecord;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.bean.PointDetail;
import cn.wise.sc.pay.domain.point.bean.PointSourceV1;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.enums.PointSourceEnum;
import cn.wise.sc.pay.domain.point.query.MemberScoreQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author qh
 * @since 2020-05-11
 */
@CrossOrigin
@RestController
@RequestMapping("/cashPointRecord")
public class  CashPointRecordController {

    final
    ICashPointRecordService iCashPointRecordService;
    final
    IUserExtraService iUserExtraService;
    final
    IPointDetailService iPointDetailService;
    final
    IPointSourceV1Service iPointSourceV1Service;
    final
    IMembershipService iMembershipService;

    public CashPointRecordController(ICashPointRecordService iCashPointRecordService, IUserExtraService iUserExtraService, IPointDetailService iPointDetailService, IPointSourceV1Service iPointSourceV1Service, IMembershipService iMembershipService) {
        this.iCashPointRecordService = iCashPointRecordService;
        this.iUserExtraService = iUserExtraService;
        this.iPointDetailService = iPointDetailService;
        this.iPointSourceV1Service = iPointSourceV1Service;
        this.iMembershipService = iMembershipService;
    }


    @PutMapping
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<String> usePoint(Long id, Long userId) {
        if (id == null || id == 0L) {
            return BaseResponse.error("id未必填项!", CashPointRecord.class.getName());
        }
        if (userId == null || userId == 0L) {
            return BaseResponse.error("用户id为必填!", CashPointRecord.class.getName());
        }

        CashPointRecord cashPointRecord = iCashPointRecordService.getById(id);
        BigDecimal money = cashPointRecord.getMoney();
        if (cashPointRecord == null || cashPointRecord.getStatus() == 1) {
            return BaseResponse.error("当前积分兑换码不存在/已使用!", CashPointRecord.class.getName());
        }
        cashPointRecord.setStatus(1);
        cashPointRecord.setUserId(userId);
        boolean isSuccess = iCashPointRecordService.updateById(cashPointRecord);
        BigDecimal point = cashPointRecord.getMoney();
        if (isSuccess) {
            QueryWrapper<PointSourceV1> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("way", PointSourceEnum.OFFLINE.getCode());
            PointSourceV1 pointSource = iPointSourceV1Service.getOne(queryWrapper);
            //设兑换规则 统一  和  按会员等级
            if (pointSource != null) {
                if (pointSource.getSetType() == 0) {
                    double conversion = pointSource.getUnionScore().doubleValue();
                    point = money
                            .multiply(new BigDecimal(conversion)
                                    .divide(new BigDecimal(100), 2, RoundingMode.HALF_DOWN))
                            .setScale(2, RoundingMode.HALF_DOWN);
                } else if (pointSource.getSetType() == 1) {
                    String json = pointSource.getMemberScores();
                    List<MemberScoreQuery> list = JSON.parseArray(json, MemberScoreQuery.class);
                    //如果用户未注册  返回青铜等级比例
                    if (userId == null || userId == 0L) {
                        cashPointRecord.setUserId(-1L);
                        BaseResponse<Membership> response = iMembershipService.getPrimaryMemberShip();
                        Membership minMembership = response.getData();
                        MemberScoreQuery minMemberScore = list.stream()
                                .filter(arg -> arg.getMemberShipId().longValue() == minMembership.getId())
                                .findFirst().orElse(null);
                        //最小会员等级与积分兑换规则匹配
                        if (minMemberScore != null) {
                            point = money
                                    .multiply(
                                            new BigDecimal(minMemberScore.getMemberScore())
                                                    .divide(new BigDecimal(100), 2, RoundingMode.HALF_DOWN))
                                    .setScale(2, RoundingMode.HALF_DOWN);
                        } else {
                            //最小会员等级和积分不匹配
                            list.sort((o1, o2) -> {
                                if (o1.getMemberScore() == o2.getMemberScore().doubleValue()) {
                                    return 1;
                                }
                                return o1.getMemberScore() - o2.getMemberScore();
                            });
                            double minScoreRatio = list.get(0).getMemberScore();
                            point = money.multiply(
                                    new BigDecimal(minScoreRatio)
                                            .divide(new BigDecimal(100), 2, RoundingMode.HALF_DOWN))
                                    .setScale(2, RoundingMode.HALF_DOWN);
                        }

                    } else {
                        //用户传过来不为空时
                        QueryWrapper<UserExtra> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.eq("user_id", userId);
                        UserExtra userExtra = iUserExtraService.getOne(queryWrapper1);
                        MemberScoreQuery memberScoreQuery = list.stream().filter(arg -> arg.getMemberShipId().longValue() == userExtra.getMembershipId())
                                .findFirst().orElse(null);
                        if (memberScoreQuery != null) {
                            //用户的会员等级 匹配积分获取比例
                            point = money
                                    .multiply(new BigDecimal(memberScoreQuery.getMemberScore())
                                            .divide(new BigDecimal(100), 2, RoundingMode.HALF_DOWN))
                                    .setScale(2, RoundingMode.HALF_DOWN);
                            cashPointRecord.setUserId(userId);
                        } else {
                            //未匹配到
                            return BaseResponse.error("用户会员等级无匹配规则!", CashPointRecord.class.getName());
                        }
                    }
                }
            }
            //使用现金  添加积分记录和增加用户积分/经验
            PointDetail pointDetail = PointDetail.builder()
                    .inOut(1)
                    .pathway(PointSourceEnum.OFFLINE.getCode())
                    .score(point.doubleValue())
                    .userId(cashPointRecord.getUserId())
                    .createTime(System.currentTimeMillis())
                    .build();
            iPointDetailService.save(pointDetail);
            iUserExtraService.changeExperience(cashPointRecord.getUserId(), point.doubleValue(), true);
            iUserExtraService.changePoints(cashPointRecord.getUserId(), point.doubleValue(), true);
            return BaseResponse.okMsg("兑换成功!", CashPointRecord.class);
        } else {
            return BaseResponse.error("兑换失败!", CashPointRecord.class.getName());
        }
    }

    @PostMapping
    public BaseResponse<CashPointRecord> getPoints(String orderNo, BigDecimal money, Long userId) {

        //没用设置线下消费兑换规则时  默认1:1兑换
        CashPointRecord cashPointRecord = CashPointRecord.builder()
                .orderNo(orderNo)
                .points(money + "")
                .money(money)
                .status(0)
                .build();

        QueryWrapper<PointSourceV1> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("way", PointSourceEnum.OFFLINE.getCode());
        PointSourceV1 pointSource = iPointSourceV1Service.getOne(queryWrapper);
        if (pointSource.getSetType() == 0) {
            double conversion = pointSource.getUnionScore().doubleValue();
            BigDecimal point = money
                    .multiply(new BigDecimal(conversion)
                            .divide(new BigDecimal(100), 2, RoundingMode.HALF_DOWN))
                    .setScale(2, RoundingMode.HALF_DOWN);
            cashPointRecord.setPoints(point.doubleValue() + "");
        } else if (pointSource.getSetType() == 1) {
            String json = pointSource.getMemberScores();
            List<MemberScoreQuery> memberQueries = JSON.parseArray(json, MemberScoreQuery.class);
            TreeSet<Integer> ratio = new TreeSet<>(new Comparator<Integer>() {
                @Override
                public int compare(Integer o1, Integer o2) {
                    if (o1.intValue() == o2) {
                        return 1;
                    }
                    return o1 - o2;
                }
            });
            memberQueries.stream().forEach(arg -> ratio.add(arg.getMemberScore()));
            double minRatio = ratio.first();
            double maxRatio = ratio.last();
            BigDecimal minPoint = money
                    .multiply(new BigDecimal(minRatio)
                            .divide(new BigDecimal(100), 2, RoundingMode.HALF_DOWN))
                    .setScale(2, RoundingMode.HALF_DOWN);
            BigDecimal maxPoint = money
                    .multiply(new BigDecimal(maxRatio)
                            .divide(new BigDecimal(100), 2, RoundingMode.HALF_DOWN))
                    .setScale(2, RoundingMode.HALF_DOWN);
            cashPointRecord.setPoints(minPoint + "~" + maxPoint);
        }
        boolean save = iCashPointRecordService.save(cashPointRecord);
        if (save) {
            return BaseResponse.ok("增加现金兑换成功!", cashPointRecord);
        } else {
            return BaseResponse.error("添加失败!", CashPointRecord.class.getName());
        }
    }
}

