package cn.wise.sc.pay.business.points.controller;


import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.domain.point.bean.PointPhoto;
import cn.wise.sc.pay.business.points.service.IMembershipService;
import cn.wise.sc.pay.business.points.service.IPointDetailService;
import cn.wise.sc.pay.business.points.service.IPointPhotoService;
import cn.wise.sc.pay.business.points.service.IPointSourceV1Service;
import cn.wise.sc.pay.common.core.model.BaseController;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.enums.PointSourceEnum;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author qh
 * @since 2020-08-03
 */
@Slf4j
@Validated
@CrossOrigin
@RestController
@RequestMapping("/pointPhoto")
public class PointPhotoController extends BaseController<IPointPhotoService, PointPhoto> {

    /**
     * 自动审核 缓存
     */
    private static boolean isAutoAudit = false;
    final
    IMembershipService iMembershipService;
    final
    IPointSourceV1Service iPointSourceV1Service;
    final
    IPointPhotoService iService;
    final
    IPointDetailService iPointDetailService;
    final
    FastDFSUtils fastDFSUtils;

    public PointPhotoController(IPointPhotoService iService,
                                IPointSourceV1Service iPointSourceV1Service,
                                IMembershipService iMembershipService,
                                IPointDetailService iPointDetailService,
                                FastDFSUtils fastDFSUtils) {
        super(iService);
        this.iService = iService;
        this.iPointSourceV1Service = iPointSourceV1Service;
        this.iMembershipService = iMembershipService;
        this.iPointDetailService = iPointDetailService;
        this.fastDFSUtils = fastDFSUtils;
    }

    @GetMapping("/setting")
    public BaseResponse<Boolean> getAutoAuditSetting() {
        return BaseResponse.ok(isAutoAudit);
    }

    @PutMapping("/setting")
    public BaseResponse<Boolean> editAutoAuditSetting() {
        isAutoAudit = !isAutoAudit;
        return BaseResponse.ok(isAutoAudit);
    }

    @Override
    @PostMapping
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse create(@RequestBody PointPhoto model) {

        //判断订单是否被使用过
        QueryWrapper<PointPhoto> qw = new QueryWrapper<>();
        qw.eq("status", 1);
        qw.eq("order_no", model.getOrderNo());
        int count = iService.count(qw);
        if (count > 0) {
            return BaseResponse.error("订单的拍照积分已被领取!", this.getClass().getName());
        }

        //判断是否为自动审核
        if (isAutoAudit) {
            model.setStatus(1);
            //添加用户积分
            addPoint(model);
        } else {
            model.setStatus(0);
        }

        try {
            iService.save(model);
            return BaseResponse.okMsg("添加成功", model.getClass());
        } catch (Exception ex) {
            log.error("查询操作失败 ===== " + iService.toString() + ".create()  ======  异常: {}", ex.getMessage());
            return BaseResponse.error("添加失败", model.getClass());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @PutMapping("/{id}/{status}")
    public BaseResponse<Boolean> audit(@NotNull(message = "id不能为空!") @PathVariable("id") Long id,
                                       @NotNull(message = "状态不能为空!") @PathVariable("status") Integer status) {
        PointPhoto updatePointPhoto = new PointPhoto();
        updatePointPhoto.setId(id);
        updatePointPhoto.setStatus(status);
        //审核通过
        if (status == 1) {
            PointPhoto pointPhoto = iService.getById(id);
            addPoint(pointPhoto);
        }
        iService.updateById(updatePointPhoto);
        return BaseResponse.ok(true);
    }

    @GetMapping("/page")
    public BaseResponse page(@NotNull(message = "分页参数不能为空!") PageQuery pageQuery,
                             Integer status) {
        QueryWrapper<PointPhoto> qw = new QueryWrapper<>();
        if (status != null) {
            qw.eq("status", status);
        }
        IPage<PointPhoto> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        qw.orderByDesc("id");
        iService.page(page, qw);
        //拼接图片路径
        List<PointPhoto> records = page.getRecords();
        records.forEach(arg -> {
            arg.setCover(fastDFSUtils.fullUrl(arg.getCover()));
        });
        return BaseResponse.ok(page);
    }

    private void addPoint(PointPhoto model) {
        //添加用户积分
        BaseResponse<Membership> membershipBaseResponse = iMembershipService.getShipByUserId(model.getUserId());
        Membership membership = membershipBaseResponse.getCode() == 200 ? membershipBaseResponse.getData() : null;
        if (membership == null) {
            log.error("获取积分信息失败！");
            throw new RuntimeException("拍照积分========获取积分信息失败!");
        }
        Double score = iPointSourceV1Service.getScore(membership.getId(), PointSourceEnum.PHOTO.getCode());
        PointDetailQuery pointDetailQuery = PointDetailQuery.builder()
                .inOrOut(1)
                .pathWay(PointSourceEnum.PHOTO.getCode())
                .userId(model.getUserId())
                .score(score)
                .build();
        iPointDetailService.addPointDetail(pointDetailQuery);
    }
}

