package cn.wise.sc.pay.business.points.controller;

import cn.wise.sc.pay.business.points.service.ICouponsService;
import cn.wise.sc.pay.business.points.service.IVirtualGoodsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.query.VirtualGoodQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 虚拟商品实控制器
 *
 * @author jiawei
 */
@RestController
@RequestMapping("/virtualGoods")
@CrossOrigin
@Api(tags = "虚拟商品")
public class VirtualGoodsController {

    @Autowired
    IVirtualGoodsService virtualGoodsService;
    @Autowired
    ICouponsService iCouponsService;

    @PostMapping
    @ApiOperation(value = "添加一条虚拟商品记录")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "virtualGoodQuery", value = "虚拟商品参数对象", dataType = "VirtualGoodQuery", paramType = "body")
            }
    )
    public BaseResponse add(@Valid @RequestBody VirtualGoodQuery virtualGoodQuery) {
        BaseResponse response = virtualGoodsService.add(virtualGoodQuery);
        return response;
    }

    @GetMapping("/page")
    @ApiOperation(value = "根据券名称与券类型id条件分页")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "name", value = "券名称", dataType = "String", paramType = "query"),
                    @ApiImplicitParam(name = "typeName", value = "券类型名称", dataType = "string", paramType = "query")
            }
    )
    public BaseResponse pageOnCondition(String name, PageQuery pageQuery, String typeName) {
        BaseResponse response = virtualGoodsService.pageOnCondition(name, pageQuery, typeName);
        return response;
    }

    @DeleteMapping("/{id}")
    @ApiOperation(value = "根据id删除")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "id", value = "id", dataType = "long", paramType = "path")
            }
    )
    public BaseResponse deleteById(@PathVariable Long id) {
        List<Long> ids = new ArrayList<>();
        ids.add(id);
        BaseResponse response = virtualGoodsService.discard(ids);
        return response;
    }

    @PutMapping
    @ApiOperation(value = "修改数据信息，传入数据时请传入id值")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(name = "virtualGoodQuery", value = "参数封装对象", dataType = "VirtualGoodQuery", paramType = "body")
            }
    )
    public BaseResponse edit(@RequestBody VirtualGoodQuery virtualGoodQuery) {
        BaseResponse response = virtualGoodsService.edit(virtualGoodQuery);
        return response;
    }

//
//    /**
//     * by qh
//     * 根据虚拟商品id获取虚拟商品
//     *
//     * @param id 必传
//     * @return
//     */
//    @GetMapping("/{id}")
//    public BaseResponse<VirtualGoods> get(@PathVariable("id") Long id) {
//
//        VirtualGoods virtualGoods = virtualGoodsService.getById(id);
//        VirtualGoodVo2 virtualGoodVo2 = new VirtualGoodVo2();
//        BeanUtil.copyProperties(virtualGoods, virtualGoodVo2);
//        if (virtualGoods.getCouponId() != null && virtualGoods.getCouponId() != 0L) {
//            Coupons coupons = iCouponsService.getById(virtualGoods.getCouponId());
//            if (coupons != null) {
//                virtualGoodVo2.setName(coupons.getName());
//            }
//
//        }
//    }
}
