package cn.wise.sc.pay.business.points.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.GoodRecord;
import cn.wise.sc.pay.domain.point.query.DeliveryInfoQuery;
import cn.wise.sc.pay.domain.point.query.GoodExchangeQuery;
import cn.wise.sc.pay.domain.point.query.GoodRecordAddressQuery;
import cn.wise.sc.pay.domain.point.query.GoodRecordQuery;
import com.baomidou.mybatisplus.extension.service.IService;

/**
 * 商品兑换记录业务接口层
 */
public interface IGoodRecordService extends IService<GoodRecord> {
    /**
     * 添加一条商品兑换记录
     *
     * @param goodRecordQuery
     * @return
     */
    BaseResponse add(GoodRecordQuery goodRecordQuery);

    /**
     * 添加一条发货信息
     *
     * @param deliveryInfoQuery
     * @return
     */
    BaseResponse addDeliveryInfo(DeliveryInfoQuery deliveryInfoQuery);

    /**
     * 虚拟商品条件分页
     *
     * @param name
     * @param pageQuery
     * @param status
     * @return
     */
    BaseResponse pageOnVirtualGoods(String name, PageQuery pageQuery, Integer status);

    /**
     * 实物商品条件分页
     *
     * @param name
     * @param pageQuery
     * @param status
     * @return
     */
    BaseResponse pageOnGiftGoods(String name, PageQuery pageQuery, Integer status);

    /**
     * 根据id获取记录
     *
     * @param id
     * @return
     */
    BaseResponse getRecord(Long id);

    /**
     * 分页显示积分兑换商品列表
     */
    BaseResponse pageExchange(PageQuery pageQuery);

    /**
     * @param id
     * @param flag
     * @return
     */
    BaseResponse increasePersonNum(Long id, Integer flag);

    /**
     * 商品兑换
     *
     * @param goodExchangeQuery
     * @return
     */
    BaseResponse exchangeGoods(GoodExchangeQuery goodExchangeQuery);

    /**
     * 根据用户id分页显示兑换记录
     *
     * @param userId
     * @return
     */
    BaseResponse pageByUserId(Long userId, PageQuery pageQuery);

    /**
     * 获取最新的兑换记录
     *
     * @param size
     * @return
     */
    BaseResponse getLatestRecords(Integer size);

    /**
     * 添加实物商品兑换记录的收货地址和收货人
     * @param goodRecordAddressQuery
     * @return
     */
    BaseResponse addAddressById(GoodRecordAddressQuery goodRecordAddressQuery);

}
