package cn.wise.sc.pay.business.points.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.mapper.CouponsMapper;
import cn.wise.sc.pay.business.points.service.ICouponTypeService;
import cn.wise.sc.pay.business.points.service.ICouponsService;
import cn.wise.sc.pay.business.points.service.IVirtualGoodsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.CouponType;
import cn.wise.sc.pay.domain.point.bean.Coupons;
import cn.wise.sc.pay.domain.point.query.CouponsQuery;
import cn.wise.sc.pay.domain.point.vo.CouponInfoVos;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 优惠券实体业务实现类
 *
 * @author jiawei
 */
@Service
public class CouponsServiceImpl extends ServiceImpl<CouponsMapper, Coupons> implements ICouponsService {

    @Autowired
    CouponsMapper couponsMapper;
    @Autowired
    ICouponTypeService couponTypeService;
    @Autowired
    IVirtualGoodsService iVirtualGoodsService;
    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    /**
     * 添加一条优惠券记录
     */
    public BaseResponse add(CouponsQuery couponsQuery) {
        if (couponsQuery.getTypeId() == null || couponsQuery.getTypeId() <= 0) {
            return BaseResponse.error("请传入正确的优惠券类型ID", CouponsServiceImpl.class.getName());
        }
        if (couponsQuery.getStatus() == null) {
            couponsQuery.setStatus(0);
        }
        Coupons coupons = CouponsQuery.query2Entity(couponsQuery);

        coupons.setCreateTime(System.currentTimeMillis());

        QueryWrapper qw = new QueryWrapper();
        qw.eq("name", coupons.getName());
        qw.eq("type_id", coupons.getTypeId());
        qw.eq("cost", coupons.getCost());
        qw.eq("able_value", coupons.getAbleValue());
        qw.eq("del_flag", 0);
        int count = this.count(qw);
        if (count > 0) {
            return BaseResponse.error("已存在!", this.getClass().getName());
        }

        boolean save = this.save(coupons);
        return save ? BaseResponse.ok(coupons) : BaseResponse.error("添加数据失败", CouponsServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据类型名称条件分页
     */
    public BaseResponse pageOnCondition(String name, PageQuery pageQuery, Integer typeId) {
        //分页对象
        Page<Coupons> couponsPage = new Page<>();
        couponsPage.setCurrent(pageQuery.getPageNo());
        couponsPage.setSize(pageQuery.getPageSize());
        //条件
        QueryWrapper<Coupons> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("create_time");
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like("name", name);
        }
        if (typeId != null) {
            queryWrapper.eq("type_id", typeId);
        }
        IPage<Coupons> page = this.page(couponsPage, queryWrapper);
        //优惠券数据  集合
        List<Coupons> records = page.getRecords();
        if (records == null || records.size() == 0) {
            return BaseResponse.ok(page, this.getClass());
        }
        //类型ids
        Set<Integer> typeIds = records.stream().map(re -> re.getTypeId()).collect(Collectors.toSet());
        //优惠券类型数据  集合
        Collection<CouponType> couponTypes = couponTypeService.listByIds(typeIds);

        List<CouponInfoVos> couponInfoVos = new ArrayList<>();
        records.stream().forEach(re -> {
            CouponInfoVos couponInfoVo = new CouponInfoVos();

            BeanUtil.copyProperties(re, couponInfoVo);
            couponTypes.forEach(ct -> {
                if (re.getTypeId().equals(ct.getId()) && re.getTypeId() != null && ct.getId() != null) {
                    couponInfoVo.setTypeName(ct.getName());
                    //拼接背景图片
                    couponInfoVo.setImage(fastDFSUtils.fullUrl(ct.getImage()));
                }
            });
            couponInfoVos.add(couponInfoVo);
        });
        Page<CouponInfoVos> vosPage = new Page<>();
        //设值返回分页对象
        vosPage.setRecords(couponInfoVos);
        vosPage.setSize(page.getSize());
        vosPage.setCurrent(page.getCurrent());
        vosPage.setTotal(page.getTotal());
        vosPage.setPages(page.getPages());
        return BaseResponse.ok(vosPage, this.getClass());
    }


    @Override
    public BaseResponse listCoupons() {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("status", 1);
        qw.eq("del_flag",0);
        List<Coupons> list = this.list(qw);
        if (CollectionUtils.isEmpty(list)) {
            return BaseResponse.ok((List<Coupons>) null, this.getClass());
        }
        return BaseResponse.ok(list);
    }

    @Override
    public BaseResponse listAvailable() {
        QueryWrapper<Coupons> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", 0);
        List<Coupons> list = this.list(queryWrapper);
        if (CollectionUtils.isEmpty(list)) {
            return BaseResponse.ok((List<Coupons>) null, this.getClass());
        }
        BaseResponse<List<Coupons>> baseResponse = BaseResponse.ok(list);
        return baseResponse;
    }

    @Override
    public BaseResponse discard(List<Integer> ids) {
        if (ids == null || ids.size() < 1) {
            return BaseResponse.error("请传入id值", CouponsServiceImpl.class.getName());
        }

        //下架所有关联的虚拟商品
        QueryWrapper qw = new QueryWrapper();
        qw.in("coupon_id", ids);
        qw.eq("del_flag", 0);
        int count = iVirtualGoodsService.count(qw);
        if (count > 0) {
            return BaseResponse.error("当前优惠已有虚拟商品在使用!", CouponsServiceImpl.class.getName());
        }

        boolean remove = this.removeByIds(ids);
        return remove ? BaseResponse.ok() : BaseResponse.error("删除失败", CouponsServiceImpl.class.getName());
    }

    @Override
    public BaseResponse edit(CouponsQuery couponsQuery) {
        if (couponsQuery.getId() == null) {
            return BaseResponse.error("请传入id值", CouponsServiceImpl.class.getName());
        }
        Coupons coupons = CouponsQuery.query2Entity(couponsQuery);
        coupons.setUpdateTime(System.currentTimeMillis());
        boolean update = this.updateById(coupons);
        return update ? BaseResponse.ok(coupons) : BaseResponse.error("更新信息失败", CouponsServiceImpl.class.getName());
    }

    @Override
    public BaseResponse infoById(Long id) {
        if (id == null) {
            return BaseResponse.error("请传入id值", CouponsServiceImpl.class.getName());
        }
        Coupons info = this.getById(id);
        return BaseResponse.ok(info, CouponsServiceImpl.class);
    }

    @Override
    /**
     * 根据id启用记录
     */
    public BaseResponse enableById(Long id) {
        if (id == null) {
            return BaseResponse.error("请传入id值", CouponsServiceImpl.class.getName());
        }
        Coupons coupon = Coupons.builder()
                .id(id)
                .status(1)
                .build();
        boolean update = this.updateById(coupon);
        return update ? BaseResponse.ok() : BaseResponse.error("启用失败", CouponsServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据id禁用记录
     */
    public BaseResponse disableById(Long id) {

        if (id == null) {
            return BaseResponse.error("请传入id值", CouponsServiceImpl.class.getName());
        }

        //判断优惠券是否被使用
        QueryWrapper qw = new QueryWrapper();
        qw.eq("coupon_id", id);
        int count = iVirtualGoodsService.count(qw);
        if (count > 0) {
            return BaseResponse.error("优惠券使用中!",this.getClass().getName());
        }

        Coupons coupon = Coupons.builder()
                .id(id)
                .status(0)
                .build();
        boolean update = this.updateById(coupon);
        return update ? BaseResponse.ok() : BaseResponse.error("禁用失败", CouponsServiceImpl.class.getName());
    }
}

