package cn.wise.sc.pay.business.points.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.mapper.MemberInterestsMapper;
import cn.wise.sc.pay.business.points.service.IMemberInterestsService;
import cn.wise.sc.pay.business.points.service.IMembershipService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.InterestsMembership;
import cn.wise.sc.pay.domain.point.bean.MemberInterests;
import cn.wise.sc.pay.domain.point.bean.MemberInterestsQuery;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.vo.MemberInterestsVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: 会员权益业务实现类
 * @author: qh
 * @create: 2020-01-06 16:37
 **/
@Service
public class MemberInterestsServiceImpl extends ServiceImpl<MemberInterestsMapper, MemberInterests> implements IMemberInterestsService {

    @Autowired
    MemberInterestsMapper memberInterestsMapper;
    @Autowired
    IMembershipService membershipService;
    @Autowired
    FastDFSUtils fastDFSUtils;

    @Override
    @Transactional
    public BaseResponse addNewInterests(MemberInterestsQuery memberInterestsQuery) {

        if (StringUtils.isEmpty(memberInterestsQuery.getName())) {
            return BaseResponse.error("请传入参数值", this.getClass().getName());
        }

        if (memberInterestsQuery.getMembership() == null || memberInterestsQuery.getMembership().length < 1) {
            return BaseResponse.error("请传入参数值", this.getClass().getName());
        }

        QueryWrapper qw = new QueryWrapper();
        qw.eq("name", memberInterestsQuery.getName());

        int count = this.count(qw);
        if (count > 0) {
            return BaseResponse.error("会员权益已存在,请检测!", MemberInterests.class);
        }

        MemberInterests memberInterests = new MemberInterests();
        BeanUtil.copyProperties(memberInterestsQuery, memberInterests);

        try {
            this.save(memberInterests);
            List<InterestsMembership> insertBatch = new ArrayList<>();
            for (int i = 0; i < memberInterestsQuery.getMembership().length; i++) {
                InterestsMembership membership = new InterestsMembership();
                membership.setInterestsId(memberInterests.getId());
                membership.setMembershipId(memberInterestsQuery.getMembership()[i]);
                insertBatch.add(membership);
            }
            memberInterestsMapper.insertBatch(insertBatch);

            return BaseResponse.okMsg("添加成功", MemberInterests.class);
        } catch (Exception ex) {
            return BaseResponse.error("添加失败", MemberInterests.class);
        }
    }

    @Override
    public BaseResponse delInterests(Long id) {

        if (id == null) {
            return BaseResponse.error("参数错误", MemberInterests.class);
        }
        try {
            this.removeById(id);
            return BaseResponse.okMsg("删除成功", MemberInterests.class);

        } catch (Exception ex) {
            return BaseResponse.error("删除失败", MemberInterests.class);
        }
    }

    @Override
    @Transactional
    public BaseResponse editInterests(MemberInterestsQuery memberInterestsQuery) {

        MemberInterests memberInterests = new MemberInterests();
        BeanUtil.copyProperties(memberInterestsQuery, memberInterests);

        try {
            if (memberInterestsQuery.getMembership().length != 0) {
                memberInterestsMapper.delInterestsMembershipByInId(memberInterests.getId());
            }
            this.updateById(memberInterests);
            List<InterestsMembership> insertBatch = new ArrayList<>();
            for (int i = 0; i < memberInterestsQuery.getMembership().length; i++) {
                InterestsMembership membership = new InterestsMembership();
                membership.setInterestsId(memberInterests.getId());
                membership.setMembershipId(memberInterestsQuery.getMembership()[i]);
                insertBatch.add(membership);
            }
            memberInterestsMapper.insertBatch(insertBatch);
            return BaseResponse.okMsg("更新成功", MemberInterests.class);
        } catch (Exception ex) {
            return BaseResponse.error("更新失败", MemberInterests.class);
        }
    }

    @Override
    public BaseResponse getInterests(Long membershipId) {

        if (membershipId == null) {
            return BaseResponse.error("会员等级id不能为空", membershipId);
        }
        try {
            List<MemberInterests> list = memberInterestsMapper.getInterests(membershipId);
            return BaseResponse.ok(list, MemberInterests.class);
        } catch (Exception ex) {
            return BaseResponse.error("查找失败", MemberInterests.class);
        }
    }

    @Override
    public BaseResponse getPage(PageQuery pageQuery) {

        IPage page = new Page(pageQuery.getPageNo(), pageQuery.getPageSize());

        QueryWrapper qw = new QueryWrapper();
        qw.orderByDesc("id");

        try {
            page = this.page(page, qw);
            List<MemberInterestsVO> list = bean2VO(page.getRecords());
            //图片背景拼接
            if (!CollectionUtils.isEmpty(list)) {
                list.stream().forEach(ls -> ls.setCover(fastDFSUtils.fullUrl(ls.getCover())));
            }
            page.setRecords(list);
            BaseResponse baseResponse = BaseResponse.ok(new MemberInterests());
            baseResponse.setPage((Page) page);

            return baseResponse;
        } catch (Exception ex) {
            return BaseResponse.error("查询失败", MemberInterests.class);
        }
    }

    /**
     * 会员权益关联会员等级
     *
     * @param memberInterests
     * @return
     */
    private List<MemberInterestsVO> bean2VO(List<MemberInterests> memberInterests) {
        //获取所有会员等级
        List<Membership> memberships = membershipService.list();
        //获取所有权益等级关联
        List<InterestsMembership> interestsMemberships = memberInterestsMapper.getInterestsMembership();
        List<MemberInterestsVO> list = new ArrayList<>(memberInterests.size());
        //将权益和会员等级名称关联
        memberInterests.stream().forEach(arg -> {
            MemberInterestsVO memberInterestsVO = new MemberInterestsVO();
            BeanUtil.copyProperties(arg, memberInterestsVO);
            //获取当前权益关联的所有会员等级id
            List<Long> membershipIds = interestsMemberships.stream()
                    .filter(im -> im.getInterestsId() == arg.getId())
                    .map(im -> im.getMembershipId())
                    .collect(Collectors.toList());
            //不为空是循环添加会员等级
            if (membershipIds != null && membershipIds.size() != 0) {
                memberships.stream()
                        .filter(ship -> membershipIds.contains(ship.getId()))
                        .forEach(ship -> memberInterestsVO.setMembership(memberInterestsVO.getMembership() + ship.getName() + ","));

                memberInterestsVO.setMembership(memberInterestsVO.getMembership().substring(0, (memberInterestsVO.getMembership().length() - 1) > 0 ? (memberInterestsVO.getMembership().length() - 1) : 0));
            } else {
                memberInterestsVO.setMembership("");
            }
            list.add(memberInterestsVO);
        });
        return list;
    }
}
