package cn.wise.sc.pay.business.points.service.impl;

import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.mapper.MembershipMapper;
import cn.wise.sc.pay.business.points.service.IMemberInterestsService;
import cn.wise.sc.pay.business.points.service.IMembershipService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.point.bean.MemberInterests;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.query.MemberQuery;
import cn.wise.sc.pay.domain.point.vo.MembershipVO;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-11-11
 */
@Service
@SuppressWarnings("all")
public class MembershipServiceImpl extends ServiceImpl<MembershipMapper, Membership> implements IMembershipService {

    @Autowired
    MembershipMapper membershipMapper;
    @Autowired
    IMemberInterestsService memberInterestsService;
    @Autowired
    FastDFSUtils fastDFSUtils;

    /**
     * 判断该等级下是否拥有用户
     *
     * @param id
     * @return
     */
    private boolean hasUsers(int id) {

        int count = membershipMapper.memberBlUsrCount(id);
//        int count = membershipMapper.numberCount(id+"");
        return count > 0 ? true : false;
    }

    /**
     * 判断是否有相同的经验值
     *
     * @param exp
     * @return
     */
    private boolean hasSameExp(Double exp) {

        int count = membershipMapper.hasSameExp(exp);

        return count > 0 ? true : false;
    }

    @Override
    public BaseResponse<Page<MembershipVO>> list(PageWrapperQuery pageWrapperQuery) {

        Page<Membership> page = new Page<>();

        page.setCurrent(pageWrapperQuery.getPageQuery().getPageNo());
        page.setSize(pageWrapperQuery.getPageQuery().getPageSize());
        page.setAsc("create_time");

        QueryWrapper ew = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);
        ew.eq("del_flag", 0);
        ew.orderByDesc("id");

        page = (Page<Membership>) this.page(page, ew);
        List<MembershipVO> membershipVOS = Lists.newArrayListWithCapacity(page.getRecords().size());

        for (Membership membership : page.getRecords()) {
            MembershipVO membershipVo = new MembershipVO(membership);
            //拼接会员背景图片
            membershipVo.setBackPhoto(fastDFSUtils.fullUrl(membership.getBackPhoto()));
            membershipVOS.add(membershipVo);
        }

        Page<MembershipVO> pageVo = new Page<>(pageWrapperQuery.getPageQuery().getPageNo(), pageWrapperQuery.getPageQuery().getPageSize());
        pageVo.setSize(page.getSize());
        pageVo.setCurrent(page.getCurrent());
//        pageVo.setTotal(page.getRecords().size());
        pageVo.setTotal(page.getTotal());
        pageVo.setRecords(membershipVOS);

        return BaseResponse.ok(pageVo);
    }

    @Override
    public BaseResponse getMembexrship(int id) {

        if (id == 0) {
            return BaseResponse.error("参数错误", Membership.class);
        }
        Membership membership = this.getById(id);
        //拼接图片
        if (membership != null) {
            membership.setBackPhoto(fastDFSUtils.fullUrl(membership.getBackPhoto()));
        }

        return BaseResponse.ok(new MembershipVO(membership));
    }

    @Override
    public BaseResponse forbiddenOrActive(int id, int status) {
        if (id == 0) {
            return BaseResponse.error("参数信息错误", Membership.class);
        } else {
            try {

                Membership membership = Membership.builder().id(Integer.toUnsignedLong(id)).status(status).build();
                this.updateById(membership);
                return BaseResponse.ok("用户状态更新成功!");
            } catch (Exception ex) {

                return BaseResponse.error("用户状态更新失败!", Membership.class);
            }
        }
    }

    @Override
    public BaseResponse editMembership(MemberQuery memberQuery) {

        Membership membershipUpdate = new Membership();

        membershipUpdate.setId(memberQuery.getId());
        membershipUpdate.setDownExperience(memberQuery.getExperience());
        membershipUpdate.setDiscount(memberQuery.getDiscount());
        membershipUpdate.setName(memberQuery.getName());
        membershipUpdate.setUpExperience(memberQuery.getUpExperience());
        membershipUpdate.setBackPhoto(memberQuery.getBackPhoto());
        membershipUpdate.setStatus(memberQuery.getStatus());
        membershipUpdate.setUpdateTime(System.currentTimeMillis());

        boolean succesFlag = this.updateById(membershipUpdate);
        if (succesFlag) {
            return BaseResponse.ok("更新成功");
        } else {
            return BaseResponse.error("更新失败", Membership.class);
        }
    }

    @Override
    public BaseResponse deleteMembership(int id) {
        if (id == 0) {
            return BaseResponse.error("参数信息错误", Membership.class);
        } else {

            boolean isMemberBelongUsers = this.hasUsers(id);
            if (isMemberBelongUsers) {
                return BaseResponse.error("该等级下已有会员!", Membership.class);
            }

            try {
                Membership membership = Membership.builder().id(Integer.toUnsignedLong(id)).delFlag(1).build();
                this.updateById(membership);
                return BaseResponse.ok("删除用户成功!");

            } catch (Exception ex) {
                return BaseResponse.error("删除用户失败!", Membership.class);
            }
        }
    }

    @Override
    public BaseResponse newMembership(MembershipVO membershipVo) {

        boolean isSameExp = this.hasSameExp(membershipVo.getExperience());
        if (isSameExp) {
            return BaseResponse.error("升级" + membershipVo.getName() + "员需要相同等级积分，请重新设置升级积分!", Membership.class);
        } else {
            Membership membership = Membership.builder()
                    .discount(membershipVo.getDiscount() / 100)
                    .downExperience(membershipVo.getExperience())
                    .name(membershipVo.getName())
                    .status(membershipVo.getStatus())
                    .backPhoto(membershipVo.getBackPhoto())
                    .createTime(System.currentTimeMillis())
                    .upExperience(membershipVo.getUpExperience())
                    .build();
            boolean save = this.save(membership);
            return save ? BaseResponse.ok("保存成功") : BaseResponse.error("保存失败！", this.getClass().getName());

        }
    }

    @Override
    public BaseResponse<Membership> getShipByUserId(long userId) {

        if (userId < 1) {
            return BaseResponse.error("参数有误", Membership.class.getName());
        }
        try {

            Membership membership = membershipMapper.getShipByUserId(userId);
            return BaseResponse.ok(membership);
        } catch (Exception ex) {

            return BaseResponse.error("获取失败", Membership.class.getName());
        }
    }

    @Override
    public Membership getNextMemberShipByMemberId(Long memberId) {

        if (memberId == null) {
            return null;
        }

        Membership member = this.getById(memberId);
        QueryWrapper qw = new QueryWrapper();
        qw.gt("down_experience", member.getDownExperience());
        qw.orderByAsc("down_experience");
        qw.last("limit 1");
        try {

            List<Membership> list = this.list(qw);
            Membership membership = list.get(0);
            return membership;
        } catch (Exception ex) {
            return null;
        }

    }

    @Override
    public BaseResponse<List<Membership>> getListByIds(List<Long> ids) {
        if (ids == null || ids.size() < 1) {
            return BaseResponse.error("请传入参数", MembershipServiceImpl.class.getName());
        }
        QueryWrapper wrapper = new QueryWrapper();
        Collection<Membership> memberships = this.listByIds(ids);
        ArrayList<Membership> membershipArrayList = new ArrayList<>();
        membershipArrayList.addAll(memberships);
        //拼接会员图片
        if (!CollectionUtils.isEmpty(membershipArrayList)) {
            membershipArrayList.stream().forEach(ms -> ms.setBackPhoto(fastDFSUtils.fullUrl(ms.getBackPhoto())));
        }
        return BaseResponse.ok(membershipArrayList);
    }

    @Override
    public BaseResponse<UserExtraVOApp> getMemberInfo(Long memberId) {
        if (memberId == null) {
            return BaseResponse.error("请传入会员id值", this.getClass().getName());
        }
        Membership membership = this.getById(memberId);
        BaseResponse interests = memberInterestsService.getInterests(membership.getId());
        List<MemberInterests> memberInterests = interests.getList();
//        UserExtraVOApp userExtraVOApp = new UserExtraVOApp();
//        BeanUtil.copyProperties(membership, userExtraVOApp);
        UserExtraVOApp userExtraVOApp = UserExtraVOApp.builder()
//                .experience(membership.get)
                .membershipId(membership.getId())
                .memberShipName(membership.getName())
                .memberInterests(memberInterests)
                //拼接图片
                .backPhoto(fastDFSUtils.fullUrl(membership.getBackPhoto()))
                .downExperience(membership.getDownExperience())
                .upExperience(membership.getUpExperience())
                .build();
        userExtraVOApp.setMemberInterests(memberInterests);
        return BaseResponse.ok(userExtraVOApp);
    }

    @Override
    public BaseResponse<Membership> getPrimaryMemberShip() {
        QueryWrapper<Membership> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByAsc("down_experience");
        queryWrapper.last("LIMIT 1");
        queryWrapper.eq("del_flag", 0);
        queryWrapper.eq("status", 1);
        Membership membership = this.getOne(queryWrapper);
//拼接会员图片
        if (membership != null) {
            membership.setBackPhoto(fastDFSUtils.fullUrl(membership.getBackPhoto()));
        }
        return BaseResponse.ok(membership, this.getClass());
    }
}
