package cn.wise.sc.pay.business.points.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.points.mapper.SignInTimeMapper;
import cn.wise.sc.pay.business.points.service.IPointDetailService;
import cn.wise.sc.pay.business.points.service.IPointSourceV1Service;
import cn.wise.sc.pay.business.points.service.ISignInTimeService;
import cn.wise.sc.pay.business.points.service.IUserExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.CumulativeScoreRule;
import cn.wise.sc.pay.domain.point.bean.PointDetail;
import cn.wise.sc.pay.domain.point.bean.PointSourceV1;
import cn.wise.sc.pay.domain.point.bean.SignInTime;
import cn.wise.sc.pay.domain.point.query.SignInTimesQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author qh
 * @since 2020-03-12
 */
@Service
@Slf4j
public class SignInTimeServiceImpl extends ServiceImpl<SignInTimeMapper, SignInTime> implements ISignInTimeService {

    @Autowired
    IPointDetailService iPointDetailService;
    @Autowired
    IPointSourceV1Service iPointSourceV1Service;
    @Autowired
    IUserExtraService iUserExtraService;

    @Override
    @Transactional
    public BaseResponse updateOrAdd(SignInTimesQuery signInTimeQuery) {

        if (signInTimeQuery.getUserId() == null || signInTimeQuery.getUserId() == 0) {
            return BaseResponse.error("用户信息为空!", new SignInTime());
        }
        if (signInTimeQuery.getTime() == null || signInTimeQuery.getTime() == 0) {
            return BaseResponse.error("签到日期为空!", new SignInTime());
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("user_id", signInTimeQuery.getUserId());
        SignInTime signInTime;
        try {
            signInTime = this.getOne(qw);
        } catch (Exception e) {
            log.error("签到记录查询异常 " +
                    "===== cn.wise.sc.pay.business.points.service.impl.SignInTimeServiceImpl.updateOrAdd() " +
                    "===== 异常:{}", e.getMessage());
            return BaseResponse.error("更新签到记录失败!", new SignInTime());
        }

        //如果为空添加一条记录
        if (signInTime == null) {
            try {
                signInTime = new SignInTime();
                signInTime.setUserId(signInTimeQuery.getUserId());
                signInTime.setContinuous(1);
                Set<String> signTimes = new LinkedHashSet<>();
                signTimes.add(dateLong2MonthDayStr(signInTimeQuery.getTime()));
                signInTime.setSignTimes(JSON.toJSONString(signTimes));
                this.save(signInTime);
            } catch (Exception e) {
                log.error("签到记录添加异常 " +
                        "===== cn.wise.sc.pay.business.points.service.impl.SignInTimeServiceImpl.updateOrAdd() " +
                        "===== 异常:{}", e.getMessage());
                return BaseResponse.error("更新签到记录失败!", new SignInTime());
            }
        } else { //不为空时 更新记录
            try {
                SignInTime signInTimeUpdate = new SignInTime();
                LinkedHashSet<String> signTimes = JSON.parseObject(signInTime.getSignTimes(), LinkedHashSet.class);
                if (signTimeIsMothStart(signTimes, signInTimeQuery.getTime())) {
                    signTimes = new LinkedHashSet<>();
                }
                signTimes.add(dateLong2MonthDayStr(signInTimeQuery.getTime()));
                signInTimeUpdate.setSignTimes(JSON.toJSONString(signTimes));
                signInTimeUpdate.setContinuous(signInTimeQuery.getContinuous());
                this.update(signInTimeUpdate, qw);
            } catch (Exception e) {
                log.error("签到记录添加异常 " +
                        "===== cn.wise.sc.pay.business.points.service.impl.SignInTimeServiceImpl.updateOrAdd() " +
                        "===== 异常:{}", e.getMessage());
                return BaseResponse.error("更新签到记录失败!", new SignInTime());
            }
        }
        addSignPoints(signInTimeQuery.getUserId(), signInTimeQuery.getContinuous());
        return BaseResponse.ok();
    }

    /**
     * 新增对应积分
     *
     * @param userId     用户id
     * @param continuous 连续签到天数
     */
    private void addSignPoints(long userId, int continuous) {

        //获取签到积分规则
        QueryWrapper qw = new QueryWrapper();
        qw.eq("way", "签到奖励");
        qw.eq("status", 0);
        PointSourceV1 pointSourceV1 = iPointSourceV1Service.getOne(qw);
        if (pointSourceV1 == null) {
            log.error("积分签到规则未设置!");
            throw new RuntimeException("积分签到规则没有设置!");
        }
        //能得到的积分数
        BigDecimal getPoints;
        //统一设置
        if (pointSourceV1.getSetType() == 2 && pointSourceV1.getNonCumulativeScore() != null) {
            PointDetail unifyPointDetail = PointDetail.builder()
                    .createTime(System.currentTimeMillis())
                    .inOut(1)
                    .score(Double.valueOf(pointSourceV1.getNonCumulativeScore()))
                    .pathway("签到奖励")
                    .userId(userId)
                    .build();
            getPoints = new BigDecimal(pointSourceV1.getNonCumulativeScore());
            iPointDetailService.save(unifyPointDetail);
            iUserExtraService.changePoints(userId, getPoints.doubleValue(), true);
            iUserExtraService.changeExperience(userId, getPoints.doubleValue(), true);
        } else if (pointSourceV1.getSetType() == 3 && StrUtil.isNotEmpty(pointSourceV1.getCumulativeScores())) {
            //非统一设置  及连续累积制
            String cumulativeScoreRuleStr = pointSourceV1.getCumulativeScores();
            List<CumulativeScoreRule> cumulativeScoreRules = JSON.parseArray(cumulativeScoreRuleStr, CumulativeScoreRule.class);
            //基础分数 CumulativeFlag = 0
            CumulativeScoreRule basicsCumulativeScoreRule = cumulativeScoreRules
                    .stream()
                    .filter(point -> "0".equals(point.getCumulativeFlag()))
                    .findFirst().orElse(null);
            //累积分数 CumulativeFlag = 1
            CumulativeScoreRule continuousCumulativeScoreRule = cumulativeScoreRules
                    .stream()
                    .filter(point -> "1".equals(point.getCumulativeFlag()))
                    .findFirst().orElse(null);
            if (basicsCumulativeScoreRule == null || continuousCumulativeScoreRule == null) {
                log.error("累积签到规则没有设置!");
                throw new RuntimeException("累积签到规则没有设置!");
            }
            //应得分数 = 基础+连续*累积分数
            getPoints = continuousCumulativeScoreRule.getCumulativeScore()
                    .multiply(new BigDecimal((continuous - 1) < 0 ? 0 : continuous - 1)).add(basicsCumulativeScoreRule.getCumulativeScore());

            PointDetail cumulativePointDetail = PointDetail.builder()
                    .userId(userId)
                    .score(getPoints.doubleValue())
                    .createTime(System.currentTimeMillis())
                    .pathway("签到奖励")
                    .inOut(1)
                    .build();
            iPointDetailService.save(cumulativePointDetail);
            iUserExtraService.changePoints(userId, getPoints.doubleValue(), true);
            iUserExtraService.changeExperience(userId, getPoints.doubleValue(), true);
        } else {
            log.error("积分签到规则设置有误!");
            throw new RuntimeException("积分签到规则没有设置!");
        }
    }

    /**
     * 判断签到时间是否为这月的第一天
     *
     * @param signTimes 签到时间记录
     * @param now       今天签到时间
     * @return
     */
    private boolean signTimeIsMothStart(LinkedHashSet<String> signTimes, Long now) {
        if (signTimes == null || signTimes.size() == 0) {
            return true;
        }
        Iterator<String> it = signTimes.iterator();
        String lastTime = "";
        while (it.hasNext()) {
            lastTime = it.next();
        }
        //判断签到最后一天月份和今天是否相等
        String[] str = lastTime.split("\\.");
        String lastSignMonth = str[1];
        if (StrUtil.isNotEmpty(lastSignMonth)) {
            Calendar calendar = new Calendar.Builder()
                    .setInstant(now)
                    .build();
            int nowMonth = (calendar.get(Calendar.MONTH) + 1);
            //相等不为新的一个月
            if (nowMonth == Integer.parseInt(lastSignMonth)) {
                return false;
            }
        }
        return true;
    }


    /**
     * 日期long转年月日
     *
     * @param date
     * @return
     */
    private String dateLong2MonthDayStr(Long date) {

        Calendar calendar = new Calendar.Builder()
                .setInstant(date)
                .build();
        int month = calendar.get(Calendar.MONDAY) + 1;
        int day = calendar.get(Calendar.DATE);
        int year = calendar.get(Calendar.YEAR);
        String result = year + "";
        if (month < 10) {
            result = result + ".0" + month;
        } else {
            result = result + "." + month;
        }
        if (day < 10) {
            result = result + ".0" + day;
        } else {
            result = result + "." + day;
        }
        return result;
    }

}
