package cn.wise.sc.pay.business.points.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.mapper.UserExtraMapper;
import cn.wise.sc.pay.business.points.service.IMemberInterestsService;
import cn.wise.sc.pay.business.points.service.IMembershipService;
import cn.wise.sc.pay.business.points.service.IPointDetailService;
import cn.wise.sc.pay.business.points.service.IUserExtraService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.point.bean.MemberInterests;
import cn.wise.sc.pay.domain.point.bean.Membership;
import cn.wise.sc.pay.domain.point.bean.UserExtra;
import cn.wise.sc.pay.domain.point.cnst.PointTypeConsts;
import cn.wise.sc.pay.domain.point.query.PointDetailQuery;
import cn.wise.sc.pay.domain.point.query.UserExreaQuery;
import cn.wise.sc.pay.domain.point.vo.UserExtraVOApp;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.TreeSet;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @description: 用户额外属性实现类
 * @author: qh
 * @create: 2020-01-06 11:07
 */
@Service
public class UserExtraServiceImpl extends ServiceImpl<UserExtraMapper, UserExtra>
    implements IUserExtraService {

  @Autowired UserExtraMapper userExtraMapper;
  @Autowired IMemberInterestsService iMemberInterestsService;
  @Autowired IMembershipService iMembershipService;
  @Autowired FastDFSUtils fastDFSUtils;
  @Autowired IPointDetailService pointDetailService;

  @Override
  public BaseResponse editUserExtra(UserExreaQuery userExreaQueryer) {
    // 用户积分等级修改；前端页面并不会传membershipId,所以需要通过当前参数经验获取membershipId
    Double experience = userExreaQueryer.getExperience();
    if (experience != null && userExreaQueryer.getMembershipId() == null) {
      // 查询所有会员等级  排序
      // 判断当前经验在的区间
      QueryWrapper<Membership> wrapper = new QueryWrapper();
      wrapper.eq("status", "1");
      wrapper.eq("del_flag", 0);
      List<Membership> memberships = iMembershipService.list(wrapper);
      TreeSet<Membership> sortByExpMemberships =
          new TreeSet<>(
              (o1, o2) ->
                  o1.getDownExperience().doubleValue() > o2.getDownExperience().doubleValue()
                      ? 1
                      : -1);
      sortByExpMemberships.addAll(memberships);

      //获取临界的会员积分
      AtomicReference<Membership> nextMember = new AtomicReference<>();
      sortByExpMemberships.stream()
          .forEach(
              arg -> {
                if (arg.getDownExperience() < experience) {
                  nextMember.set(arg);
                }
              });
      if (nextMember.get() != null) {
        userExreaQueryer.setMembershipId(nextMember.get().getId());
      }
    }
    // 根据userID获取用户变动的积分
    double changePoint = 0d;
    if (userExreaQueryer.getUserId() != null
        && userExreaQueryer.getUserId() > 0
        && userExreaQueryer.getPoints() != null
        && userExreaQueryer.getType() != 1) {
      QueryWrapper<UserExtra> userExtraQueryWrapper = new QueryWrapper<>();
      userExtraQueryWrapper.eq("user_id", userExreaQueryer.getUserId());
      UserExtra userExtra = this.getOne(userExtraQueryWrapper);
      changePoint = userExreaQueryer.getPoints() - userExtra.getPoints();
    }
    UpdateWrapper<UserExtra> uw = new UpdateWrapper<>();
    UserExtra userExtra = new UserExtra();
    BeanUtil.copyProperties(userExreaQueryer, userExtra);
    uw.eq("user_id", userExreaQueryer.getUserId());
    try {
      boolean update = this.update(userExtra, uw);
      // 修改积分后，添加积分详情记录
      BaseResponse response = null;
      if (changePoint != 0d && userExreaQueryer.getType() != 1) {
        PointDetailQuery userExtraQuery =
            PointDetailQuery.builder()
                .pathWay(PointTypeConsts.ADMIN_SET)
                .inOrOut(changePoint > 0d ? 1 : 0)
                .userId(userExreaQueryer.getUserId())
                .score(Math.abs(changePoint))
                .build();
        response = pointDetailService.insertPointDetailInfo(userExtraQuery);
      }
      if (changePoint != 0d && userExreaQueryer.getType() != 1) {
        if (response != null && response.getCode() == 200 && update) {
          return BaseResponse.okMsg("更新成功", UserExtra.class);
        }
        return BaseResponse.okMsg("更新成功", UserExtra.class);
      }
      return update
          ? BaseResponse.okMsg("更新成功", UserExtra.class)
          : BaseResponse.error("更新失败", UserExtra.class);
    } catch (Exception ex) {
      return BaseResponse.error("更新失败", UserExtra.class);
    }
  }

  @Override
  public BaseResponse getList(List<Long> userIds) {
    if (userIds == null || userIds.size() == 0) {
      return BaseResponse.error("查询失败", UserExtra.class.getName());
    }

    try {
      List<UserExtra> list = userExtraMapper.getList(userIds);

      return BaseResponse.ok(list, UserExtra.class);
    } catch (Exception ex) {
      return BaseResponse.error("查询失败", UserExtra.class.getName());
    }
  }

  @Override
  public BaseResponse delUserExtra(Long userId) {

    if (userId == null) {
      return BaseResponse.error("参数错误", UserExtra.class.getName());
    }
    QueryWrapper qw = new QueryWrapper();
    qw.eq("user_id", userId);
    this.remove(qw);

    return BaseResponse.ok(null);
  }

  @Override
  public BaseResponse changePoints(Long userId, double point, boolean isIncrease) {

    if (userId == null || point == 0) {
      return BaseResponse.error("参数错误", UserExtra.class.getName());
    }

    UpdateWrapper uw = new UpdateWrapper();
    uw.eq("user_id", userId);
    if (isIncrease) {
      uw.setSql("points = points+" + point);
    } else {
      uw.setSql("points = points-" + point);
    }
    try {
      this.update(uw);
      return BaseResponse.okMsg("更新成功", UserExtra.class);
    } catch (Exception ex) {
      return BaseResponse.error("更新失败", UserExtra.class.getName());
    }
  }

  @Override
  public BaseResponse changeExperience(Long userId, double experience, boolean isIncrease) {
    if (userId == null || experience == 0) {
      return BaseResponse.error("参数错误", UserExtra.class.getName());
    }

    UpdateWrapper uw = new UpdateWrapper();
    uw.eq("user_id", userId);
    if (isIncrease) {
      uw.setSql("experience = experience+" + experience);
    } else {
      uw.setSql("experience = experience-" + experience);
    }
    try {
      this.update(uw);
      return BaseResponse.okMsg("更新成功", UserExtra.class);
    } catch (Exception ex) {
      return BaseResponse.error("更新失败", UserExtra.class.getName());
    }
  }

  @Override
  public BaseResponse<UserExtraVOApp> getUserExtra(Long userId, Integer flag) {

    QueryWrapper qw = new QueryWrapper();
    qw.eq("user_id", userId);

    UserExtra userExtra = this.getOne(qw);

    if (userExtra == null) {
      return BaseResponse.okMsg("没找到相关数据", UserExtra.class);
    }
    // 会员权益
    BaseResponse baseResponse = iMemberInterestsService.getInterests(userExtra.getMembershipId());
    if (baseResponse.getCode() == 200) {
      // 会员权益
      List<MemberInterests> list = baseResponse.getList();
      // 下一等级的会员记录
      Membership nextMembership =
          iMembershipService.getNextMemberShipByMemberId(userExtra.getMembershipId());
      // 当前会员的等级记录
      Membership membership = iMembershipService.getById(userExtra.getMembershipId());

      if (membership != null) {
        UserExtraVOApp userExtraVOApp = null;
        if (nextMembership != null) {
          userExtraVOApp =
              UserExtraVOApp.builder()
                  .experience(userExtra.getExperience())
                  .points(userExtra.getPoints())
                  .discount(membership.getDiscount())
                  .nextDiscount(nextMembership.getDiscount())
                  .nextPoints(nextMembership.getDownExperience())
                  // 拼接图片
                  .backPhoto(fastDFSUtils.fullUrl(nextMembership.getBackPhoto()))
                  //                        .memberShipName(userExtra.getMemberShipName())
                  .memberShipName(nextMembership.getName())
                  .memberInterests(list)
                  .userId(userExtra.getUserId())
                  .membershipId(userExtra.getMembershipId())
                  .upExperience(nextMembership.getUpExperience())
                  .downExperience(nextMembership.getDownExperience())
                  .build();
        }
        if (flag == 0) {
          userExtraVOApp.setBackPhoto(fastDFSUtils.fullUrl(membership.getBackPhoto()));
          userExtraVOApp.setMemberShipName(membership.getName());
          userExtraVOApp.setUpExperience(membership.getUpExperience());
          userExtraVOApp.setDownExperience(membership.getDownExperience());
        }
        return BaseResponse.ok(userExtraVOApp);
      } else {
        //                return BaseResponse.error("没找到相关会员等级", UserExtra.class);
        return BaseResponse.error("没找到相关会员等级", UserExtra.class.getName());
      }
    } else {
      //            return BaseResponse.error("没找到相关权益", UserExtra.class);
      return BaseResponse.error("没找到相关权益", UserExtra.class.getName());
    }
  }

  @Override
  public BaseResponse addNewUserExtra(UserExreaQuery userExreaQuery) {

    UserExtra userExtra = new UserExtra();
    BeanUtil.copyProperties(userExreaQuery, userExtra);
    try {
      this.save(userExtra);
      return BaseResponse.okMsg("添加成功", UserExtra.class);
    } catch (Exception ex) {
      return BaseResponse.error("添加失败", UserExtra.class);
    }
  }

  @Override
  public BaseResponse<List<UserExtra>> getByMemberId(Long membershipId) {

    if (membershipId == null) {
      return BaseResponse.error("请传入参数值", UserExtra.class.getName());
    }
    QueryWrapper<UserExtra> wrapper = new QueryWrapper<>();
    wrapper.eq("membership_id", membershipId);
    List<UserExtra> list = this.list(wrapper);

    return BaseResponse.ok(list);
  }

  @Override
  public BaseResponse<List<UserExtra>> getListByUserId(List<Long> userIds) {
    if (userIds == null || userIds.size() < 1) {
      return BaseResponse.error("请传入参数", UserExtraServiceImpl.class.getName());
    }
    QueryWrapper<UserExtra> queryWrapper = new QueryWrapper<>();
    queryWrapper.in("user_id", userIds);
    List<UserExtra> list = this.list(queryWrapper);
    return BaseResponse.ok(list);
  }

  @Override
  public UserExtra getUserExtraByUserId(Long userId) {
    QueryWrapper<UserExtra> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("user_id", userId);
    UserExtra one = this.getOne(queryWrapper);
    return one;
  }
}
