package cn.wise.sc.pay.business.sms.service.impl;

import cn.wise.sc.pay.business.sms.service.SmsService;
import cn.wise.sc.pay.business.sms.utils.AliYunSmsUtil;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import com.aliyuncs.exceptions.ClientException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.concurrent.TimeUnit;

/**
 * 短信发送实现类
 */
@Service
public class SmsServiceImpl implements SmsService {
    // redis 值存在时间
    private long expire = 60;
    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public BaseResponse sendSms(String phone)   {
        if (StringUtils.isEmpty(phone)) {
            return BaseResponse.error("手机号码不能为空", this.getClass().getName());
        }
        //随机生产验证码
//        int verificationCode = (int)Math.random()*10000;
        int newCode = (int)((Math.random()*9+1)*100000);
        String verificationCode = String.valueOf(newCode);
//        redisTemplate.opsForValue().set(phone, verificationCode);
        //设置缓存值并设置有效期
        redisTemplate.opsForValue().set(phone, verificationCode, expire, TimeUnit.SECONDS);
        BaseResponse baseResponse = null;
        try {
            baseResponse = AliYunSmsUtil.sendSms(phone, verificationCode);
//            IhuyiSmsUtil.sendSms(phone,verificationCode);
        } catch (ClientException e) {
            baseResponse = BaseResponse.error(e.getMessage(), this.getClass().getName());
        }
        return baseResponse;
    }

    @Override
    public BaseResponse checkSms(String telephone, String verificationCode) {
        if (StringUtils.isEmpty(telephone)){
            return BaseResponse.error("电话号码不能为空", this.getClass().getName());
        }
        if (StringUtils.isEmpty(verificationCode)){
            return BaseResponse.error("验证码不能为空", this.getClass().getName());
        }
        String value = (String)redisTemplate.opsForValue().get(telephone);
        if (value == null) {
            return BaseResponse.error("验证码无效，请重新尝试！", this.getClass().getName());
        }
        if (value.equals(verificationCode)){
            return  BaseResponse.ok("验证成功");
        }
        return BaseResponse.error("验证码不一致", this.getClass().getName());
    }
}
