package cn.wise.sc.pay.task.timer.order.handler;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.api.order.service.IOrderCallExService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.domain.order.model.OrderCallException;
import cn.wise.sc.pay.domain.order.model.OrderCallExceptionStatus;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import com.alibaba.fastjson.JSON;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 确认支付后没有收到回调处理
 * @author: qh
 * @create: 2020-05-06 10:42
 */
@Component
public class OrderConfirmNoNoticeHandler {

  final IOrdersService iOrdersService;
  @Autowired
  IOrderCallExService iOrderCallExService;

  public OrderConfirmNoNoticeHandler(IOrdersService iOrdersService) {
    this.iOrdersService = iOrdersService;
  }

  /** 对支付确认 但没有收到回调的订单进行支付异常处理 */
  @XxlJob("OrderConfirmNoNoticeHandler")
  public ReturnT<String> execute(String s) {

    // 查询订单支付状态为确认支付和最后更新时间超过一分钟的订单
    PageWrapperQuery wrapperQuery = PageWrapperQueryUtil.create();
    Map<String, Object> eq = new HashMap<>(1);
    eq.put("order_status", OrderStatus.CONFIRM.name());
    wrapperQuery.setEq(eq);
    Map<String, Object> lt = new HashMap<>(1);
    lt.put("update_time", System.currentTimeMillis() - 60 * 1000);
    wrapperQuery.setLt(lt);
    BaseResponse response = iOrdersService.getOrders(wrapperQuery);

    if (response.getCode() == 400) {
      return ReturnT.FAIL;
    }
    // 获取订单集合添加异常记录
    List<Orders> list = response.getList();

    List<OrderCallException> insertList = new ArrayList<>(list.size());
    for (Orders orders : list) {
      if (StrUtil.isNotEmpty(orders.getUnifiedOrderNo())){
        OrderCallException callException = new OrderCallException();
        callException.setOrderNo(orders.getUnifiedOrderNo());
        callException.setFailCount(0);
        callException.setCreateTime(System.currentTimeMillis());
        callException.setUpdateTime(System.currentTimeMillis());
        callException.setOrderStatus(OrderCallExceptionStatus.PAY_EXCEPTION.name());
        callException.setSellerId(orders.getSellerId());
        insertList.add(callException);
      }
    }
    BaseResponse exceptionInfoBatch = iOrderCallExService.createExceptionInfoBatch(insertList);
    if (exceptionInfoBatch.getCode() == 400) {
      return ReturnT.FAIL;
    }
    return ReturnT.SUCCESS;
  }

  public static void main(String[] args) {
    OrderCallException orderCallException = new OrderCallException();
    orderCallException.setRefundNo("623734454329539984");
    orderCallException.setOrderNo("97569475868920567");
    orderCallException.setOrderStatus(OrderStatus.REFUND.name());
    orderCallException.setFailCount(0);
    System.out.println(JSON.toJSONString(orderCallException));
  }
}
