package cn.wise.sc.pay.task.timer.order.handler;

import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.order.service.IXpenseTrackerService;
import cn.wise.sc.pay.business.goods.model.bean.Goods;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.exhibition.bean.Exhibition;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.XpenseTracker;
import com.alibaba.fastjson.JSON;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class XpenseTrackerHandler{

    final
    IOrdersService iOrdersService;
    final
    IXpenseTrackerService xpenseTrackerService;

    public XpenseTrackerHandler(IOrdersService iOrdersService, IXpenseTrackerService xpenseTrackerService) {
        this.iOrdersService = iOrdersService;
        this.xpenseTrackerService = xpenseTrackerService;
    }

    @XxlJob("XpenseTrackerHandler")
    public ReturnT<String> execute(String s) throws Exception {
        long start = System.currentTimeMillis();
        long end  = start + 10000;

        PageWrapperQuery wrapperQuery = new PageWrapperQuery();
        Map ge = new HashMap();
        ge.put("create_time",start);
        wrapperQuery.setGe(ge);
        Map lt = new HashMap();
        lt.put("update_time",end);
        wrapperQuery.setLt(lt);
        OrderResponse response = iOrdersService.getOrders(wrapperQuery);
        if (response.getCode() == 400){
            return ReturnT.FAIL;
        }
        List<Orders> list = response.getList();
        List<XpenseTracker> xpenseTrackers = new ArrayList<>();
        for (Orders o :
                list) {
            xpenseTrackers.addAll( getXpense(o));
        }
        xpenseTrackerService.createBatch(xpenseTrackers);

        return ReturnT.SUCCESS;
    }

    private  List<XpenseTracker> getXpense(Orders order){
        List<XpenseTracker> xpenseTrackers = new ArrayList<>();
        if ("EXHIBITION".equals(order.getPaas())){
            List<Exhibition> list = JSON.parseArray(order.getGoods(),Exhibition.class);
            for (Exhibition ex:list
                 ) {
                XpenseTracker xpenseTracker = XpenseTracker.builder()
                        .name(ex.getTitle())
                        .price(ex.getPrice())
                        .userId(order.getUserId().longValue())
                        .createTime(order.getUpdateTime())
                        .build();
                xpenseTrackers.add(xpenseTracker);
            }
        }
        if ("CASHIER".equals(order.getPaas())){
            List<Goods> list = JSON.parseArray(order.getGoods(),Goods.class);
            for (Goods ex:list
            ) {
                XpenseTracker xpenseTracker = XpenseTracker.builder()
                        .name(order.getPaymentMethodName() +ex.getName())
                        .userId(order.getUserId().longValue())
                        .createTime(order.getUpdateTime())
                        .build();
                xpenseTrackers.add(xpenseTracker);
            }
        }
        return xpenseTrackers;
    }
}
